Package: ssmodels
Title: Sample Selection Models
Version: 2.0.2
Language: en-US
Authors@R: c(
  person( given = "Fernando", family = "de Souza Bastos", role = c( "aut", "cre" ),
          email = "fernando.bastos@ufv.br" ),
  person( given = "Wagner", family = "Barreto de Souza", role = "aut" ))
Author: Fernando de Souza Bastos [aut, cre],
  Wagner Barreto de Souza [aut]
Maintainer: Fernando de Souza Bastos <fernando.bastos@ufv.br>
Depends: R (>= 3.6.0)
Imports: MASS, sn (>= 2.1.0), miscTools (>= 0.6-26), Rdpack (>= 2.4)
Suggests: knitr (>= 1.24), testthat (>= 3.0.0), numDeriv (>=
        2016.8-1.1), maxLik (>= 1.3-6), mvtnorm (>= 1.0-11),
        sampleSelection (>= 1.2-6), kableExtra (>= 1.1.0), kfigr (>=
        1.2), ggplot2 (>= 3.2.1), gridExtra (>= 2.3)
Description: In order to facilitate the adjustment of the sample selection models
  existing in the literature, we created the 'ssmodels' package. Our package
  allows the adjustment of the classic Heckman model (Heckman (1976),
  Heckman (1979) <doi:10.2307/1912352>), and the estimation of the parameters of
  this model via the maximum likelihood method and two-step method, in addition
  to the adjustment of the Heckman-t models introduced in the literature by
  Marchenko and Genton (2012) <doi:10.1080/01621459.2012.656011> and the
  Heckman-Skew model introduced in the literature by Ogundimu and Hutton (2016)
  <doi:10.1111/sjos.12171>. We also implemented functions to adjust the
  generalized version of the Heckman model, introduced by Bastos, Barreto-Souza,
  and Genton (2021) <doi:10.5705/ss.202021.0068>, that allows the inclusion of
  covariables to the dispersion and correlation parameters, and a function to
  adjust the Heckman-BS model introduced by Bastos and Barreto-Souza (2020)
  <doi:10.1080/02664763.2020.1780570> that uses the Birnbaum-Saunders
  distribution as a joint distribution of the selection and primary regression
  variables. This package extends and complements existing R packages such as 
  'sampleSelection' (Toomet and Henningsen, 2008) and 'ssmrob' (Zhelonkin et al., 2016), providing additional robust and flexible sample selection models.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RdMacros: Rdpack
BugReports: https://github.com/fsbmat-ufv/ssmodels/issues
Config/testthat/edition: 3
URL: https://fsbmat-ufv.github.io/ssmodels/
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-30 04:06:43 UTC; Fernando
Repository: CRAN
Date/Publication: 2026-05-30 04:20:02 UTC
Built: R 4.5.2; ; 2026-05-30 04:56:18 UTC; unix

Package: ssmodels
Title: Sample Selection Models
Version: 2.0.2
Language: en-US
Authors@R: c(
  person( given = "Fernando", family = "de Souza Bastos", role = c( "aut", "cre" ),
          email = "fernando.bastos@ufv.br" ),
  person( given = "Wagner", family = "Barreto de Souza", role = "aut" ))
Author: Fernando de Souza Bastos [aut, cre],
  Wagner Barreto de Souza [aut]
Maintainer: Fernando de Souza Bastos <fernando.bastos@ufv.br>
Depends: R (>= 3.6.0)
Imports: MASS, sn (>= 2.1.0), miscTools (>= 0.6-26), Rdpack (>= 2.4)
Suggests: knitr (>= 1.24), testthat (>= 3.0.0), numDeriv (>=
        2016.8-1.1), maxLik (>= 1.3-6), mvtnorm (>= 1.0-11),
        sampleSelection (>= 1.2-6), kableExtra (>= 1.1.0), kfigr (>=
        1.2), ggplot2 (>= 3.2.1), gridExtra (>= 2.3)
Description: In order to facilitate the adjustment of the sample selection models
  existing in the literature, we created the 'ssmodels' package. Our package
  allows the adjustment of the classic Heckman model (Heckman (1976),
  Heckman (1979) <doi:10.2307/1912352>), and the estimation of the parameters of
  this model via the maximum likelihood method and two-step method, in addition
  to the adjustment of the Heckman-t models introduced in the literature by
  Marchenko and Genton (2012) <doi:10.1080/01621459.2012.656011> and the
  Heckman-Skew model introduced in the literature by Ogundimu and Hutton (2016)
  <doi:10.1111/sjos.12171>. We also implemented functions to adjust the
  generalized version of the Heckman model, introduced by Bastos, Barreto-Souza,
  and Genton (2021) <doi:10.5705/ss.202021.0068>, that allows the inclusion of
  covariables to the dispersion and correlation parameters, and a function to
  adjust the Heckman-BS model introduced by Bastos and Barreto-Souza (2020)
  <doi:10.1080/02664763.2020.1780570> that uses the Birnbaum-Saunders
  distribution as a joint distribution of the selection and primary regression
  variables. This package extends and complements existing R packages such as 
  'sampleSelection' (Toomet and Henningsen, 2008) and 'ssmrob' (Zhelonkin et al., 2016), providing additional robust and flexible sample selection models.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RdMacros: Rdpack
BugReports: https://github.com/fsbmat-ufv/ssmodels/issues
Config/testthat/edition: 3
URL: https://fsbmat-ufv.github.io/ssmodels/
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-30 04:06:43 UTC; Fernando
Repository: CRAN
Date/Publication: 2026-05-30 04:20:02 UTC
Built: R 4.6.0; ; 2026-05-30 05:38:02 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(summary,HeckmanBS)
S3method(summary,HeckmanCL)
S3method(summary,HeckmanGe)
S3method(summary,HeckmanSK)
S3method(summary,HeckmantS)
export(HCinitial)
export(HeckmanBS)
export(HeckmanCL)
export(HeckmanGe)
export(HeckmanSK)
export(HeckmantS)
export(IMR)
export(extract_model_components)
export(postprocess_theta)
export(step2)
export(summary.HeckmanBS)
export(summary.HeckmanCL)
export(summary.HeckmanGe)
export(summary.HeckmanSK)
export(summary.HeckmantS)
export(twostep)
importFrom(Rdpack,reprompt)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,terms)
importFrom(utils,tail)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ssmodels <a href='https://fsbmat-ufv.github.io/ssmodels/'><img src='man/figures/logo.png' align="right" height="80" style="margin:10px;" /></a>

# ssmodels

<!-- badges: start -->
<!-- badges: end -->
<p style="text-align: justify;">
In order to facilitate the adjustment of the sample selection models
existing in the literature, we created the ssmodels package. Our package
allows the adjustment of the classic Heckman model (Heckman (1976),
Heckman (1979)), and the estimation of the parameters of this model via
the maximum likelihood method and two-step method, in addition to the
adjustment of the Heckman-t models, introduced in the literature by
Marchenko and Genton (2012) and the Heckman-Skew model introduced in the
literature by Ogundimu and Hutton (2016). We also implemented functions
to adjust the generalized version of the Heckman model, introduced by
Bastos, Barreto-Souza, and Genton (2021), that allows the inclusion of
covariables to the dispersion and correlation parameters and a function
to adjust the Heckman-BS model introduced by Bastos and Barreto-Souza
(2020) that uses the Birnbaum-Saunders distribution as a joint
distribution of the selection and primary regression variables.
</p>

## Installation

You can install the released version of ssmodels from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("ssmodels")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("fsbmat-ufv/ssmodels")
```

## Code of Conduct

`{ssmodels}` is released with a [Contributor Code of
Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html).

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-bastos" class="csl-entry">

Bastos, Fernando de Souza, and Wagner Barreto-Souza. 2020.
“Birnbaum–Saunders Sample Selection Model.” *Journal of Applied
Statistics*.

</div>

<div id="ref-bastosBarreto" class="csl-entry">

Bastos, Fernando de Souza, Wagner Barreto-Souza, and Marc G Genton.
2021. “A Generalized Heckman Model with Varying Sample Selection Bias
and Dispersion Parameters.” *Statistica Sinica*.

</div>

<div id="ref-heckman1976common" class="csl-entry">

Heckman, James J. 1976. “The Common Structure of Statistical Models of
Truncation, Sample Selection and Limited Dependent Variables and a
Simple Estimator for Such Models.” In *Annals of Economic and Social
Measurement, Volume 5, Number 4*, 475–92. NBER.

</div>

<div id="ref-heckman1979sample" class="csl-entry">

———. 1979. “Sample Selection Bias as a Specification Error.”
*Econometrica: Journal of the Econometric Society*, 153–61.

</div>

<div id="ref-marchenko2012heckman" class="csl-entry">

Marchenko, Yulia V, and Marc G Genton. 2012. “A Heckman Selection-t
Model.” *Journal of the American Statistical Association* 107 (497):
304–17.

</div>

<div id="ref-ogundimu2016sample" class="csl-entry">

Ogundimu, Emmanuel O, and Jane L Hutton. 2016. “A Sample Selection Model
with Skew-Normal Distribution.” *Scandinavian Journal of Statistics* 43
(1): 172–90.

</div>

</div>

# ============================================================
# Validacao do gradiente do modelo Heckman classico via numDeriv
# Banco: MEPS2001
# Pacote: ssmodels
# ============================================================

rm(list = ls())

pkgs <- c("pkgload", "numDeriv", "dplyr", "tibble", "readr", "sampleSelection")
for (p in pkgs) {
  if (!requireNamespace(p, quietly = TRUE)) install.packages(p)
}

suppressPackageStartupMessages({
  library(dplyr)
  library(tibble)
})

# ------------------------------------------------------------
# 1. Carregar pacote local ou instalado
# ------------------------------------------------------------
find_pkg_root <- function(path = getwd()) {
  cur <- normalizePath(path, winslash = "/", mustWork = TRUE)
  for (i in seq_len(8)) {
    if (file.exists(file.path(cur, "DESCRIPTION")) && dir.exists(file.path(cur, "R"))) {
      return(cur)
    }
    parent <- dirname(cur)
    if (identical(parent, cur)) break
    cur <- parent
  }
  NA_character_
}

pkg_root <- find_pkg_root()
if (!is.na(pkg_root)) {
  pkgload::load_all(pkg_root, reset = TRUE, quiet = TRUE)
} else {
  library(ssmodels)
}

# ------------------------------------------------------------
# 2. Carregar banco MEPS2001
# ------------------------------------------------------------
ok_data <- tryCatch({
  data("MEPS2001", package = "ssmodels", envir = environment())
  exists("MEPS2001", inherits = FALSE)
}, error = function(e) FALSE)

if (!ok_data && !is.na(pkg_root)) {
  candidate_data <- file.path(pkg_root, "data", "MEPS2001.rda")
  if (file.exists(candidate_data)) load(candidate_data)
}
if (!exists("MEPS2001")) stop("Nao foi possivel localizar o banco MEPS2001.")

dados <- as.data.frame(MEPS2001)

# ------------------------------------------------------------
# 3. Formulas de validacao
# ------------------------------------------------------------
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins

# ------------------------------------------------------------
# 4. Carregar funcoes independentes
# ------------------------------------------------------------
local_candidates <- c(
  if (!is.na(pkg_root)) file.path(pkg_root, "inst", "validation", "HeckmanCL_loglik_score_independente.R"),
  "HeckmanCL_loglik_score_independente.R",
  "inst/validation/HeckmanCL_loglik_score_independente.R",
  "../inst/validation/HeckmanCL_loglik_score_independente.R",
  "../HeckmanCL_loglik_score_independente.R"
)
source_file <- local_candidates[file.exists(local_candidates)][1]
if (is.na(source_file)) stop("Arquivo HeckmanCL_loglik_score_independente.R nao encontrado.")
source(source_file)

components_cl <- heckmanCL_prepare(selectEq, outcomeEq, dados)

# ------------------------------------------------------------
# 5. Ajustar HeckmanCL()
# ------------------------------------------------------------
fit_cl <- ssmodels::HeckmanCL(
  selection = selectEq,
  outcome   = outcomeEq,
  data      = dados
)

theta_hat <- fit_cl$coefficients
param_names <- c(
  paste0("gamma:", colnames(components_cl$XS)),
  paste0("beta:", colnames(components_cl$XO)),
  "sigma",
  "rho"
)
names(theta_hat) <- param_names

ll_fit <- fit_cl$loglik
ll_ind <- heckmanCL_loglik_independente(theta_hat, selectEq, outcomeEq, dados, components_cl)

cat("\nLogLik HeckmanCL():          ", ll_fit, "\n")
cat("LogLik funcao independente: ", ll_ind, "\n\n")

# ------------------------------------------------------------
# 6. Comparacao opcional com sampleSelection
# ------------------------------------------------------------
fit_sample_ml <- tryCatch(
  sampleSelection::selection(
    selection = selectEq,
    outcome   = outcomeEq,
    data      = dados,
    method    = "ml"
  ),
  error = function(e) NULL
)

if (!is.null(fit_sample_ml)) {
  ll_sample <- as.numeric(stats::logLik(fit_sample_ml))
  cat("LogLik sampleSelection::selection(method='ml'): ", ll_sample, "\n")
  cat("Diferenca ssmodels - sampleSelection:          ", ll_fit - ll_sample, "\n\n")
}

# ------------------------------------------------------------
# 7. Funcoes para numDeriv
# ------------------------------------------------------------
fn_cl <- function(par) {
  heckmanCL_loglik_independente(par, selectEq, outcomeEq, dados, components_cl)
}

gr_cl <- function(par) {
  heckmanCL_score_independente(par, selectEq, outcomeEq, dados, components_cl)
}

validar_ponto <- function(theta, nome_ponto) {
  ll <- fn_cl(theta)
  if (!is.finite(ll)) return(NULL)

  g_ana <- gr_cl(theta)
  g_num <- numDeriv::grad(fn_cl, theta, method = "Richardson")

  tibble(
    ponto = nome_ponto,
    parametro = param_names,
    analitico = as.numeric(g_ana),
    numerico = as.numeric(g_num),
    diferenca = analitico - numerico,
    erro_abs = abs(diferenca),
    erro_rel = erro_abs / pmax(1, abs(numerico))
  )
}

res_theta_hat <- validar_ponto(theta_hat, "theta_hat")
cat("Resumo da validacao em theta_hat:\n")
print(res_theta_hat, n = Inf)
cat("\nMaximo erro absoluto:", max(res_theta_hat$erro_abs, na.rm = TRUE), "\n")
cat("Maximo erro relativo :", max(res_theta_hat$erro_rel, na.rm = TRUE), "\n")

# ------------------------------------------------------------
# 8. Perturbacoes seguras
# ------------------------------------------------------------
set.seed(20260530)
perturbacoes <- list()
tentativas <- 0L
while (length(perturbacoes) < 5L && tentativas < 200L) {
  tentativas <- tentativas + 1L
  step <- stats::rnorm(length(theta_hat), mean = 0, sd = 0.01 * pmax(1, abs(theta_hat)))
  cand <- theta_hat + step

  idx_sigma <- length(cand) - 1L
  idx_rho <- length(cand)

  cand[idx_sigma] <- max(cand[idx_sigma], 0.05)
  cand[idx_rho] <- min(max(cand[idx_rho], -0.95), 0.95)

  ll_cand <- fn_cl(cand)
  if (!is.finite(ll_cand)) next

  perturbacoes[[length(perturbacoes) + 1L]] <- cand
}

res_pert <- dplyr::bind_rows(lapply(seq_along(perturbacoes), function(j) {
  validar_ponto(perturbacoes[[j]], paste0("perturbado_", j))
}))

res_detalhado <- dplyr::bind_rows(res_theta_hat, res_pert)
res_resumo <- res_detalhado |>
  group_by(ponto) |>
  summarise(
    max_erro_abs = max(erro_abs, na.rm = TRUE),
    max_erro_rel = max(erro_rel, na.rm = TRUE),
    .groups = "drop"
  )

cat("\nResumo geral:\n")
print(res_resumo)

# ------------------------------------------------------------
# 9. Validacao da densidade/probabilidade individual
# ------------------------------------------------------------
dens_cl <- heckmanCL_density_components(theta_hat, selectEq, outcomeEq, dados, components_cl)
dens_cl_50 <- head(dens_cl, 50)

# ------------------------------------------------------------
# 10. Salvar resultados
# ------------------------------------------------------------
readr::write_csv(res_detalhado, "validacao_gradiente_HeckmanCL_detalhada.csv")
readr::write_csv(res_resumo, "validacao_gradiente_HeckmanCL_resumo.csv")
readr::write_csv(dens_cl_50, "validacao_densidade_HeckmanCL_primeiras_50.csv")
saveRDS(
  list(
    fit = fit_cl,
    fit_sampleSelection = fit_sample_ml,
    theta_hat = theta_hat,
    logLik_fit = ll_fit,
    logLik_independente = ll_ind,
    validacao_detalhada = res_detalhado,
    validacao_resumo = res_resumo,
    densidade_primeiras_50 = dens_cl_50
  ),
  "validacao_gradiente_HeckmanCL.rds"
)

cat("\nArquivos gerados:\n")
cat("\n- validacao_gradiente_HeckmanCL_detalhada.csv")
cat("\n- validacao_gradiente_HeckmanCL_resumo.csv")
cat("\n- validacao_densidade_HeckmanCL_primeiras_50.csv")
cat("\n- validacao_gradiente_HeckmanCL.rds\n")

# ============================================================
# Validacao da log-verossimilhanca e gradiente do Heckman-t
# Modelo de Marchenko & Genton (2012), usando MEPS2001.
# ============================================================

rm(list = ls())

pkgs <- c("pkgload", "numDeriv", "dplyr", "tibble", "readr")
for (p in pkgs) {
  if (!requireNamespace(p, quietly = TRUE)) install.packages(p)
}

library(dplyr)
library(tibble)

# ------------------------------------------------------------
# Localizar raiz do pacote
# ------------------------------------------------------------
find_pkg_root <- function(path = getwd()) {
  cur <- normalizePath(path, winslash = "/", mustWork = TRUE)
  for (i in seq_len(8)) {
    if (file.exists(file.path(cur, "DESCRIPTION")) && dir.exists(file.path(cur, "R"))) return(cur)
    parent <- dirname(cur)
    if (identical(parent, cur)) break
    cur <- parent
  }
  stop("Nao encontrei a raiz do pacote. Execute dentro da pasta ssmodels ou em uma subpasta dela.")
}

pkg_root <- find_pkg_root()
message("Raiz do pacote: ", pkg_root)

pkgload::load_all(pkg_root, reset = TRUE, quiet = TRUE)

# Carregar funcoes independentes
validation_file <- file.path(pkg_root, "inst", "validation", "HeckmantS_loglik_score_independente.R")
if (!file.exists(validation_file)) {
  validation_file <- file.path(getwd(), "HeckmantS_loglik_score_independente.R")
}
if (!file.exists(validation_file)) {
  stop("Nao encontrei HeckmantS_loglik_score_independente.R.")
}
source(validation_file)

# ------------------------------------------------------------
# Dados MEPS2001
# ------------------------------------------------------------
if (file.exists(file.path(pkg_root, "data", "MEPS2001.rda"))) {
  load(file.path(pkg_root, "data", "MEPS2001.rda"))
} else {
  data("MEPS2001", package = "ssmodels")
}

dados <- as.data.frame(MEPS2001)

dados <- dados |>
  mutate(
    dambexp_num = as.integer(dambexp == 1),
    lnambx_obs = ifelse(dambexp_num == 1, lnambx, NA_real_)
  )

selectEq <- dambexp_num ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx_obs ~ age + female + educ + blhisp + totchr + ins

# ------------------------------------------------------------
# Ajuste do modelo do pacote
# ------------------------------------------------------------
fit_t <- ssmodels::HeckmantS(
  selection = selectEq,
  outcome   = outcomeEq,
  data      = dados,
  df        = 12
)

print(summary(fit_t))

comp <- .ss_get_tS_components(selectEq, outcomeEq, dados)

# Obter theta_hat na escala livre
if (!is.null(fit_t$coefficients_link)) {
  theta_hat_link <- fit_t$coefficients_link
} else {
  theta_hat_link <- heckmant_natural_to_link(fit_t$coefficients, comp)
}

theta_hat_nat <- heckmant_link_to_natural(theta_hat_link, comp)

ll_pkg <- fit_t$loglik
ll_ind <- heckmant_loglik_link(theta_hat_link, comp)

cat("\nLogLik HeckmantS():          ", ll_pkg, "\n")
cat("LogLik funcao independente: ", ll_ind, "\n")
cat("Diferenca:                  ", ll_ind - ll_pkg, "\n")

# ------------------------------------------------------------
# Validacao do gradiente em theta_hat
# ------------------------------------------------------------
loglik_fun <- function(th) heckmant_loglik_link(th, comp)
score_fun  <- function(th) heckmant_score_link(th, comp)

safe_num_grad <- function(th) {
  numDeriv::grad(loglik_fun, th, method = "Richardson")
}

compare_grad <- function(th, label) {
  g_ana <- score_fun(th)
  g_num <- safe_num_grad(th)
  tibble(
    ponto = label,
    parametro = names(th),
    analitico = as.numeric(g_ana),
    numerico = as.numeric(g_num),
    diferenca = analitico - numerico,
    erro_abs = abs(diferenca),
    erro_rel = abs(diferenca) / pmax(1, abs(numerico))
  )
}

res_hat <- compare_grad(theta_hat_link, "theta_hat")

cat("\nResumo da validacao em theta_hat:\n")
print(res_hat, n = Inf)
cat("\nMaximo erro absoluto:", max(res_hat$erro_abs, na.rm = TRUE), "\n")
cat("Maximo erro relativo :", max(res_hat$erro_rel, na.rm = TRUE), "\n")

# ------------------------------------------------------------
# Perturbacoes seguras
# ------------------------------------------------------------
set.seed(20260530)
perturbacoes <- list()
tries <- 0
while (length(perturbacoes) < 5 && tries < 200) {
  tries <- tries + 1
  th <- theta_hat_link + stats::rnorm(length(theta_hat_link), sd = 0.02)
  th_nat <- heckmant_link_to_natural(th, comp)
  ll <- loglik_fun(th)

  if (is.finite(ll) && th_nat["sigma"] > 0.05 && abs(th_nat["rho"]) < 0.95 &&
      th_nat["df"] > 2.05 && th_nat["df"] < 80) {
    g1 <- try(score_fun(th), silent = TRUE)
    g2 <- try(safe_num_grad(th), silent = TRUE)
    if (!inherits(g1, "try-error") && !inherits(g2, "try-error") &&
        all(is.finite(g1)) && all(is.finite(g2))) {
      perturbacoes[[length(perturbacoes) + 1L]] <- th
    }
  }
}

res_pert <- dplyr::bind_rows(lapply(seq_along(perturbacoes), function(i) {
  compare_grad(perturbacoes[[i]], paste0("perturbado_", i))
}))

res_all <- dplyr::bind_rows(res_hat, res_pert)

resumo <- res_all |>
  group_by(ponto) |>
  summarise(
    max_erro_abs = max(erro_abs, na.rm = TRUE),
    max_erro_rel = max(erro_rel, na.rm = TRUE),
    .groups = "drop"
  )

cat("\nResumo geral:\n")
print(resumo)

# ------------------------------------------------------------
# Checagem das contribuicoes de densidade/log-verossimilhanca
# ------------------------------------------------------------
dens <- heckmant_density_components(theta_hat_nat, comp)
cat("\nChecagem da soma das contribuicoes de loglik observado:\n")
cat("Soma por density_components:", sum(dens$loglik_observed), "\n")
cat("Soma por loglik independente:", ll_ind, "\n")

# ------------------------------------------------------------
# Exportar resultados
# ------------------------------------------------------------
readr::write_csv(res_all, "validacao_gradiente_HeckmantS_detalhada.csv")
readr::write_csv(resumo, "validacao_gradiente_HeckmantS_resumo.csv")
readr::write_csv(head(dens, 50), "validacao_densidade_HeckmantS_primeiras_50.csv")

saveRDS(
  list(
    fit = fit_t,
    theta_hat_link = theta_hat_link,
    theta_hat_natural = theta_hat_nat,
    logLik_package = ll_pkg,
    logLik_independent = ll_ind,
    gradiente_detalhado = res_all,
    resumo = resumo,
    densidade_primeiras_50 = head(dens, 50)
  ),
  "validacao_gradiente_HeckmantS.rds"
)

cat("\nArquivos gerados:\n")
cat("\n- validacao_gradiente_HeckmantS_detalhada.csv")
cat("\n- validacao_gradiente_HeckmantS_resumo.csv")
cat("\n- validacao_densidade_HeckmantS_primeiras_50.csv")
cat("\n- validacao_gradiente_HeckmantS.rds\n")

# ============================================================
# Funções independentes para validação do modelo Heckman-BS
# Pacote: ssmodels
# Objetivo: comparar gradiente analítico contra numDeriv::grad()
# ============================================================

.bs_inv_logit_2 <- function(x) {
  2 / (1 + exp(-x)) - 1
}

.bs_rho_star_from_rho <- function(rho) {
  rho <- pmin(pmax(rho, -0.999999), 0.999999)
  log((1 + rho) / (1 - rho))
}

.bs_safe_ratio <- function(x) {
  exp(stats::dnorm(x, log = TRUE) - stats::pnorm(x, log.p = TRUE))
}

.bs_extract_components_independente <- function(selection, outcome, data) {
  if (exists("extract_model_components", mode = "function")) {
    return(extract_model_components(selection = selection, outcome = outcome, data = data))
  }

  mf_s <- stats::model.frame(selection, data = data, na.action = stats::na.pass)
  YS_raw <- stats::model.response(mf_s)
  XS <- stats::model.matrix(selection, data = mf_s)

  if (is.factor(YS_raw)) {
    lev <- levels(YS_raw)
    if (length(lev) != 2L) stop("A variável de seleção deve possuir dois níveis.")
    YS <- as.integer(YS_raw == lev[2L])
  } else {
    vals <- sort(unique(stats::na.omit(YS_raw)))
    if (!all(vals %in% c(0, 1))) {
      stop("A variável de seleção deve ser binária, codificada como 0/1, ou fator com dois níveis.")
    }
    lev <- vals
    YS <- as.integer(YS_raw)
  }

  mf_o <- stats::model.frame(outcome, data = data, na.action = stats::na.pass)
  YO <- stats::model.response(mf_o)
  XO <- stats::model.matrix(outcome, data = mf_o)

  keep <- !is.na(YS) &
    stats::complete.cases(XS) &
    stats::complete.cases(XO) &
    (YS == 0L | !is.na(YO))

  XS <- XS[keep, , drop = FALSE]
  XO <- XO[keep, , drop = FALSE]
  YS <- YS[keep]
  YO <- YO[keep]

  list(
    XS = XS,
    YS = YS,
    NXS = ncol(XS),
    XO = XO,
    YO = YO,
    NXO = ncol(XO),
    YSLevels = lev
  )
}

heckmanBS_prepare <- function(selection, outcome, data) {
  comp <- .bs_extract_components_independente(selection, outcome, data)
  if (any(comp$YS == 1L & (!is.finite(comp$YO) | comp$YO <= 0))) {
    stop("HeckmanBS requer valores estritamente positivos da variável resposta nas observações selecionadas.")
  }
  comp
}

heckmanBS_loglik_independente <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanBS_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  if (any(!is.finite(par))) return(NA_real_)

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  iphi1  <- max(ibeta) + 1L
  irho   <- iphi1 + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  phi1  <- par[iphi1]
  rho_star <- par[irho]

  if (!is.finite(phi1) || phi1 < 1e-3 || phi1 > 100) return(NA_real_)
  if (!is.finite(rho_star)) return(NA_real_)

  rho <- .bs_inv_logit_2(rho_star)
  phi2 <- 1
  r <- 1 - rho^2
  if (!is.finite(r) || r <= 1e-8) return(NA_real_)

  XS0 <- XS[YS == 0L, , drop = FALSE]
  XS1 <- XS[YS == 1L, , drop = FALSE]
  YO1 <- YO[YS == 1L]
  XO1 <- XO[YS == 1L, , drop = FALSE]

  eta0 <- as.vector(XS0 %*% gamma)
  eta1 <- as.vector(XS1 %*% gamma)
  mu1  <- as.vector(exp(XO1 %*% beta))
  xi0  <- exp(eta0)
  xi1  <- exp(eta1)

  a1 <- sqrt(YO1 * (phi1 + 1) / (phi1 * mu1)) -
    sqrt((phi1 * mu1) / (YO1 * (phi1 + 1)))

  log_bs_density <-
    (-phi1 / 4) * a1^2 +
    log(
      sqrt((phi1 + 1) / (phi1 * mu1 * YO1)) +
        sqrt((phi1 * mu1) / ((phi1 + 1) * YO1^3))
    ) +
    log((1 / (2 * sqrt(2 * pi))) * sqrt(phi1 / 2))

  integral_arg <-
    sqrt((phi2 + 1) / (2 * xi1 * r)) * ((phi2 * xi1) / (phi2 + 1) - 1) +
    rho * sqrt(phi1 / (2 * r)) * a1

  log_selected <- log_bs_density + stats::pnorm(integral_arg, log.p = TRUE)

  cens_arg <- sqrt(phi2 / 2) *
    (sqrt((phi2 + 1) / (phi2 * xi0)) - sqrt((phi2 * xi0) / (phi2 + 1)))

  log_censored <- stats::pnorm(cens_arg, log.p = TRUE)

  ll <- sum(log_selected) + sum(log_censored)
  if (!is.finite(ll)) return(NA_real_)
  ll
}

heckmanBS_score_independente <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanBS_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  if (any(!is.finite(par))) return(rep(NA_real_, length(par)))

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  iphi1  <- max(ibeta) + 1L
  irho   <- iphi1 + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  phi1  <- par[iphi1]
  rho_star <- par[irho]

  if (!is.finite(phi1) || phi1 < 1e-3 || phi1 > 100 || !is.finite(rho_star)) {
    return(rep(NA_real_, length(par)))
  }

  rho <- .bs_inv_logit_2(rho_star)
  phi2 <- 1
  r <- 1 - rho^2
  if (!is.finite(r) || r <= 1e-8) return(rep(NA_real_, length(par)))

  XS0 <- XS[YS == 0L, , drop = FALSE]
  XS1 <- XS[YS == 1L, , drop = FALSE]
  YO1 <- YO[YS == 1L]
  XO1 <- XO[YS == 1L, , drop = FALSE]

  xi0 <- as.vector(exp(XS0 %*% gamma))
  xi1 <- as.vector(exp(XS1 %*% gamma))
  mu1 <- as.vector(exp(XO1 %*% beta))

  a <- sqrt(YO1 * (phi1 + 1) / (phi1 * mu1)) -
    sqrt((phi1 * mu1) / (YO1 * (phi1 + 1)))

  b <- sqrt((phi1 + 1) / (phi1 * mu1 * YO1)) +
    sqrt((phi1 * mu1) / ((phi1 + 1) * YO1^3))

  c_arg <- sqrt((phi2 + 1) / (2 * xi1 * r)) * ((phi2 * xi1) / (phi2 + 1) - 1) +
    rho * sqrt(phi1 / (2 * r)) * a

  d_arg <- sqrt(phi2 / 2) *
    (sqrt((phi2 + 1) / (phi2 * xi0)) - sqrt((phi2 * xi0) / (phi2 + 1)))

  lambda_c <- .bs_safe_ratio(c_arg)
  lambda_d <- .bs_safe_ratio(d_arg)

  # Derivadas auxiliares em relação aos preditores lineares
  da_dbeta <- sweep(XO1, 1,
                    (-0.5) * (
                      sqrt(YO1 * (phi1 + 1) / (phi1 * mu1)) +
                        sqrt((phi1 * mu1) / (YO1 * (phi1 + 1)))
                    ), `*`)

  db_dbeta <- sweep(XO1, 1,
                    0.5 * (
                      sqrt((mu1 * phi1) / (YO1^3 * (phi1 + 1))) -
                        sqrt((phi1 + 1) / (YO1 * phi1 * mu1))
                    ), `*`)

  dd_dgamma <- sweep(XS0, 1,
                     (-0.5) * sqrt(phi2 / 2) * (
                       sqrt((phi2 + 1) / (phi2 * xi0)) +
                         sqrt((phi2 * xi0) / (phi2 + 1))
                     ), `*`)

  da_dphi <- (-0.5) * (
    sqrt(YO1 / (phi1^3 * mu1 * (phi1 + 1))) +
      sqrt(mu1 / (YO1 * (phi1 + 1)^3 * phi1))
  )

  db_dphi <- 0.5 * (
    sqrt(mu1 / (phi1 * (phi1 + 1)^3 * YO1^3)) -
      sqrt(1 / (phi1^3 * (phi1 + 1) * mu1 * YO1))
  )

  dlog_const_dphi <- 1 / (2 * phi1)

  dc_dphi <- rho / (2 * sqrt(2 * phi1 * r)) * a +
    rho * sqrt(phi1 / (2 * r)) * da_dphi

  dc_dbeta <- sweep(da_dbeta, 1, rho * sqrt(phi1 / (2 * r)), `*`)

  # c = A(xi, rho) + rho sqrt(phi/(2r)) a
  # dA/deta = (xi + 2)/(4 sqrt(xi*r)), with phi2=1
  dc_dgamma <- sweep(XS1, 1, (xi1 + 2) / (4 * sqrt(xi1 * r)), `*`)

  # Derivada em relação a rho_star
  A <- sqrt((phi2 + 1) / (2 * xi1 * r)) * ((phi2 * xi1) / (phi2 + 1) - 1)
  dA_drho <- A * rho / r
  dB_drho <- sqrt(phi1 / 2) * (r^(-0.5) + rho^2 * r^(-1.5)) * a
  dc_drho <- dA_drho + dB_drho
  drho_drhostar <- 2 * exp(-rho_star) / (1 + exp(-rho_star))^2
  dc_drhostar <- dc_drho * drho_drhostar

  grad <- numeric(length(par))

  if (nrow(XS0) > 0L) {
    grad[igamma] <- grad[igamma] + colSums(sweep(dd_dgamma, 1, lambda_d, `*`))
  }

  if (nrow(XS1) > 0L) {
    grad[igamma] <- grad[igamma] + colSums(sweep(dc_dgamma, 1, lambda_c, `*`))

    grad[ibeta] <- grad[ibeta] +
      colSums((-phi1 / 2) * sweep(da_dbeta, 1, a, `*`) +
                sweep(db_dbeta, 1, 1 / b, `*`) +
                sweep(dc_dbeta, 1, lambda_c, `*`))

    grad[iphi1] <- grad[iphi1] +
      sum(-a^2 / 4 - (phi1 / 2) * a * da_dphi + db_dphi / b +
            dlog_const_dphi + lambda_c * dc_dphi)

    grad[irho] <- grad[irho] + sum(lambda_c * dc_drhostar)
  }

  grad
}

heckmanBS_density_components <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanBS_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  iphi1  <- max(ibeta) + 1L
  irho   <- iphi1 + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  phi1  <- par[iphi1]
  rho <- .bs_inv_logit_2(par[irho])
  phi2 <- 1
  r <- 1 - rho^2

  out <- data.frame(
    row = seq_along(YS),
    selected = YS,
    value = NA_real_,
    log_contribution = NA_real_,
    type = ifelse(YS == 1L, "selected_density", "censored_probability")
  )

  if (any(YS == 1L)) {
    idx1 <- which(YS == 1L)
    xi1 <- as.vector(exp(XS[idx1, , drop = FALSE] %*% gamma))
    mu1 <- as.vector(exp(XO[idx1, , drop = FALSE] %*% beta))
    y1 <- YO[idx1]
    a <- sqrt(y1 * (phi1 + 1) / (phi1 * mu1)) - sqrt((phi1 * mu1) / (y1 * (phi1 + 1)))
    log_bs_density <- (-phi1 / 4) * a^2 +
      log(sqrt((phi1 + 1) / (phi1 * mu1 * y1)) + sqrt((phi1 * mu1) / ((phi1 + 1) * y1^3))) +
      log((1 / (2 * sqrt(2 * pi))) * sqrt(phi1 / 2))
    c_arg <- sqrt((phi2 + 1) / (2 * xi1 * r)) * ((phi2 * xi1) / (phi2 + 1) - 1) +
      rho * sqrt(phi1 / (2 * r)) * a
    logc <- log_bs_density + stats::pnorm(c_arg, log.p = TRUE)
    out$value[idx1] <- exp(logc)
    out$log_contribution[idx1] <- logc
  }

  if (any(YS == 0L)) {
    idx0 <- which(YS == 0L)
    xi0 <- as.vector(exp(XS[idx0, , drop = FALSE] %*% gamma))
    d_arg <- sqrt(phi2 / 2) * (sqrt((phi2 + 1) / (phi2 * xi0)) - sqrt((phi2 * xi0) / (phi2 + 1)))
    logd <- stats::pnorm(d_arg, log.p = TRUE)
    out$value[idx0] <- exp(logd)
    out$log_contribution[idx0] <- logd
  }

  out
}

test_that("HeckmanBS analytical gradient agrees with independent score", {
  testthat::skip_on_cran()
  testthat::skip_if_not_installed("numDeriv")

  data("MEPS2001", package = "ssmodels")
  dados <- as.data.frame(MEPS2001)

  selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
  outcomeBS <- ambexp ~ age + female + educ + blhisp + totchr + ins

  validation_file <- system.file(
    "validation",
    "HeckmanBS_loglik_score_independente.R",
    package = "ssmodels"
  )
  testthat::skip_if(validation_file == "", "HeckmanBS validation helper not installed")
  source(validation_file, local = TRUE)

  components_bs <- heckmanBS_prepare(selectEq, outcomeBS, dados)
  fit_bs <- ssmodels::HeckmanBS(selection = selectEq, outcome = outcomeBS, data = dados)

  theta_link <- fit_bs$coefficients_link
  names(theta_link) <- c(
    paste0("gamma:", colnames(components_bs$XS)),
    paste0("beta:", colnames(components_bs$XO)),
    "phi1",
    "rho_star"
  )

  score_independent <- heckmanBS_score_independente(
    theta_link, selectEq, outcomeBS, dados, components_bs
  )
  score_numeric <- numDeriv::grad(
    func = heckmanBS_loglik_independente,
    x = theta_link,
    selection = selectEq,
    outcome = outcomeBS,
    data = dados,
    components = components_bs,
    method = "Richardson"
  )

  package_error <- max(abs(as.numeric(fit_bs$gradient_link) - as.numeric(score_independent)))
  numeric_error <- max(abs(as.numeric(score_independent) - as.numeric(score_numeric)))

  testthat::expect_true(all(is.finite(fit_bs$gradient_link)))
  testthat::expect_lt(package_error, 1e-7)
  testthat::expect_lt(numeric_error, 1e-4)
})

## ----setup, include=FALSE-----------------------------------------------------
library(ssmodels)
library(ggplot2)
require(knitr)
require(kfigr)
library(kableExtra)
options(knitr.table.format = "latex")
knitr::opts_chunk$set(echo = TRUE, fig.align = "center", message=FALSE, 
                      warning=FALSE, fig.height=5, fig.width=7.2)

## ----fig4, fig.align= "center", fig.width = 7.2, fig.height= 5, anchor="Figura", fig.alt= "Log of Expenditures Medical"----
library(ssmodels)
#Leitura do dados MEPS2001
data(MEPS2001)
#tornando visiveis as colunas do data-frame
attach(MEPS2001)
barfill <- "grey"
barlines <- "black"
p1 <- ggplot(MEPS2001,aes(ambexp))+geom_histogram(colour = barlines, fill = barfill)+
  scale_x_continuous(name = "(a) Expenditures Medical",
                     breaks = seq(0, 15000, 2500),
                     limits=c(0, 15000))+
  scale_y_continuous(name = "Count",
                     breaks = seq(0, 800, 100),
                     limits=c(0, 800))
p2 <- ggplot(MEPS2001,aes(lambexp))+geom_histogram(colour = barlines, fill = barfill)+
  scale_x_continuous(name = "(b) Log of Expenditures Medical",
                     breaks = seq(0, 11, 1),
                     limits=c(0, 11))+
  scale_y_continuous(name = "Count",
                     breaks = seq(0, 300, 100),
                     limits=c(0, 300))
gridExtra::grid.arrange(p1, p2, ncol=2)

## ----warning=FALSE------------------------------------------------------------
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins 
outcomeS <- cbind(age,female,totchr,ins)
outcomeC <- 1
outcomeBS <- ambexp ~ age + female + educ + blhisp + totchr + ins 
mCL <- HeckmanCL(selectEq, outcomeEq, data = MEPS2001)
mBS <- HeckmanBS(selectEq, outcomeBS, data = MEPS2001)
mSK <- HeckmanSK(selectEq, outcomeEq, data = MEPS2001,lambda = 1)
mtS <- HeckmantS(selectEq, outcomeEq, data = MEPS2001,df=12)
mGe <- HeckmanGe(selectEq, outcomeEq,outcomeS, outcomeC, data = MEPS2001)
Parameters <- c("Intercept", "age", "female", "educ", "blhisp", "totchr", "ins", "income",
   "Intercept", "age", "female", "educ", "blhisp", "totchr", "ins", "sigma", "age", "female", "totchr", "ins", "rho", "nu", "lambda")
HBS <- round(mBS$coefficients, digits = 3)
HCL <- round(mCL$coefficients, digits = 3)
HSK <- round(mSK$coefficients, digits = 3)
HtS <- round(mtS$coefficients, digits = 3)
HGe <- round(mGe$coefficients, digits = 3)
Results <- data.frame("Parameters"= Parameters,
                      "HeckmanGe" = c(HGe[1:21], "NA", "NA"), 
                      "HeckmanCL" = c(HCL[1:16], "NA", "NA", "NA", "NA", HCL[17], "NA", "NA"),
                      "HeckmanBS" = c(HBS[1:16], "NA", "NA", "NA", "NA", HBS[17], "NA", "NA"), 
                      "HeckmantS" = c(HtS[1:16], "NA", "NA", "NA", "NA", HtS[17:18], "NA" ),
                      "HeckmanSK" = c(HSK[1:16], "NA", "NA", "NA", "NA", HSK[17], "NA", HSK[18]))
kable(Results, format = "html", align = c("c", "c", "c", "c", "c"))

## ----warning=FALSE------------------------------------------------------------
summary(mCL)

## ----warning=FALSE------------------------------------------------------------
summary(mGe)

## ----warning=FALSE------------------------------------------------------------
summary(mBS)

## ----warning=FALSE------------------------------------------------------------
summary(mtS)

## ----warning=FALSE------------------------------------------------------------
summary(mSK)

## ----warning=FALSE------------------------------------------------------------
library(ssmodels)
data(nhanes)
attach(nhanes)
perc <- function(x,data){
    nna <- ifelse(sum(is.na(x))!=0,summary(x)[[7]],0)
    perc <- ifelse(sum(is.na(x))!=0,(nna/length(data$id))*100,0)
   return(perc)
}
Variables <- c("SBP (mm Hg)", "Age (year)", "Gender", "BMI (Kg/$m^{2}$)", "Education (years)", "Race", "Income ($\\$1000$ per year)", "Numbers Obs.")
perc1 <- round(perc(sbp,nhanes), digits = 2)
perc2 <- round(perc(age,nhanes), digits = 2)
perc3 <- round(perc(gender,nhanes), digits = 2)
perc4 <- round(perc(bmi,nhanes), digits = 2)
perc5 <- round(perc(educ,nhanes), digits = 2)
perc6 <- round(perc(race,nhanes), digits = 2)
perc7 <- round(perc(Income,nhanes), digits = 2)
nObs <- length(Income)
Percentage <- c(perc1, perc2, perc3, perc4, perc5, perc6, perc7, nObs)
df <- subset(nhanes, !is.na(sbp))
df <- subset(df, !is.na(bmi))
attach(df)
perc11 <- round(perc(sbp,df), digits = 2)
perc12 <- round(perc(age,df), digits = 2)
perc13 <- round(perc(gender,df), digits = 2)
perc14 <- round(perc(bmi,df), digits = 2)
perc15 <- round(perc(educ,df), digits = 2)
perc16 <- round(perc(race,df), digits = 2)
perc17 <- round(perc(Income,df), digits = 2)
nObs1 <- length(Income)
Percentage1 <- c(perc11, perc12, perc13, perc14, perc15, perc16, perc17, nObs1)
table <- data.frame("Variables" = Variables, "Percentage of Missing"= Percentage, "Without Missing"= Percentage1)
kable(table, format = "html", align = c("c", "c", "c"))

## ----warning=FALSE, fig.alt="Log Systolic blood pressure"---------------------
library("gridExtra")
barfill <- "grey"
barlines <- "black"
p1 <- ggplot(df, aes(Income)) + geom_histogram( breaks = seq(0, 10, 0.5), aes(y = ..density..), colour = barlines, fill = barfill)+
  scale_x_continuous(name = "Income",
                   breaks = seq(0, 10, 2),
                   limits=c(0, 10))

p2 <- ggplot(df, aes(log(sbp))) + geom_histogram( colour = barlines, fill = barfill) +
  scale_x_continuous(name = "Log Systolic blood pressure")
grid.arrange(p1, p2, ncol=2)

## ----warning = FALSE----------------------------------------------------------
df$YS <- ifelse(is.na(df$Income),0,1)
df$educ <- ifelse(df$educ<=2,0,1)
df$Income <- ifelse(is.na(df$Income),0,df$Income)
attach(df)
selectionEq <- YS~age+gender+educ+race
outcomeEq   <- log(sbp)~age+gender+educ+bmi+Income
outcomeBS   <- sbp~age+gender+educ+bmi+Income
mCL <- HeckmanCL(selectionEq, outcomeEq, data = df)
mBS <- HeckmanBS(selectionEq, outcomeBS, data = df)
mSK <- HeckmanSK(selectionEq, outcomeEq, data = df, lambda = 0)
mtS <- HeckmantS(selectionEq, outcomeEq, data = df, df = 15)

    Parameters <- c("Intercept", "age", "gender", "educ", "race", "Intercept", "age", "gender", "educ", "bmi", "income", "sigma", "rho", "nu", "lambda")
HBS <- round(mBS$coefficients, digits = 5)
HCL <- round(mCL$coefficients, digits = 5)
HSK <- round(mSK$coefficients, digits = 5)
HtS <- round(mtS$coefficients, digits = 5)
Results <- data.frame("Parameters"= Parameters, 
                      "HeckmanCL" = c(HCL[1:13], "NA", "NA"),
                      "HeckmanBS" = c(HBS[1:13], "NA", "NA"), 
                      "HeckmantS" = c(HtS[1:13], HtS[14], "NA"),
                      "HeckmanSK" = c(HSK[1:13], "NA", HSK[14]))
kable(Results, format = "html", align = c("c", "c", "c", "c", "c"))

## ----warning = FALSE----------------------------------------------------------
summary(mCL)

## ----warning = FALSE----------------------------------------------------------
summary(mtS)

## ----warning = FALSE----------------------------------------------------------
summary(mBS)

## ----warning = FALSE----------------------------------------------------------
summary(mSK)

# ============================================================
# Funções independentes para validar a log-verossimilhança
# e o gradiente do modelo Heckman Generalizado.
#
# Parametrização:
#   sigma_i = exp(E_i %*% lambda)
#   rho_i   = tanh(V_i %*% kappa)
#
# A ordem dos parâmetros é:
#   theta = c(gamma, beta, lambda, kappa)
#
# Fórmula-base:
#   l_i(theta) = u_i{log Phi(zeta_i) + log phi(z_i) - log sigma_i}
#              + (1-u_i) log Phi(-mu2_i),
#
# em que
#   z_i     = (y_i - mu1_i)/sigma_i,
#   zeta_i  = (mu2_i + rho_i z_i)/sqrt(1-rho_i^2).
# ============================================================

.heckmanGE_mills <- function(x) {
  exp(stats::dnorm(x, log = TRUE) - stats::pnorm(x, log.p = TRUE))
}

.heckmanGE_sech <- function(x) 1 / cosh(x)

heckmanGE_make_components <- function(selection,
                                      outcome,
                                      dispersion,
                                      correlation,
                                      data,
                                      weights = NULL) {
  yS_name <- as.character(selection[[2L]])

  vars_selection  <- all.vars(selection)
  vars_outcome    <- all.vars(outcome)
  vars_dispersion <- all.vars(dispersion)
  vars_correlation <- all.vars(correlation)

  complete_selection <- stats::complete.cases(data[, vars_selection, drop = FALSE])
  complete_correlation <- stats::complete.cases(data[, vars_correlation, drop = FALSE])
  complete_outdisp <- stats::complete.cases(
    data[, unique(c(vars_outcome, vars_dispersion)), drop = FALSE]
  )

  ys_raw <- data[[yS_name]]
  if (is.factor(ys_raw)) {
    if (nlevels(ys_raw) != 2L) stop("A variável de seleção deve ter exatamente dois níveis.")
    ys_num <- as.integer(ys_raw == levels(ys_raw)[2L])
  } else {
    ys_num <- as.integer(ys_raw)
  }

  keep <- complete_selection & complete_correlation &
    ((ys_num == 0L) | ((ys_num == 1L) & complete_outdisp))

  data2 <- data[keep, , drop = FALSE]

  mfS <- stats::model.frame(selection, data = data2, na.action = stats::na.pass)
  YS_raw <- stats::model.response(mfS)
  if (is.factor(YS_raw)) {
    YS <- as.integer(YS_raw == levels(YS_raw)[2L])
  } else {
    YS <- as.integer(YS_raw)
  }
  XS <- stats::model.matrix(stats::terms(mfS), mfS)

  mfO <- stats::model.frame(outcome, data = data2, na.action = stats::na.pass)
  YO <- stats::model.response(mfO)
  XO <- stats::model.matrix(attr(mfO, "terms"), mfO)

  mfD <- stats::model.frame(dispersion, data = data2, na.action = stats::na.pass)
  Msigma <- stats::model.matrix(attr(mfD, "terms"), mfD)

  mfC <- stats::model.frame(correlation, data = data2, na.action = stats::na.pass)
  Mrho <- stats::model.matrix(attr(mfC, "terms"), mfC)

  if (is.null(weights)) {
    w <- rep(1, length(YS))
  } else {
    w_all <- eval(substitute(weights), data2, parent.frame())
    w <- as.numeric(w_all)
    if (length(w) != length(YS)) stop("O vetor de pesos possui comprimento incompatível.")
  }

  list(
    data = data2,
    YS = YS,
    YO = YO,
    XS = XS,
    XO = XO,
    Msigma = Msigma,
    Mrho = Mrho,
    w = w,
    index = heckmanGE_parameter_index(XS, XO, Msigma, Mrho)
  )
}

heckmanGE_parameter_index <- function(XS, XO, Msigma, Mrho) {
  NXS <- ncol(XS)
  NXO <- ncol(XO)
  NE  <- ncol(Msigma)
  NV  <- ncol(Mrho)

  istartS <- seq_len(NXS)
  istartO <- seq.int(NXS + 1L, NXS + NXO)
  ilambda <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
  ikappa  <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)

  list(
    NXS = NXS,
    NXO = NXO,
    NE = NE,
    NV = NV,
    istartS = istartS,
    istartO = istartO,
    ilambda = ilambda,
    ikappa = ikappa,
    npar = NXS + NXO + NE + NV,
    parameter_names = c(
      paste0("selection:", colnames(XS)),
      paste0("outcome:", colnames(XO)),
      paste0("dispersion:", colnames(Msigma)),
      paste0("correlation:", colnames(Mrho))
    )
  )
}

heckmanGE_loglik_indep <- function(theta,
                                   YS,
                                   XS,
                                   YO,
                                   XO,
                                   Msigma,
                                   Mrho,
                                   w = NULL) {
  idx <- heckmanGE_parameter_index(XS, XO, Msigma, Mrho)
  if (length(theta) != idx$npar) stop("Comprimento incorreto de theta.")
  if (is.null(w)) w <- rep(1, length(YS))

  gamma  <- theta[idx$istartS]
  beta   <- theta[idx$istartO]
  lambda <- theta[idx$ilambda]
  kappa  <- theta[idx$ikappa]

  YS <- as.integer(YS)
  YO_clean <- ifelse(is.na(YO), 0, YO)

  mu2 <- drop(XS %*% gamma)
  mu1 <- drop(XO %*% beta)
  sigma <- as.numeric(exp(Msigma %*% lambda))
  rho <- as.numeric(tanh(Mrho %*% kappa))

  if (any(!is.finite(sigma)) || any(sigma <= 0)) return(NA_real_)
  if (any(!is.finite(rho)) || any(abs(rho) >= 1)) return(NA_real_)

  r <- sqrt(1 - rho^2)
  z <- (YO_clean - mu1) / sigma
  zeta <- (mu2 + rho * z) / r

  ll0 <- stats::pnorm(-mu2, log.p = TRUE)
  ll1 <- stats::dnorm(z, log = TRUE) - log(sigma) +
    stats::pnorm(zeta, log.p = TRUE)

  sum(w * ifelse(YS == 0L, ll0, ll1))
}

heckmanGE_score_i_indep <- function(theta,
                                    YS,
                                    XS,
                                    YO,
                                    XO,
                                    Msigma,
                                    Mrho,
                                    w = NULL) {
  idx <- heckmanGE_parameter_index(XS, XO, Msigma, Mrho)
  if (length(theta) != idx$npar) stop("Comprimento incorreto de theta.")
  if (is.null(w)) w <- rep(1, length(YS))

  gamma  <- theta[idx$istartS]
  beta   <- theta[idx$istartO]
  lambda <- theta[idx$ilambda]
  kappa  <- theta[idx$ikappa]

  YS <- as.integer(YS)
  YO_clean <- ifelse(is.na(YO), 0, YO)

  XS0 <- XS[YS == 0L, , drop = FALSE]
  XS1 <- XS[YS == 1L, , drop = FALSE]
  XO1 <- XO[YS == 1L, , drop = FALSE]
  YO1 <- YO_clean[YS == 1L]
  MS1 <- Msigma[YS == 1L, , drop = FALSE]
  MR1 <- Mrho[YS == 1L, , drop = FALSE]

  mu20 <- drop(XS0 %*% gamma)
  mu21 <- drop(XS1 %*% gamma)
  mu11 <- drop(XO1 %*% beta)

  sigma1 <- as.numeric(exp(MS1 %*% lambda))
  eta_rho1 <- as.numeric(MR1 %*% kappa)
  rho1 <- tanh(eta_rho1)

  r1 <- sqrt(1 - rho1^2)
  A_rho <- 1 / r1
  A_rrho <- rho1 / r1

  z1 <- (YO1 - mu11) / sigma1
  zeta1 <- mu21 * A_rho + z1 * A_rrho

  Mmu2 <- .heckmanGE_mills(-mu20)
  Mzeta <- .heckmanGE_mills(zeta1)

  sech2 <- .heckmanGE_sech(eta_rho1)^2
  Q_rho <- mu21 * rho1 * A_rho^2 + z1 * (1 + A_rrho^2)

  gradient <- matrix(0, nrow = length(YS), ncol = idx$npar)
  colnames(gradient) <- idx$parameter_names

  gradient[YS == 0L, idx$istartS] <- -XS0 * Mmu2
  gradient[YS == 1L, idx$istartS] <- XS1 * (Mzeta * A_rho)
  gradient[YS == 1L, idx$istartO] <- XO1 *
    ((z1 - Mzeta * A_rrho) / sigma1)
  gradient[YS == 1L, idx$ilambda] <- MS1 *
    (z1^2 - 1 - Mzeta * z1 * A_rrho)
  gradient[YS == 1L, idx$ikappa] <- MR1 *
    (Mzeta * A_rho * sech2 * Q_rho)

  gradient * w
}

heckmanGE_score_indep <- function(theta,
                                  YS,
                                  XS,
                                  YO,
                                  XO,
                                  Msigma,
                                  Mrho,
                                  w = NULL) {
  colSums(
    heckmanGE_score_i_indep(
      theta = theta,
      YS = YS,
      XS = XS,
      YO = YO,
      XO = XO,
      Msigma = Msigma,
      Mrho = Mrho,
      w = w
    )
  )
}

heckmanGE_validate_gradient <- function(theta,
                                        YS,
                                        XS,
                                        YO,
                                        XO,
                                        Msigma,
                                        Mrho,
                                        w = NULL,
                                        ponto = "theta") {
  if (!requireNamespace("numDeriv", quietly = TRUE)) {
    stop("Instale o pacote numDeriv para usar esta função.")
  }

  idx <- heckmanGE_parameter_index(XS, XO, Msigma, Mrho)
  names(theta) <- idx$parameter_names

  loglik_fun <- function(par) {
    heckmanGE_loglik_indep(par, YS, XS, YO, XO, Msigma, Mrho, w)
  }

  g_ana <- heckmanGE_score_indep(theta, YS, XS, YO, XO, Msigma, Mrho, w)
  g_num <- numDeriv::grad(loglik_fun, theta, method = "Richardson")

  out <- data.frame(
    ponto = ponto,
    parametro = idx$parameter_names,
    analitico = as.numeric(g_ana),
    numerico = as.numeric(g_num),
    diferenca = as.numeric(g_ana - g_num),
    erro_abs = abs(as.numeric(g_ana - g_num)),
    stringsAsFactors = FALSE
  )

  out$erro_rel <- out$erro_abs / pmax(1, abs(out$numerico), abs(out$analitico))
  out
}

# ============================================================
# Funcoes independentes para validar o modelo Heckman-t
# Marchenko & Genton (2012), Equacao (14) e Apendice A.
# ============================================================

.ss_t_ratio <- function(x, df) {
  exp(stats::dt(x, df = df, log = TRUE) - stats::pt(x, df = df, log.p = TRUE))
}

.ss_get_tS_components <- function(selection, outcome, data) {
  if (exists("extract_model_components", mode = "function")) {
    comp <- extract_model_components(selection = selection, outcome = outcome, data = data)
  } else if (requireNamespace("ssmodels", quietly = TRUE)) {
    comp <- ssmodels::extract_model_components(selection = selection, outcome = outcome, data = data)
  } else {
    stop("A funcao extract_model_components() nao foi encontrada. Carregue o pacote ssmodels.")
  }
  comp$YO_clean <- ifelse(is.na(comp$YO), 0, comp$YO)
  comp
}

# ------------------------------------------------------------
# Transforma parametro em escala livre para escala natural.
# Ordem na escala livre:
#   gamma, beta, log.sigma, atanh.rho, log.df.minus.2
# Ordem natural:
#   gamma, beta, sigma, rho, nu
# ------------------------------------------------------------
heckmant_link_to_natural <- function(theta_link, comp) {
  NXS <- comp$NXS
  NXO <- comp$NXO
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  inu <- irho + 1L

  theta_nat <- theta_link
  theta_nat[isigma] <- exp(theta_link[isigma])
  theta_nat[irho] <- tanh(theta_link[irho])
  theta_nat[inu] <- 2 + exp(theta_link[inu])
  names(theta_nat) <- c(colnames(comp$XS), colnames(comp$XO), "sigma", "rho", "df")
  theta_nat
}

heckmant_natural_to_link <- function(theta_nat, comp) {
  NXS <- comp$NXS
  NXO <- comp$NXO
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  inu <- irho + 1L

  theta_link <- theta_nat
  theta_link[isigma] <- log(theta_nat[isigma])
  theta_link[irho] <- atanh(max(min(theta_nat[irho], 0.999999), -0.999999))
  theta_link[inu] <- log(max(theta_nat[inu] - 2, .Machine$double.eps))
  names(theta_link) <- c(colnames(comp$XS), colnames(comp$XO),
                         "log.sigma", "atanh.rho", "log.df.minus.2")
  theta_link
}

# ------------------------------------------------------------
# Log-verossimilhanca em escala natural.
# Equacao (14) de Marchenko & Genton (2012):
# l_i = u_i[log t_nu(z_i) - log sigma + log T_{nu+1}(eta_i)]
#       + (1-u_i)log T_nu(-w_i'gama)
# ------------------------------------------------------------
heckmant_loglik_natural <- function(theta_nat, comp) {
  NXS <- comp$NXS
  NXO <- comp$NXO
  istartS <- seq_len(NXS)
  istartO <- seq.int(NXS + 1L, NXS + NXO)
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  inu <- irho + 1L

  gamma <- theta_nat[istartS]
  beta <- theta_nat[istartO]
  sigma <- theta_nat[isigma]
  rho <- theta_nat[irho]
  nu <- theta_nat[inu]

  if (any(!is.finite(theta_nat)) || !is.finite(sigma) || sigma <= 0 ||
      !is.finite(rho) || abs(rho) >= 1 || !is.finite(nu) || nu <= 2) {
    return(-Inf)
  }

  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO_clean <- comp$YO_clean

  eta_s <- drop(XS %*% gamma)
  mu_y <- drop(XO %*% beta)
  z <- (YO_clean - mu_y) / sigma
  r <- sqrt(1 - rho^2)
  Q <- sqrt((nu + 1) / (nu + z^2))
  eta <- Q * ((rho * z + eta_s) / r)

  ll0 <- stats::pt(-eta_s, df = nu, log.p = TRUE)
  ll1 <- stats::dt(z, df = nu, log = TRUE) - log(sigma) +
    stats::pt(eta, df = nu + 1, log.p = TRUE)

  val <- sum(ifelse(YS == 0L, ll0, ll1))
  if (!is.finite(val)) return(-Inf)
  val
}

heckmant_loglik_link <- function(theta_link, comp) {
  heckmant_loglik_natural(heckmant_link_to_natural(theta_link, comp), comp)
}

# ------------------------------------------------------------
# Gradiente analitico em escala natural.
# Os componentes beta, gamma, rho e sigma seguem o Apendice A.
# O componente nu e calculado por diferenciacao numerica escalar,
# como recomendado no proprio artigo para os termos envolvendo T_nu.
# ------------------------------------------------------------
heckmant_score_natural <- function(theta_nat, comp) {
  if (!requireNamespace("numDeriv", quietly = TRUE)) {
    stop("Instale o pacote numDeriv para calcular o componente em df.")
  }

  NXS <- comp$NXS
  NXO <- comp$NXO
  istartS <- seq_len(NXS)
  istartO <- seq.int(NXS + 1L, NXS + NXO)
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  inu <- irho + 1L

  gamma <- theta_nat[istartS]
  beta <- theta_nat[istartO]
  sigma <- theta_nat[isigma]
  rho <- theta_nat[irho]
  nu <- theta_nat[inu]

  if (any(!is.finite(theta_nat)) || sigma <= 0 || abs(rho) >= 1 || nu <= 2) {
    return(rep(NA_real_, length(theta_nat)))
  }

  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO_clean

  idx1 <- YS == 1L
  idx0 <- YS == 0L

  XS1 <- XS[idx1, , drop = FALSE]
  XO1 <- XO[idx1, , drop = FALSE]
  Y1 <- YO[idx1]
  XS0 <- XS[idx0, , drop = FALSE]

  eta1 <- drop(XS1 %*% gamma)
  eta0 <- drop(XS0 %*% gamma)
  mu1 <- drop(XO1 %*% beta)

  z <- (Y1 - mu1) / sigma
  r <- sqrt(1 - rho^2)
  A <- 1 / r
  Arho <- rho / r
  Q <- sqrt((nu + 1) / (nu + z^2))
  zeta <- (rho * z + eta1) / r
  eta <- Q * zeta
  M <- .ss_t_ratio(eta, df = nu + 1)

  # gamma
  score_gamma <- rep(0, NXS)
  if (any(idx1)) {
    score_gamma <- score_gamma + colSums(XS1 * as.numeric(A * Q * M))
  }
  if (any(idx0)) {
    M0 <- .ss_t_ratio(-eta0, df = nu)
    score_gamma <- score_gamma - colSums(XS0 * as.numeric(M0))
  }

  # beta
  score_beta <- rep(0, NXO)
  if (any(idx1)) {
    term_beta <- (Q / sigma) *
      (Q * z + (zeta * z / (nu + z^2) - Arho) * M)
    score_beta <- colSums(XO1 * as.numeric(term_beta))
  }

  # sigma
  score_sigma <- 0
  if (any(idx1)) {
    term_sigma <- (-1 + Q^2 * z^2 +
                     Q * z * (zeta * z / (nu + z^2) - Arho) * M) / sigma
    score_sigma <- sum(term_sigma)
  }

  # rho
  score_rho <- 0
  if (any(idx1)) {
    score_rho <- sum(Q * A^3 * (z + rho * eta1) * M)
  }

  # nu: derivada numerica escalar da log-verossimilhanca em escala natural
  score_nu <- numDeriv::grad(
    func = function(nu_value) {
      th <- theta_nat
      th[inu] <- nu_value
      heckmant_loglik_natural(th, comp)
    },
    x = nu,
    method = "Richardson"
  )

  out <- c(score_gamma, score_beta, score_sigma, score_rho, score_nu)
  names(out) <- c(colnames(comp$XS), colnames(comp$XO), "sigma", "rho", "df")
  out
}

heckmant_score_link <- function(theta_link, comp) {
  theta_nat <- heckmant_link_to_natural(theta_link, comp)
  score_nat <- heckmant_score_natural(theta_nat, comp)

  NXS <- comp$NXS
  NXO <- comp$NXO
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  inu <- irho + 1L

  jac <- rep(1, length(theta_link))
  jac[isigma] <- theta_nat[isigma]       # d sigma / d log.sigma
  jac[irho] <- 1 - theta_nat[irho]^2     # d rho / d atanh.rho
  jac[inu] <- theta_nat[inu] - 2         # d nu / d log(nu - 2)

  out <- score_nat * jac
  names(out) <- names(theta_link)
  out
}

# ------------------------------------------------------------
# Contribuicoes de densidade/likelihood para checagem didatica.
# Retorna densidade condicional f(y | U=1) e contribuicao observada.
# ------------------------------------------------------------
heckmant_density_components <- function(theta_nat, comp) {
  NXS <- comp$NXS
  NXO <- comp$NXO
  gamma <- theta_nat[seq_len(NXS)]
  beta <- theta_nat[seq.int(NXS + 1L, NXS + NXO)]
  sigma <- theta_nat[NXS + NXO + 1L]
  rho <- theta_nat[NXS + NXO + 2L]
  nu <- theta_nat[NXS + NXO + 3L]

  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO_clean

  eta_s <- drop(XS %*% gamma)
  mu_y <- drop(XO %*% beta)
  z <- (YO - mu_y) / sigma
  r <- sqrt(1 - rho^2)
  Q <- sqrt((nu + 1) / (nu + z^2))
  eta <- Q * ((rho * z + eta_s) / r)

  f_y <- stats::dt(z, df = nu) / sigma
  p_sel_given_y <- stats::pt(eta, df = nu + 1)
  p_sel <- stats::pt(eta_s, df = nu)
  p_not_sel <- stats::pt(-eta_s, df = nu)

  f_cond <- f_y * p_sel_given_y / p_sel
  likelihood_observed <- ifelse(YS == 1L, f_y * p_sel_given_y, p_not_sel)

  data.frame(
    selected = YS,
    z = z,
    eta_selection = eta_s,
    eta_conditional = eta,
    f_y = f_y,
    p_sel_given_y = p_sel_given_y,
    p_sel = p_sel,
    f_cond_y_given_selected = f_cond,
    likelihood_observed = likelihood_observed,
    loglik_observed = log(likelihood_observed)
  )
}

# ============================================================
# Validação do gradiente analítico da função HeckmanSK()
# Modelo: Selection skew-normal model de Ogundimu e Hutton
# Estratégia: comparar gradiente analítico corrigido com numDeriv::grad()
# Banco: MEPS2001
# ============================================================

rm(list = ls())

# ------------------------------------------------------------
# 1. Pacotes necessários
# ------------------------------------------------------------
# pkgs <- c("pkgload", "numDeriv", "sn", "dplyr", "tibble")
#
# for (p in pkgs) {
#   if (!requireNamespace(p, quietly = TRUE)) {
#     install.packages(p)
#   }
# }

library(pkgload)
library(numDeriv)
library(sn)
library(dplyr)
library(tibble)

# ------------------------------------------------------------
# 2. Carregar pacote local
#    O script funciona tanto na raiz do pacote quanto em ssmodels/Teste.
# ------------------------------------------------------------
if (file.exists("DESCRIPTION")) {
  pkg_root <- "."
} else if (file.exists(file.path("..", "DESCRIPTION"))) {
  pkg_root <- ".."
} else if (file.exists(file.path("..", "..", "DESCRIPTION"))) {
  pkg_root <- file.path("..", "..")
} else {
  stop("Execute este script na raiz do pacote ssmodels, dentro de ssmodels/Teste ou em inst/validation.")
}

pkgload::load_all(pkg_root, reset = TRUE, quiet = TRUE)

# ------------------------------------------------------------
# 3. Carregar MEPS2001
# ------------------------------------------------------------
if (exists("MEPS2001")) rm(MEPS2001)

data("MEPS2001", package = "ssmodels", envir = environment())
dados <- as.data.frame(MEPS2001)

# Padronização para comparação e estimação.
dados <- dados |>
  mutate(
    dambexp_num = as.integer(dambexp == 1),
    lnambx_obs  = ifelse(dambexp_num == 1, lnambx, NA_real_)
  )

selectEq <- dambexp_num ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx_obs ~ age + female + educ + blhisp + totchr + ins

# ------------------------------------------------------------
# 4. Funções independentes de log-verossimilhança e gradiente
# ------------------------------------------------------------
.sk_components <- function(selection, outcome, data) {
  comp <- ssmodels::extract_model_components(selection = selection,
                                             outcome = outcome,
                                             data = data)
  comp$YO[is.na(comp$YO)] <- 0
  comp
}

.sk_loglik <- function(theta, comp) {
  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO
  NXS <- ncol(XS)
  NXO <- ncol(XO)

  istartS <- seq_len(NXS)
  istartO <- seq(from = NXS + 1L, length.out = NXO)
  isigma  <- NXS + NXO + 1L
  irho    <- NXS + NXO + 2L
  ilamb1  <- NXS + NXO + 3L

  gamma  <- theta[istartS]
  beta   <- theta[istartO]
  sigma  <- theta[isigma]
  rho    <- theta[irho]
  lambda <- theta[ilamb1]

  if (!is.finite(sigma) || sigma <= 0) return(NA_real_)
  if (!is.finite(rho) || abs(rho) >= 1) return(NA_real_)
  if (!is.finite(lambda)) return(NA_real_)

  eta <- as.numeric(XS %*% gamma)
  mu  <- as.numeric(XO %*% beta)
  z   <- (YO - mu) / sigma
  r   <- 1 - rho^2
  u   <- 1 + lambda^2 - (lambda * rho)^2
  q   <- lambda * rho / sqrt(u)

  omega <- (eta + rho * z) / sqrt(r)

  ll_selected <-
    log(2) - log(sigma) +
    stats::dnorm(z, log = TRUE) +
    stats::pnorm(lambda * z, log.p = TRUE) +
    stats::pnorm(omega, log.p = TRUE)

  # P(S = 0 | w) = F_SN(-eta; q)
  F0 <- sn::psn(-eta, xi = 0, omega = 1, alpha = q)
  F0 <- pmax(F0, .Machine$double.xmin)
  ll_censored <- log(F0)

  sum(ifelse(YS == 1, ll_selected, ll_censored))
}

.sk_grad <- function(theta, comp) {
  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO
  NXS <- ncol(XS)
  NXO <- ncol(XO)
  nObs <- length(YS)
  nParam <- NXS + NXO + 3L

  istartS <- seq_len(NXS)
  istartO <- seq(from = NXS + 1L, length.out = NXO)
  isigma  <- NXS + NXO + 1L
  irho    <- NXS + NXO + 2L
  ilamb1  <- NXS + NXO + 3L

  gamma  <- theta[istartS]
  beta   <- theta[istartO]
  sigma  <- theta[isigma]
  rho    <- theta[irho]
  lambda <- theta[ilamb1]

  if (!is.finite(sigma) || sigma <= 0) return(rep(NA_real_, nParam))
  if (!is.finite(rho) || abs(rho) >= 1) return(rep(NA_real_, nParam))
  if (!is.finite(lambda)) return(rep(NA_real_, nParam))

  sel0 <- YS == 0
  sel1 <- YS == 1

  XS0 <- XS[sel0, , drop = FALSE]
  XS1 <- XS[sel1, , drop = FALSE]
  XO1 <- XO[sel1, , drop = FALSE]
  YO1 <- YO[sel1]

  eta0 <- as.numeric(XS0 %*% gamma)
  eta1 <- as.numeric(XS1 %*% gamma)
  mu1  <- as.numeric(XO1 %*% beta)

  z <- (YO1 - mu1) / sigma
  r <- 1 - rho^2
  u <- 1 + lambda^2 - (lambda * rho)^2
  q <- lambda * rho / sqrt(u)

  omega <- (eta1 + rho * z) / sqrt(r)
  a     <- lambda * z

  K1 <- exp(stats::dnorm(omega, log = TRUE) -
              stats::pnorm(omega, log.p = TRUE))
  K3 <- exp(stats::dnorm(a, log = TRUE) -
              stats::pnorm(a, log.p = TRUE))

  F0 <- sn::psn(-eta0, xi = 0, omega = 1, alpha = q)
  F0 <- pmax(F0, .Machine$double.xmin)

  K2 <- exp(stats::dnorm(eta0, log = TRUE) +
              stats::pnorm(-q * eta0, log.p = TRUE) -
              log(F0))

  K4 <- exp(stats::dnorm(eta0 * sqrt(1 + lambda^2) / sqrt(u), log = TRUE) -
              log(F0))

  G <- matrix(0, nrow = nObs, ncol = nParam)

  if (nrow(XS1) > 0L) {
    G[sel1, istartS] <- XS1 * as.numeric(K1 / sqrt(r))
  }
  if (nrow(XS0) > 0L) {
    G[sel0, istartS] <- XS0 * as.numeric(-2 * K2)
  }

  if (nrow(XO1) > 0L) {
    G[sel1, istartO] <- XO1 * as.numeric(
      z / sigma -
      lambda * K3 / sigma -
      rho * K1 / (sigma * sqrt(r))
    )
  }

  if (sum(sel1) > 0L) {
    G[sel1, isigma] <-
      -1 / sigma +
      z^2 / sigma -
      lambda * z * K3 / sigma -
      rho * z * K1 / (sigma * sqrt(r))

    G[sel1, irho] <-
      K1 * (z + rho * eta1) / (r^(3 / 2))

    G[sel1, ilamb1] <-
      K3 * z
  }

  if (sum(sel0) > 0L) {
    G[sel0, irho] <-
      -2 * lambda * K4 / sqrt(2 * pi * u)

    G[sel0, ilamb1] <-
      -2 * rho * K4 / ((1 + lambda^2) * sqrt(2 * pi * u))
  }

  colSums(G)
}

.validar_um_ponto <- function(theta, comp, nome = "theta") {
  g_ana <- .sk_grad(theta, comp)
  g_num <- numDeriv::grad(func = .sk_loglik, x = theta, comp = comp,
                          method = "Richardson")

  tibble(
    ponto = nome,
    parametro = names(theta),
    analitico = as.numeric(g_ana),
    numerico = as.numeric(g_num),
    diferenca = analitico - numerico,
    erro_abs = abs(diferenca),
    erro_rel = abs(diferenca) / (1 + abs(numerico))
  )
}

# ------------------------------------------------------------
# 5. Ajuste do modelo e validação no máximo encontrado
# ------------------------------------------------------------
fit_sk <- ssmodels::HeckmanSK(selection = selectEq,
                              outcome   = outcomeEq,
                              data      = dados)

comp <- .sk_components(selectEq, outcomeEq, dados)
theta_hat <- fit_sk$coefficients

cat("\nLogLik HeckmanSK(): ", fit_sk$loglik, "\n")
cat("LogLik função independente: ", .sk_loglik(theta_hat, comp), "\n")

valid_hat <- .validar_um_ponto(theta_hat, comp, nome = "theta_hat")

cat("\nResumo da validação em theta_hat:\n")
print(valid_hat, n = Inf)
cat("\nMáximo erro absoluto:", max(valid_hat$erro_abs, na.rm = TRUE), "\n")
cat("Máximo erro relativo :", max(valid_hat$erro_rel, na.rm = TRUE), "\n")

# ------------------------------------------------------------
# 6. Validação em pontos perturbados, para evitar validação apenas no ótimo
# ------------------------------------------------------------
set.seed(123)
B <- 5L
NXS <- comp$NXS
NXO <- comp$NXO
isigma <- NXS + NXO + 1L
irho   <- NXS + NXO + 2L
ilamb  <- NXS + NXO + 3L

valid_list <- vector("list", B + 1L)
valid_list[[1L]] <- valid_hat

for (b in seq_len(B)) {
  theta_b <- theta_hat + stats::rnorm(length(theta_hat), mean = 0, sd = 0.03)
  theta_b[isigma] <- max(0.2, theta_b[isigma])
  theta_b[irho]   <- max(-0.85, min(0.85, theta_b[irho]))
  theta_b[ilamb]  <- max(-4, min(4, theta_b[ilamb]))
  names(theta_b)  <- names(theta_hat)

  valid_list[[b + 1L]] <- .validar_um_ponto(theta_b, comp, nome = paste0("perturbado_", b))
}

valid_all <- dplyr::bind_rows(valid_list)

resumo <- valid_all |>
  group_by(ponto) |>
  summarise(
    max_erro_abs = max(erro_abs, na.rm = TRUE),
    max_erro_rel = max(erro_rel, na.rm = TRUE),
    .groups = "drop"
  )

cat("\nResumo geral:\n")
print(resumo, n = Inf)

write.csv(valid_all, "validacao_gradiente_HeckmanSK_detalhada.csv", row.names = FALSE)
write.csv(resumo, "validacao_gradiente_HeckmanSK_resumo.csv", row.names = FALSE)

saveRDS(
  list(
    fit_sk = fit_sk,
    theta_hat = theta_hat,
    validacao_detalhada = valid_all,
    validacao_resumo = resumo
  ),
  "validacao_gradiente_HeckmanSK.rds"
)

cat("\nArquivos gerados:\n")
cat("\n- validacao_gradiente_HeckmanSK_detalhada.csv")
cat("\n- validacao_gradiente_HeckmanSK_resumo.csv")
cat("\n- validacao_gradiente_HeckmanSK.rds\n")

# ============================================================
# Validação SEGURA do gradiente analítico da função HeckmanSK()
# Modelo: Selection skew-normal model de Ogundimu e Hutton
# Estratégia: comparar gradiente analítico corrigido com numDeriv::grad()
# Banco: MEPS2001
#
# Este script foi escrito para evitar falsos alarmes causados por pontos
# perturbados numericamente instáveis. A validação principal é feita em
# theta_hat; as perturbações são pequenas e aceitas apenas quando ficam
# em região numericamente regular da log-verossimilhança.
# ============================================================

rm(list = ls())

# ------------------------------------------------------------
# 1. Pacotes necessários
# ------------------------------------------------------------
# pkgs <- c("pkgload", "numDeriv", "sn", "dplyr", "tibble")
#
# for (p in pkgs) {
#   if (!requireNamespace(p, quietly = TRUE)) {
#     install.packages(p)
#   }
# }

library(pkgload)
library(numDeriv)
library(sn)
library(dplyr)
library(tibble)

# ------------------------------------------------------------
# 2. Carregar pacote local
#    O script funciona na raiz do pacote, em ssmodels/Teste ou
#    quando copiado para outro diretório indicando pkg_root manualmente.
# ------------------------------------------------------------
if (file.exists("DESCRIPTION")) {
  pkg_root <- "."
} else if (file.exists(file.path("..", "DESCRIPTION"))) {
  pkg_root <- ".."
} else if (file.exists(file.path("..", "..", "DESCRIPTION"))) {
  pkg_root <- file.path("..", "..")
} else {
  stop("Execute este script na raiz do pacote ssmodels, dentro de ssmodels/Teste ou dentro de inst/validation.")
}

pkgload::load_all(pkg_root, reset = TRUE, quiet = TRUE)

# ------------------------------------------------------------
# 3. Carregar MEPS2001
# ------------------------------------------------------------
if (exists("MEPS2001")) rm(MEPS2001)

data("MEPS2001", package = "ssmodels", envir = environment())
dados <- as.data.frame(MEPS2001)

dados <- dados |>
  mutate(
    dambexp_num = as.integer(dambexp == 1),
    lnambx_obs  = ifelse(dambexp_num == 1, lnambx, NA_real_)
  )

selectEq  <- dambexp_num ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx_obs  ~ age + female + educ + blhisp + totchr + ins

# ------------------------------------------------------------
# 4. Funções independentes de log-verossimilhança e gradiente
# ------------------------------------------------------------
.sk_components <- function(selection, outcome, data) {
  comp <- ssmodels::extract_model_components(selection = selection,
                                             outcome   = outcome,
                                             data      = data)
  comp$YO[is.na(comp$YO)] <- 0
  comp
}

.sk_loglik <- function(theta, comp) {
  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO
  NXS <- ncol(XS)
  NXO <- ncol(XO)

  istartS <- seq_len(NXS)
  istartO <- seq(from = NXS + 1L, length.out = NXO)
  isigma  <- NXS + NXO + 1L
  irho    <- NXS + NXO + 2L
  ilamb1  <- NXS + NXO + 3L

  gamma  <- theta[istartS]
  beta   <- theta[istartO]
  sigma  <- theta[isigma]
  rho    <- theta[irho]
  lambda <- theta[ilamb1]

  if (!is.finite(sigma) || sigma <= 0) return(NA_real_)
  if (!is.finite(rho) || abs(rho) >= 1) return(NA_real_)
  if (!is.finite(lambda)) return(NA_real_)

  eta <- as.numeric(XS %*% gamma)
  mu  <- as.numeric(XO %*% beta)
  z   <- (YO - mu) / sigma
  r   <- 1 - rho^2
  u   <- 1 + lambda^2 - (lambda * rho)^2
  q   <- lambda * rho / sqrt(u)

  omega <- (eta + rho * z) / sqrt(r)

  ll_selected <-
    log(2) - log(sigma) +
    stats::dnorm(z, log = TRUE) +
    stats::pnorm(lambda * z, log.p = TRUE) +
    stats::pnorm(omega, log.p = TRUE)

  F0 <- sn::psn(-eta, xi = 0, omega = 1, alpha = q)
  F0 <- pmax(F0, .Machine$double.xmin)
  ll_censored <- log(F0)

  out <- sum(ifelse(YS == 1, ll_selected, ll_censored))
  if (!is.finite(out)) return(NA_real_)
  out
}

.sk_grad <- function(theta, comp) {
  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO
  NXS <- ncol(XS)
  NXO <- ncol(XO)
  nObs <- length(YS)
  nParam <- NXS + NXO + 3L

  istartS <- seq_len(NXS)
  istartO <- seq(from = NXS + 1L, length.out = NXO)
  isigma  <- NXS + NXO + 1L
  irho    <- NXS + NXO + 2L
  ilamb1  <- NXS + NXO + 3L

  gamma  <- theta[istartS]
  beta   <- theta[istartO]
  sigma  <- theta[isigma]
  rho    <- theta[irho]
  lambda <- theta[ilamb1]

  if (!is.finite(sigma) || sigma <= 0) return(rep(NA_real_, nParam))
  if (!is.finite(rho) || abs(rho) >= 1) return(rep(NA_real_, nParam))
  if (!is.finite(lambda)) return(rep(NA_real_, nParam))

  sel0 <- YS == 0
  sel1 <- YS == 1

  XS0 <- XS[sel0, , drop = FALSE]
  XS1 <- XS[sel1, , drop = FALSE]
  XO1 <- XO[sel1, , drop = FALSE]
  YO1 <- YO[sel1]

  eta0 <- as.numeric(XS0 %*% gamma)
  eta1 <- as.numeric(XS1 %*% gamma)
  mu1  <- as.numeric(XO1 %*% beta)

  z <- (YO1 - mu1) / sigma
  r <- 1 - rho^2
  u <- 1 + lambda^2 - (lambda * rho)^2
  q <- lambda * rho / sqrt(u)

  omega <- (eta1 + rho * z) / sqrt(r)
  a     <- lambda * z

  K1 <- exp(stats::dnorm(omega, log = TRUE) -
              stats::pnorm(omega, log.p = TRUE))
  K3 <- exp(stats::dnorm(a, log = TRUE) -
              stats::pnorm(a, log.p = TRUE))

  F0 <- sn::psn(-eta0, xi = 0, omega = 1, alpha = q)
  F0 <- pmax(F0, .Machine$double.xmin)

  K2 <- exp(stats::dnorm(eta0, log = TRUE) +
              stats::pnorm(-q * eta0, log.p = TRUE) -
              log(F0))

  K4 <- exp(stats::dnorm(eta0 * sqrt(1 + lambda^2) / sqrt(u), log = TRUE) -
              log(F0))

  G <- matrix(0, nrow = nObs, ncol = nParam)

  if (nrow(XS1) > 0L) {
    G[sel1, istartS] <- XS1 * as.numeric(K1 / sqrt(r))
  }
  if (nrow(XS0) > 0L) {
    G[sel0, istartS] <- XS0 * as.numeric(-2 * K2)
  }

  if (nrow(XO1) > 0L) {
    G[sel1, istartO] <- XO1 * as.numeric(
      z / sigma -
        lambda * K3 / sigma -
        rho * K1 / (sigma * sqrt(r))
    )
  }

  if (sum(sel1) > 0L) {
    G[sel1, isigma] <-
      -1 / sigma +
      z^2 / sigma -
      lambda * z * K3 / sigma -
      rho * z * K1 / (sigma * sqrt(r))

    G[sel1, irho] <-
      K1 * (z + rho * eta1) / (r^(3 / 2))

    G[sel1, ilamb1] <-
      K3 * z
  }

  if (sum(sel0) > 0L) {
    G[sel0, irho] <-
      -2 * lambda * K4 / sqrt(2 * pi * u)

    G[sel0, ilamb1] <-
      -2 * rho * K4 / ((1 + lambda^2) * sqrt(2 * pi * u))
  }

  colSums(G)
}

# Diagnóstico simples para rejeitar pontos com alta chance de instabilidade
# numérica antes da chamada a numDeriv::grad().
.sk_is_safe_theta <- function(theta, comp) {
  XS <- comp$XS
  XO <- comp$XO
  YS <- comp$YS
  YO <- comp$YO
  NXS <- ncol(XS)
  NXO <- ncol(XO)

  sigma <- theta[NXS + NXO + 1L]
  rho   <- theta[NXS + NXO + 2L]
  lambda <- theta[NXS + NXO + 3L]

  if (!is.finite(sigma) || sigma <= 0.10) return(FALSE)
  if (!is.finite(rho) || abs(rho) >= 0.90) return(FALSE)
  if (!is.finite(lambda) || abs(lambda) > 5) return(FALSE)

  ll <- .sk_loglik(theta, comp)
  if (!is.finite(ll)) return(FALSE)

  gamma <- theta[seq_len(NXS)]
  beta  <- theta[seq(from = NXS + 1L, length.out = NXO)]
  eta <- as.numeric(XS %*% gamma)
  mu  <- as.numeric(XO %*% beta)
  z   <- (YO - mu) / sigma
  r   <- 1 - rho^2
  u   <- 1 + lambda^2 - (lambda * rho)^2
  q   <- lambda * rho / sqrt(u)
  omega <- (eta + rho * z) / sqrt(r)

  # Evita pontos nos quais pequenas perturbações podem levar a probabilidade
  # numericamente nula em alguma das componentes da log-verossimilhança.
  log_p1 <- stats::pnorm(lambda * z[YS == 1], log.p = TRUE)
  log_p2 <- stats::pnorm(omega[YS == 1], log.p = TRUE)
  F0 <- sn::psn(-eta[YS == 0], xi = 0, omega = 1, alpha = q)
  log_p0 <- log(pmax(F0, .Machine$double.xmin))

  min_log_prob <- min(c(log_p1, log_p2, log_p0), na.rm = TRUE)
  is.finite(min_log_prob) && min_log_prob > -650
}

.validar_um_ponto <- function(theta, comp, nome = "theta") {
  if (!.sk_is_safe_theta(theta, comp)) {
    return(tibble(
      ponto = nome,
      parametro = names(theta),
      analitico = NA_real_,
      numerico = NA_real_,
      diferenca = NA_real_,
      erro_abs = NA_real_,
      erro_rel = NA_real_,
      status = "descartado_regiao_instavel"
    ))
  }

  g_ana <- .sk_grad(theta, comp)
  g_num <- numDeriv::grad(func = .sk_loglik, x = theta, comp = comp,
                          method = "Richardson")

  status <- if (all(is.finite(g_ana)) && all(is.finite(g_num))) {
    "avaliado"
  } else {
    "gradiente_nao_finito"
  }

  tibble(
    ponto = nome,
    parametro = names(theta),
    analitico = as.numeric(g_ana),
    numerico = as.numeric(g_num),
    diferenca = analitico - numerico,
    erro_abs = abs(diferenca),
    erro_rel = abs(diferenca) / (1 + abs(numerico)),
    status = status
  )
}

# ------------------------------------------------------------
# 5. Ajuste do modelo e validação no máximo encontrado
# ------------------------------------------------------------
fit_sk <- ssmodels::HeckmanSK(selection = selectEq,
                              outcome   = outcomeEq,
                              data      = dados)

comp <- .sk_components(selectEq, outcomeEq, dados)
theta_hat <- fit_sk$coefficients

cat("\nLogLik HeckmanSK(): ", fit_sk$loglik, "\n")
cat("LogLik função independente: ", .sk_loglik(theta_hat, comp), "\n")

valid_hat <- .validar_um_ponto(theta_hat, comp, nome = "theta_hat")

cat("\nResumo da validação em theta_hat:\n")
print(valid_hat, n = Inf)
cat("\nMáximo erro absoluto em theta_hat:", max(valid_hat$erro_abs, na.rm = TRUE), "\n")
cat("Máximo erro relativo em theta_hat :", max(valid_hat$erro_rel, na.rm = TRUE), "\n")

# ------------------------------------------------------------
# 6. Validação em pontos perturbados seguros
# ------------------------------------------------------------
set.seed(123)
B <- 5L
NXS <- comp$NXS
NXO <- comp$NXO
isigma <- NXS + NXO + 1L
irho   <- NXS + NXO + 2L
ilamb  <- NXS + NXO + 3L

valid_list <- list(theta_hat = valid_hat)
accepted <- 0L
attempts <- 0L
max_attempts <- 200L
sd_perturb <- 0.005

while (accepted < B && attempts < max_attempts) {
  attempts <- attempts + 1L

  theta_b <- theta_hat + stats::rnorm(length(theta_hat), mean = 0, sd = sd_perturb)
  theta_b[isigma] <- max(0.2, theta_b[isigma])
  theta_b[irho]   <- max(-0.80, min(0.80, theta_b[irho]))
  theta_b[ilamb]  <- max(-4, min(4, theta_b[ilamb]))
  names(theta_b)  <- names(theta_hat)

  if (!.sk_is_safe_theta(theta_b, comp)) next

  accepted <- accepted + 1L
  valid_list[[paste0("perturbado_seguro_", accepted)]] <-
    .validar_um_ponto(theta_b, comp, nome = paste0("perturbado_seguro_", accepted))
}

if (accepted < B) {
  warning("Foram aceitos apenas ", accepted, " pontos perturbados seguros em ", attempts, " tentativas.")
}

valid_all <- dplyr::bind_rows(valid_list)

resumo <- valid_all |>
  group_by(ponto, status) |>
  summarise(
    max_erro_abs = max(erro_abs, na.rm = TRUE),
    max_erro_rel = max(erro_rel, na.rm = TRUE),
    .groups = "drop"
  )

cat("\nResumo geral:\n")
print(resumo, n = Inf)
cat("\nPontos perturbados seguros aceitos:", accepted, "de", attempts, "tentativas.\n")

# write.csv(valid_all, "validacao_gradiente_HeckmanSK_detalhada.csv", row.names = FALSE)
# write.csv(resumo, "validacao_gradiente_HeckmanSK_resumo.csv", row.names = FALSE)
#
# saveRDS(
#   list(
#     fit_sk = fit_sk,
#     theta_hat = theta_hat,
#     validacao_detalhada = valid_all,
#     validacao_resumo = resumo,
#     pontos_perturbados_aceitos = accepted,
#     tentativas = attempts
#   ),
#   "validacao_gradiente_HeckmanSK.rds"
# )
#
# cat("\nArquivos gerados:\n")
# cat("\n- validacao_gradiente_HeckmanSK_detalhada.csv")
# cat("\n- validacao_gradiente_HeckmanSK_resumo.csv")
# cat("\n- validacao_gradiente_HeckmanSK.rds\n")

# ============================================================
# Funcoes independentes para validacao do modelo Heckman classico
# Pacote: ssmodels
# Objetivo: comparar gradiente analitico contra numDeriv::grad()
# ============================================================

.cl_safe_ratio <- function(x) {
  exp(stats::dnorm(x, log = TRUE) - stats::pnorm(x, log.p = TRUE))
}

.cl_extract_components_independente <- function(selection, outcome, data) {
  if (exists("extract_model_components", mode = "function")) {
    return(extract_model_components(selection = selection, outcome = outcome, data = data))
  }

  mf_s <- stats::model.frame(selection, data = data, na.action = stats::na.pass)
  YS_raw <- stats::model.response(mf_s)
  XS <- stats::model.matrix(selection, data = mf_s)

  if (is.factor(YS_raw)) {
    lev <- levels(YS_raw)
    if (length(lev) != 2L) stop("A variavel de selecao deve possuir dois niveis.")
    YS <- as.integer(YS_raw == lev[2L])
  } else {
    vals <- sort(unique(stats::na.omit(YS_raw)))
    if (!all(vals %in% c(0, 1))) {
      stop("A variavel de selecao deve ser binaria, codificada como 0/1, ou fator com dois niveis.")
    }
    lev <- vals
    YS <- as.integer(YS_raw)
  }

  mf_o <- stats::model.frame(outcome, data = data, na.action = stats::na.pass)
  YO <- stats::model.response(mf_o)
  XO <- stats::model.matrix(outcome, data = mf_o)

  keep <- !is.na(YS) &
    stats::complete.cases(XS) &
    stats::complete.cases(XO) &
    (YS == 0L | !is.na(YO))

  XS <- XS[keep, , drop = FALSE]
  XO <- XO[keep, , drop = FALSE]
  YS <- YS[keep]
  YO <- YO[keep]

  list(
    XS = XS,
    YS = YS,
    NXS = ncol(XS),
    XO = XO,
    YO = YO,
    NXO = ncol(XO),
    YSLevels = lev
  )
}

heckmanCL_prepare <- function(selection, outcome, data) {
  .cl_extract_components_independente(selection, outcome, data)
}

heckmanCL_loglik_independente <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanCL_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  if (any(!is.finite(par))) return(NA_real_)

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  isigma <- max(ibeta) + 1L
  irho   <- isigma + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  sigma <- par[isigma]
  rho   <- par[irho]

  if (!is.finite(sigma) || sigma <= 0) return(NA_real_)
  if (!is.finite(rho) || abs(rho) >= 1) return(NA_real_)

  r <- sqrt(1 - rho^2)
  if (!is.finite(r) || r <= 1e-8) return(NA_real_)

  XS0 <- XS[YS == 0L, , drop = FALSE]
  XS1 <- XS[YS == 1L, , drop = FALSE]
  XO1 <- XO[YS == 1L, , drop = FALSE]
  YO1 <- YO[YS == 1L]

  eta0 <- as.vector(XS0 %*% gamma)
  eta1 <- as.vector(XS1 %*% gamma)
  mu1  <- as.vector(XO1 %*% beta)
  z1   <- (YO1 - mu1) / sigma

  B1 <- (eta1 + rho * z1) / r

  ll0 <- stats::pnorm(-eta0, log.p = TRUE)
  ll1 <- stats::dnorm(z1, log = TRUE) - log(sigma) + stats::pnorm(B1, log.p = TRUE)

  ll <- sum(ll0) + sum(ll1)
  if (!is.finite(ll)) return(NA_real_)
  ll
}

heckmanCL_score_independente <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanCL_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  if (any(!is.finite(par))) return(rep(NA_real_, length(par)))

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  isigma <- max(ibeta) + 1L
  irho   <- isigma + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  sigma <- par[isigma]
  rho   <- par[irho]

  if (!is.finite(sigma) || sigma <= 0) return(rep(NA_real_, length(par)))
  if (!is.finite(rho) || abs(rho) >= 1) return(rep(NA_real_, length(par)))

  r <- sqrt(1 - rho^2)
  if (!is.finite(r) || r <= 1e-8) return(rep(NA_real_, length(par)))

  XS0 <- XS[YS == 0L, , drop = FALSE]
  XS1 <- XS[YS == 1L, , drop = FALSE]
  XO1 <- XO[YS == 1L, , drop = FALSE]
  YO1 <- YO[YS == 1L]

  eta0 <- as.vector(XS0 %*% gamma)
  eta1 <- as.vector(XS1 %*% gamma)
  mu1  <- as.vector(XO1 %*% beta)
  u1   <- YO1 - mu1
  z1   <- u1 / sigma
  B1   <- (eta1 + rho * z1) / r

  lambda0 <- .cl_safe_ratio(-eta0)
  lambdaB <- .cl_safe_ratio(B1)

  grad <- numeric(length(par))

  if (nrow(XS0) > 0L) {
    grad[igamma] <- grad[igamma] - colSums(sweep(XS0, 1, lambda0, `*`))
  }

  if (nrow(XS1) > 0L) {
    grad[igamma] <- grad[igamma] + colSums(sweep(XS1, 1, lambdaB / r, `*`))

    grad[ibeta] <- grad[ibeta] +
      colSums(sweep(XO1, 1, z1 / sigma - rho * lambdaB / (sigma * r), `*`))

    grad[isigma] <- grad[isigma] +
      sum(z1^2 / sigma - 1 / sigma - rho * z1 * lambdaB / (sigma * r))

    grad[irho] <- grad[irho] +
      sum(lambdaB * (z1 + rho * eta1) / r^3)
  }

  names(grad) <- names(par)
  grad
}

heckmanCL_density_components <- function(par, selection, outcome, data, components = NULL) {
  if (is.null(components)) components <- heckmanCL_prepare(selection, outcome, data)

  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO

  igamma <- seq_len(NXS)
  ibeta  <- seq.int(max(igamma) + 1L, length.out = NXO)
  isigma <- max(ibeta) + 1L
  irho   <- isigma + 1L

  gamma <- par[igamma]
  beta  <- par[ibeta]
  sigma <- par[isigma]
  rho   <- par[irho]
  r <- sqrt(1 - rho^2)

  out <- data.frame(
    row = seq_along(YS),
    selected = YS,
    value = NA_real_,
    log_contribution = NA_real_,
    type = ifelse(YS == 1L, "selected_density", "censored_probability")
  )

  if (any(YS == 1L)) {
    idx1 <- which(YS == 1L)
    eta1 <- as.vector(XS[idx1, , drop = FALSE] %*% gamma)
    mu1  <- as.vector(XO[idx1, , drop = FALSE] %*% beta)
    z1   <- (YO[idx1] - mu1) / sigma
    B1   <- (eta1 + rho * z1) / r
    logc <- stats::dnorm(z1, log = TRUE) - log(sigma) + stats::pnorm(B1, log.p = TRUE)
    out$value[idx1] <- exp(logc)
    out$log_contribution[idx1] <- logc
  }

  if (any(YS == 0L)) {
    idx0 <- which(YS == 0L)
    eta0 <- as.vector(XS[idx0, , drop = FALSE] %*% gamma)
    logd <- stats::pnorm(-eta0, log.p = TRUE)
    out$value[idx0] <- exp(logd)
    out$log_contribution[idx0] <- logd
  }

  out
}

# ============================================================
# Validação do gradiente do HeckmanGe/HeckmanGE com numDeriv
# Banco: MEPS2001
# ============================================================

rm(list = ls())

find_pkg_root <- function() {
  candidates <- c(".", "..", "../..", "../../..")
  for (cc in candidates) {
    if (file.exists(file.path(cc, "DESCRIPTION")) && dir.exists(file.path(cc, "R"))) {
      return(normalizePath(cc, winslash = "/"))
    }
  }
  stop("Não encontrei a raiz do pacote. Execute o script na raiz do pacote, em Teste, ou em inst/validation.")
}

pkg_root <- find_pkg_root()

pkgs <- c("pkgload", "numDeriv", "dplyr", "tibble")
for (p in pkgs) {
  if (!requireNamespace(p, quietly = TRUE)) {
    install.packages(p)
  }
}

suppressPackageStartupMessages({
  library(dplyr)
  library(tibble)
})

pkgload::load_all(pkg_root, reset = TRUE, quiet = TRUE)

source_file <- file.path(pkg_root, "inst", "validation", "HeckmanGE_loglik_score_independente.R")
if (!file.exists(source_file)) {
  source_file <- "HeckmanGE_loglik_score_independente.R"
}
source(source_file)

# ------------------------------------------------------------
# Carrega MEPS2001
# ------------------------------------------------------------
ok_data <- tryCatch({
  data("MEPS2001", package = "ssmodels", envir = environment())
  exists("MEPS2001")
}, error = function(e) FALSE)

if (!ok_data) {
  candidate_data <- c(
    file.path(pkg_root, "data", "MEPS2001.rda"),
    file.path(pkg_root, "data", "MEPS2001.RData"),
    file.path("data", "MEPS2001.rda"),
    file.path("..", "data", "MEPS2001.rda")
  )
  candidate_data <- candidate_data[file.exists(candidate_data)]
  if (length(candidate_data) == 0L) stop("Não encontrei o banco MEPS2001.")
  load(candidate_data[1])
}

dados <- as.data.frame(MEPS2001)

dados <- dados |>
  mutate(
    dambexp_num = as.integer(dambexp == 1),
    lnambx_obs = ifelse(dambexp_num == 1, lnambx, NA_real_)
  )

# ------------------------------------------------------------
# Fórmulas: exemplo recomendado pelo pacote heckmanGE
# ------------------------------------------------------------
selectEq <- dambexp_num ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx_obs ~ age + female + educ + blhisp + totchr + ins
dispersionEq <- ~ age + female + totchr + ins
correlationEq <- ~ age

# ------------------------------------------------------------
# Ajuste pelo ssmodels::HeckmanGe
# ------------------------------------------------------------
fit_ge <- ssmodels::HeckmanGe(
  selection = selectEq,
  outcome = outcomeEq,
  outcomeS = dispersionEq,
  outcomeC = correlationEq,
  data = dados
)

summary(fit_ge)

# ------------------------------------------------------------
# Componentes independentes
# ------------------------------------------------------------
comp <- heckmanGE_make_components(
  selection = selectEq,
  outcome = outcomeEq,
  dispersion = dispersionEq,
  correlation = correlationEq,
  data = dados
)

idx <- heckmanGE_parameter_index(comp$XS, comp$XO, comp$Msigma, comp$Mrho)

theta_hat <- if (!is.null(fit_ge$coefficients_link)) {
  as.numeric(fit_ge$coefficients_link)
} else {
  as.numeric(fit_ge$coefficients)
}

names(theta_hat) <- idx$parameter_names

ll_fit <- fit_ge$loglik
ll_ind <- heckmanGE_loglik_indep(
  theta = theta_hat,
  YS = comp$YS,
  XS = comp$XS,
  YO = comp$YO,
  XO = comp$XO,
  Msigma = comp$Msigma,
  Mrho = comp$Mrho,
  w = comp$w
)

cat("\nLogLik HeckmanGe(): ", ll_fit, "\n")
cat("LogLik função independente: ", ll_ind, "\n")
cat("Diferença: ", ll_ind - ll_fit, "\n")

# ------------------------------------------------------------
# Validação no estimador
# ------------------------------------------------------------
res_hat <- heckmanGE_validate_gradient(
  theta = theta_hat,
  YS = comp$YS,
  XS = comp$XS,
  YO = comp$YO,
  XO = comp$XO,
  Msigma = comp$Msigma,
  Mrho = comp$Mrho,
  w = comp$w,
  ponto = "theta_hat"
)

cat("\nResumo da validação em theta_hat:\n")
print(tibble::as_tibble(res_hat), n = Inf)

cat("\nMáximo erro absoluto em theta_hat: ", max(res_hat$erro_abs, na.rm = TRUE), "\n")
cat("Máximo erro relativo em theta_hat : ", max(res_hat$erro_rel, na.rm = TRUE), "\n")

# ------------------------------------------------------------
# Perturbações seguras
# ------------------------------------------------------------
set.seed(123)
res_list <- list(theta_hat = res_hat)
accepted <- 0L
attempt <- 0L

loglik0 <- function(par) {
  heckmanGE_loglik_indep(
    theta = par,
    YS = comp$YS,
    XS = comp$XS,
    YO = comp$YO,
    XO = comp$XO,
    Msigma = comp$Msigma,
    Mrho = comp$Mrho,
    w = comp$w
  )
}

while (accepted < 5L && attempt < 100L) {
  attempt <- attempt + 1L
  theta_pert <- theta_hat + stats::rnorm(length(theta_hat), sd = 0.01)
  if (!is.finite(loglik0(theta_pert))) next

  accepted <- accepted + 1L
  res_list[[paste0("perturbado_", accepted)]] <- heckmanGE_validate_gradient(
    theta = theta_pert,
    YS = comp$YS,
    XS = comp$XS,
    YO = comp$YO,
    XO = comp$XO,
    Msigma = comp$Msigma,
    Mrho = comp$Mrho,
    w = comp$w,
    ponto = paste0("perturbado_", accepted)
  )
}

res_all <- dplyr::bind_rows(res_list)
res_summary <- res_all |>
  group_by(ponto) |>
  summarise(
    max_erro_abs = max(erro_abs, na.rm = TRUE),
    max_erro_rel = max(erro_rel, na.rm = TRUE),
    .groups = "drop"
  )

cat("\nResumo geral:\n")
print(res_summary, n = Inf)

# ------------------------------------------------------------
# Comparação opcional com heckmanGE::estfun()
# ------------------------------------------------------------
comparacao_heckmanGE <- NULL
if (requireNamespace("heckmanGE", quietly = TRUE)) {
  fit_ref <- tryCatch(
    heckmanGE::heckmanGE(
      selection = selectEq,
      outcome = outcomeEq,
      dispersion = dispersionEq,
      correlation = correlationEq,
      data = dados
    ),
    error = function(e) e
  )

  if (!inherits(fit_ref, "error")) {
    g_ref <- colSums(heckmanGE::estfun(fit_ref))
    comparacao_heckmanGE <- tibble::tibble(
      parametro = names(g_ref),
      gradiente_heckmanGE_estfun = as.numeric(g_ref)
    )
    cat("\nGradiente agregado de heckmanGE::estfun() calculado com sucesso.\n")
  } else {
    cat("\nNão foi possível ajustar heckmanGE::heckmanGE(): ", conditionMessage(fit_ref), "\n")
  }
} else {
  cat("\nPacote heckmanGE não instalado; comparação opcional com estfun() foi ignorada.\n")
}

# ------------------------------------------------------------
# Salva arquivos
# ------------------------------------------------------------
utils::write.csv(res_all, "validacao_gradiente_HeckmanGE_detalhada.csv", row.names = FALSE)
utils::write.csv(res_summary, "validacao_gradiente_HeckmanGE_resumo.csv", row.names = FALSE)

if (!is.null(comparacao_heckmanGE)) {
  utils::write.csv(comparacao_heckmanGE, "comparacao_gradiente_heckmanGE_estfun.csv", row.names = FALSE)
}

saveRDS(
  list(
    fit_ge = fit_ge,
    theta_hat = theta_hat,
    logLik_fit = ll_fit,
    logLik_indep = ll_ind,
    componentes = comp,
    validacao_detalhada = res_all,
    validacao_resumo = res_summary,
    comparacao_heckmanGE = comparacao_heckmanGE
  ),
  "validacao_gradiente_HeckmanGE.rds"
)

cat("\nArquivos gerados:\n")
cat("\n- validacao_gradiente_HeckmanGE_detalhada.csv")
cat("\n- validacao_gradiente_HeckmanGE_resumo.csv")
cat("\n- validacao_gradiente_HeckmanGE.rds")
if (!is.null(comparacao_heckmanGE)) cat("\n- comparacao_gradiente_heckmanGE_estfun.csv")
cat("\n")

#' Inverse Mills Ratio
#'
#' @description
#' Computes the column vector of the Inverse Mills Ratio (IMR)
#' from a Probit selection equation.
#'
#' @param selection A formula for the selection equation.
#' @param data A data frame containing the variables.
#'
#' @return
#' A column vector (matrix with one column) containing the Inverse Mills Ratio
#' computed from the Probit model fitted to the selection equation.
#'
#' @examples
#' data(MEPS2001)
#' attach(MEPS2001)
#' selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
#' IMR(selectEq, data = MEPS2001)
#'
#' @export
IMR <- function(selection, data = sys.frame(sys.parent())) {

  # Step 1: Build model frame from selection equation
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("selection", "data", "subset"), names(mf), 0)
  mfS <- mf[c(1, m)]
  mfS$drop.unused.levels <- TRUE
  mfS$na.action <- na.pass
  mfS[[1]] <- as.name("model.frame")
  names(mfS)[2] <- "formula"
  mfS <- eval(mfS, parent.frame())

  # Step 2: Extract design matrix and response vector
  mtS <- terms(mfS)
  XS <- model.matrix(mtS, mfS)
  YS <- model.response(mfS)

  # Step 3: Fit Probit model to selection equation
  fit1 <- glm(YS ~ XS - 1, family = binomial(link = "probit"))

  # Step 4: Compute Inverse Mills Ratio (IMR)
  imr <- dnorm(fit1$linear.predictors) / pnorm(fit1$linear.predictors)

  # Step 5: Return IMR as column matrix
  return(cbind(imr))
}

#' Generalized Heckman Model Estimation
#'
#' @description
#' Estimates the parameters of a generalized Heckman selection model,
#' allowing covariates in the scale and correlation components.
#'
#' @param selection Formula for the selection equation.
#' @param outcome Formula for the outcome equation.
#' @param outcomeS Covariates for the log-scale model. It may be a matrix,
#'   vector, one-sided formula, or 1 for an intercept-only scale model.
#' @param outcomeC Covariates for the correlation model. It may be a matrix,
#'   vector, one-sided formula, or 1 for an intercept-only correlation model.
#' @param data Data frame containing all variables.
#' @param start Optional starting values. If supplied on the natural classical
#'   scale, sigma and rho are internally mapped to log(sigma) and atanh(rho).
#'
#' @return A list with the fitted model parameters and diagnostics.
#' @export
HeckmanGe <- function(selection, outcome, outcomeS = 1, outcomeC = 1,
                      data = sys.frame(sys.parent()), start = NULL) {

  components <- extract_model_components(selection = selection,
                                         outcome = outcome,
                                         outcomeS = outcomeS,
                                         outcomeC = outcomeC,
                                         data = data)
  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO
  Msigma <- components$Msigma
  NE <- components$NE
  Mrho <- components$Mrho
  NV <- components$NV
  YSLevels <- components$YSLevels

  YO_clean <- ifelse(is.na(YO), 0, YO)

  loglik_gen <- function(par) {
    if (any(!is.finite(par))) return(NA_real_)

    istartS <- seq_len(NXS)
    istartO <- seq.int(NXS + 1L, NXS + NXO)
    ilambda <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
    ikappa  <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)

    gamma <- par[istartS]
    beta <- par[istartO]
    lambda <- par[ilambda]
    kappa <- par[ikappa]

    mu2 <- drop(XS %*% gamma)
    mu1 <- drop(XO %*% beta)
    sigma <- as.numeric(exp(Msigma %*% lambda))
    rho <- as.numeric(tanh(Mrho %*% kappa))

    r <- sqrt(pmax(1 - rho^2, .Machine$double.eps))
    z <- (YO_clean - mu1) / sigma
    zeta <- (mu2 + rho * z) / r

    ll0 <- stats::pnorm(-mu2, log.p = TRUE)
    ll1 <- stats::dnorm(z, log = TRUE) - log(sigma) +
      stats::pnorm(zeta, log.p = TRUE)

    sum(ifelse(YS == 0L, ll0, ll1))
  }

  gradlik_gen <- function(par) {
    istartS <- seq_len(NXS)
    istartO <- seq.int(NXS + 1L, NXS + NXO)
    ilambda <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
    ikappa  <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)
    nParam <- NXS + NXO + NE + NV

    if (any(!is.finite(par))) return(rep(NA_real_, nParam))

    gamma <- par[istartS]
    beta <- par[istartO]
    lambda <- par[ilambda]
    kappa <- par[ikappa]

    XS0 <- XS[YS == 0L, , drop = FALSE]
    XS1 <- XS[YS == 1L, , drop = FALSE]
    XO1 <- XO[YS == 1L, , drop = FALSE]
    YO1 <- YO_clean[YS == 1L]
    MS1 <- Msigma[YS == 1L, , drop = FALSE]
    MR1 <- Mrho[YS == 1L, , drop = FALSE]

    mu20 <- drop(XS0 %*% gamma)
    mu21 <- drop(XS1 %*% gamma)
    mu11 <- drop(XO1 %*% beta)

    sigma1 <- as.numeric(exp(MS1 %*% lambda))
    rho1 <- as.numeric(tanh(MR1 %*% kappa))
    r1 <- sqrt(pmax(1 - rho1^2, .Machine$double.eps))
    z1 <- (YO1 - mu11) / sigma1
    zeta1 <- (mu21 + rho1 * z1) / r1

    lambda0 <- exp(stats::dnorm(-mu20, log = TRUE) -
                     stats::pnorm(-mu20, log.p = TRUE))
    lambda1 <- exp(stats::dnorm(zeta1, log = TRUE) -
                     stats::pnorm(zeta1, log.p = TRUE))

    gradient <- matrix(0, nrow = length(YS), ncol = nParam)
    gradient[YS == 0L, istartS] <- -XS0 * lambda0
    gradient[YS == 1L, istartS] <- XS1 * (lambda1 / r1)
    gradient[YS == 1L, istartO] <- XO1 *
      (z1 / sigma1 - lambda1 * rho1 / (sigma1 * r1))
    gradient[YS == 1L, ilambda] <- MS1 *
      (z1^2 - 1 - lambda1 * rho1 * z1 / r1)
    gradient[YS == 1L, ikappa] <- MR1 *
      (lambda1 * (z1 + rho1 * mu21) / r1)

    colSums(gradient)
  }

  if (is.null(start)) {
    start_classic <- tryCatch(step2(YS, XS, YO, XO), error = function(e) NULL)
    if (is.null(start_classic) || any(!is.finite(start_classic))) {
      message("Two-step starting values failed; using neutral default start values.")
      start_classic <- c(rep(0, NXS + NXO), sigma = 1, rho = 0)
    }
  } else {
    start_classic <- start
  }

  sigma0 <- max(start_classic[NXS + NXO + 1L], .Machine$double.eps)
  rho0 <- .ssmodels_clip_rho(start_classic[NXS + NXO + 2L])

  lambda_start <- c(log(sigma0), rep(0, max(NE - 1L, 0L)))
  kappa_start <- c(atanh(rho0), rep(0, max(NV - 1L, 0L)))

  start_opt <- c(start_classic[seq_len(NXS + NXO)], lambda_start, kappa_start)

  sigma_names <- if (NE == 1L) "sigma" else paste0("log.sigma.", colnames(Msigma))
  rho_names <- if (NV == 1L) "rho" else paste0("atanh.rho.", colnames(Mrho))
  names(start_opt) <- c(colnames(XS), colnames(XO), sigma_names, rho_names)

  theta_HG <- stats::optim(start_opt, loglik_gen, gradlik_gen,
                           method = "BFGS", hessian = TRUE,
                           control = list(fnscale = -1, maxit = 1000))

  vcov_link <- .ssmodels_safe_vcov(theta_HG$hessian)
  se_link <- .ssmodels_safe_se(vcov_link)
  coef_out <- theta_HG$par
  se_out <- se_link

  ilambda <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
  ikappa  <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)

  if (NE == 1L) {
    coef_out[ilambda] <- exp(theta_HG$par[ilambda])
    se_out[ilambda] <- abs(coef_out[ilambda]) * se_link[ilambda]
  }
  if (NV == 1L) {
    coef_out[ikappa] <- tanh(theta_HG$par[ikappa])
    se_out[ikappa] <- (1 - coef_out[ikappa]^2) * se_link[ikappa]
  }
  names(coef_out) <- names(start_opt)
  names(se_out) <- names(start_opt)

  nObs <- length(YS)
  nParam <- length(start_opt)
  result <- list(
    coefficients   = coef_out,
    coefficients_link = theta_HG$par,
    value          = theta_HG$value,
    loglik         = theta_HG$value,
    counts         = theta_HG$counts[2],
    hessian        = theta_HG$hessian,
    fisher_infoHG  = vcov_link,
    prop_sigmaHG   = se_out,
    level          = YSLevels,
    nObs           = nObs,
    nParam         = nParam,
    N0             = sum(YS == 0L),
    N1             = sum(YS == 1L),
    NXS            = NXS,
    NXO            = NXO,
    df             = nObs - nParam,
    aic            = -2 * theta_HG$value + 2 * nParam,
    bic            = -2 * theta_HG$value + nParam * log(nObs),
    initial.value  = start_opt,
    NE             = NE,
    NV             = NV,
    Msigma         = Msigma,
    Mrho           = Mrho
  )

  class(result) <- c("HeckmanGe", class(theta_HG))
  result
}

#' Summary of Heckman-t Model
#'
#' @description
#' Print estimates of the parameters of the Heckman-t model using Maximum Likelihood Estimation.
#'
#' @param object An object of class \code{HeckmantS}.
#' @param ... Additional arguments (currently unused).
#'
#' @return
#' Prints the summary output including coefficient tables and model fit statistics.
#'
#' @export summary.HeckmantS
#' @export
summary.HeckmantS <- function(object, ...) {
  # Extract components from the HeckmantS object
  fisher_infotS <- object$fisher_infotS       # Fisher information matrix (not printed)
  prop_sigmatS  <- object$prop_sigmatS        # Standard errors of the estimates
  coeffstS      <- object$coefficients        # Estimated coefficients
  counts        <- object$counts              # Number of iterations in optimization
  value         <- object$value               # Log-likelihood at convergence
  loglik        <- object$loglik              # Log-likelihood function
  NObs          <- object$nObs                # Total number of observations
  nParam        <- object$nParam              # Number of free parameters
  df            <- object$df                  # Degrees of freedom
  NXS           <- object$NXS                 # Number of covariates in selection equation
  NXO           <- object$NXO                 # Number of covariates in outcome equation
  N0            <- object$N0                  # Number of censored observations
  N1            <- object$N1                  # Number of observed (uncensored) observations
  aic           <- object$aic                 # Akaike Information Criterion
  bic           <- object$bic                 # Bayesian Information Criterion

  # Generate coefficient tables
  tb1 <- miscTools::coefTable(coeffstS[1:NXS], prop_sigmatS[1:NXS], df = df)                         # Selection equation
  tb2 <- miscTools::coefTable(coeffstS[(NXS + 1):(NXS + NXO)], prop_sigmatS[(NXS + 1):(NXS + NXO)], df = df)  # Outcome equation
  tb3 <- miscTools::coefTable(coeffstS[(NXS + NXO + 1):(NXS + NXO + 3)], prop_sigmatS[(NXS + NXO + 1):(NXS + NXO + 3)], df = df)  # Error and df terms
  tb  <- miscTools::coefTable(coeffstS, prop_sigmatS, df = df)  # Full model (not printed but kept)

  # Print summary output
  cat("\n")
  cat("--------------------------------------------------------------\n")
  cat("      t-Student Heckman Model (Package: ssmodels)             \n")
  cat("--------------------------------------------------------------\n")
  cat("--------------------------------------------------------------\n")
  cat("Maximum Likelihood estimation \n")
  cat("optim function with method BFGS - iterations number:", counts, "\n")
  cat("Log-Likelihood:", loglik, "\n")
  cat("AIC:", aic, "BIC:", bic, "\n")
  cat("Number of observations:", NObs, "(", N0, "censored and", N1, "observed )\n")
  cat(nParam, "free parameters ( df =", df, ")\n")
  cat("--------------------------------------------------------------\n")

  cat("Probit selection equation:\n")
  printCoefmat(tb1, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  cat("Outcome equation:\n")
  printCoefmat(tb2, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  cat("Error terms:\n")
  printCoefmat(tb3, signif.stars = TRUE, signif.legend = TRUE, digits = 4)
  cat("--------------------------------------------------------------\n")
}

#' Classic Heckman Model fit Function
#'
#' @description
#' Estimates the parameters of the classic Heckman model
#' via Maximum Likelihood method. The initial start is obtained
#' via the two-step method.
#'
#' @return
#' Returns a list with the following components.
#'
#' Coefficients: Returns a numerical vector with the best estimated values
#' of the model parameters;
#'
#' Value: The value of function to be minimized (or maximized) corresponding
#' to par.
#'
#' loglik: Maximized value of the log-likelihood function calculated from the estimated coefficients.
#'
#' counts: Component of the Optim function. A two-element integer vector
#' giving the number of calls to fn and gr respectively. This excludes
#' those calls needed to compute the Hessian, if requested, and any calls
#' to fn to compute a finite-difference approximation to the gradient.
#'
#' hessian: Component of the Optim function, with pre-defined option
#' hessian=TRUE. A symmetric matrix giving an estimate of the Hessian
#' at the solution found. Note that this is the Hessian of the unconstrained
#' problem even if the box constraints are active.
#'
#' fisher_infoHC: Fisher information matrix
#'
#' prop_sigmaHC: Square root of the Fisher information matrix diagonal
#'
#' level: Selection variable levels
#'
#' nObs: Numeric value representing the size of the database
#'
#' nParam: Numerical value representing the number of model parameters
#'
#' N0: Numerical value representing the number of unobserved entries
#'
#' N1: Numerical value representing the number of complete entries
#'
#' NXS: Numerical value representing the number of parameters of the
#' selection model
#'
#' NXO: Numerical value representing the number of parameters of the
#' regression model
#'
#' df: Numerical value that represents the difference between the size
#' of the response vector of the selection equation and the number of
#' model parameters
#'
#' aic: Numerical value representing Akaike's information criterion.
#'
#' bic: Numerical value representing Schwarz's Bayesian Criterion
#'
#' initial.value: Numerical vector that represents the input values
#' (Initial Values) used in the parameter estimation.
#'
#' @param selection Selection equation.
#' @param outcome Primary Regression Equation.
#' @param start initial values.
#' @param data Database.
#' @examples
#' data(MEPS2001)
#' attach(MEPS2001)
#' selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
#' outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins
#' HeckmanCL(selectEq, outcomeEq, data = MEPS2001)
#' @export HeckmanCL
#' @export
HeckmanCL <- function(selection, outcome, data = sys.frame(sys.parent()), start = NULL) {
  # Extract model components from selection and outcome formulas
  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  XS  <- components$XS       # Design matrix for selection equation
  YS  <- components$YS       # Response variable for selection equation
  NXS <- components$NXS      # Number of selection parameters
  XO  <- components$XO       # Design matrix for outcome equation
  YO  <- components$YO       # Response variable for outcome equation
  NXO <- components$NXO      # Number of outcome parameters
  YSLevels <- components$YSLevels  # Levels of the selection variable

  ######################################
  # Log-likelihood function definition
  ######################################
  loglik_HC <- function(start) {
    # Parameter indices
    istartS <- 1:NXS
    istartO <- seq(tail(istartS, 1) + 1, length.out = NXO)
    isigma  <- tail(istartO, 1) + 1
    irho    <- isigma + 1

    # Extract parameters
    g     <- start[istartS]
    b     <- start[istartO]
    sigma <- start[isigma]
    rho   <- start[irho]

    # Check parameter validity
    if (!is.finite(sigma) || sigma <= 0) return(NA)
    if (!is.finite(rho) || abs(rho) >= 1) return(NA)

    # Compute linear predictors
    XS.g     <- XS %*% g
    XO.b     <- XO %*% b
    YO_clean <- ifelse(is.na(YO), 0, YO)
    u2       <- YO_clean - XO.b
    r        <- sqrt(1 - rho^2)
    B        <- (XS.g + (rho / sigma) * u2) / r

    # Log-likelihood contributions
    ll <- ifelse(YS == 0,
                 pnorm(-XS.g, log.p = TRUE),
                 dnorm(u2 / sigma, log = TRUE) - log(sigma) + pnorm(B, log.p = TRUE))

    return(sum(ll))
  }

  ######################################
  # Gradient of the log-likelihood
  ######################################
  gradlik_HC <- function(start) {
    nObs <- length(YS)

    # Parameter indices
    istartS <- 1:NXS
    istartO <- seq(tail(istartS, 1) + 1, length.out = NXO)
    isigma  <- tail(istartO, 1) + 1
    irho    <- isigma + 1
    nParam  <- irho

    # Extract parameters
    g     <- start[istartS]
    b     <- start[istartO]
    sigma <- start[isigma]
    rho   <- start[irho]

    # Check parameter validity
    if (!is.finite(sigma) || sigma <= 0) return(rep(NA, nParam))
    if (!is.finite(rho) || abs(rho) >= 1) return(rep(NA, nParam))

    # Partition data based on selection indicator
    XS0 <- XS[YS == 0, , drop = FALSE]
    XS1 <- XS[YS == 1, , drop = FALSE]
    XO1 <- XO[YS == 1, , drop = FALSE]
    YO_clean <- ifelse(is.na(YO), 0, YO)
    YO1 <- YO_clean[YS == 1]

    # Linear predictors and residuals
    XS0.g <- drop(XS0 %*% g)
    XS1.g <- drop(XS1 %*% g)
    XO1.b <- drop(XO1 %*% b)
    u2    <- YO1 - XO1.b
    r     <- sqrt(1 - rho^2)

    # Compute correction term
    B <- (XS1.g + (rho / sigma) * u2) / r
    lambdaB <- exp(dnorm(B, log = TRUE) - pnorm(B, log.p = TRUE))

    # Initialize gradient matrix
    gradient <- matrix(0, nrow = nObs, ncol = nParam)

    # Gradient contributions
    gradient[YS == 0, istartS] <- -XS0 * exp(dnorm(-XS0.g, log = TRUE) - pnorm(-XS0.g, log.p = TRUE))
    gradient[YS == 1, istartS] <-  XS1 * lambdaB / r
    gradient[YS == 1, istartO] <-  XO1 * (u2 / sigma^2 - lambdaB * rho / (sigma * r))
    gradient[YS == 1, isigma]  <- (u2^2 / sigma^3 - lambdaB * rho * u2 / (sigma^2 * r)) - 1 / sigma
    gradient[YS == 1, irho]    <- lambdaB * (u2 / sigma + rho * XS1.g) / r^3

    return(colSums(gradient))
  }

  ######################################
  # Starting values (if not provided)
  ######################################
  if (is.null(start)) {
    start <- tryCatch(step2(YS, XS, YO, XO), error = function(e) NULL)
    if (is.null(start) || any(!is.finite(start))) {
      message("Two-step starting values failed; using neutral default start values.")
      start <- c(rep(0, NXS + NXO), 1, 0)
    }
    start[NXS + NXO + 1L] <- max(start[NXS + NXO + 1L], .Machine$double.eps)
    start[NXS + NXO + 2L] <- .ssmodels_clip_rho(start[NXS + NXO + 2L])
  }

  ######################################
  # Maximum Likelihood Estimation
  ######################################
  theta_HC <- optim(start,
                    loglik_HC,
                    gradlik_HC,
                    method  = "BFGS",
                    hessian = TRUE,
                    control = list(fnscale = -1))

  # Assign names to estimated parameters
  names(theta_HC$par) <- c(colnames(XS), colnames(XO), "sigma", "rho")

  ######################################
  # Organize results
  ######################################
  a   <- start
  a1  <- theta_HC$par
  a2  <- theta_HC$value
  a3  <- theta_HC$counts[2]
  a4  <- theta_HC$hessian
  a5  <- .ssmodels_safe_vcov(a4)
  a6  <- .ssmodels_safe_se(a5)
  a7  <- YSLevels
  a8  <- length(YS)
  a9  <- length(start)
  a10 <- sum(YS == 0)
  a11 <- sum(YS == 1)
  a12 <- NXS
  a13 <- NXO
  a14 <- a8 - a9
  a15 <- -2 * a2 + 2 * a9
  a16 <- -2 * a2 + a9 * log(a8)

  # Create result object
  result <- list(coefficients   = a1,
                 value          = a2,
                 loglik         = a2,
                 counts         = a3,
                 hessian        = a4,
                 fisher_infoHC  = a5,
                 prop_sigmaHC   = a6,
                 level          = a7,
                 nObs           = a8,
                 nParam         = a9,
                 N0             = a10,
                 N1             = a11,
                 NXS            = a12,
                 NXO            = a13,
                 df             = a14,
                 aic            = a15,
                 bic            = a16,
                 initial.value  = a)

  class(result) <- c("HeckmanCL", class(theta_HC))
  return(result)
}

#' Summary of Generalized Heckman Model
#'
#' @description
#' Print estimates of the parameters of the Generalized Heckman model using Maximum Likelihood Estimation.
#'
#' @param object An object of class \code{HeckmanGe}.
#' @param ... Additional arguments (currently unused).
#'
#' @return
#' Prints the summary output including coefficient tables and model fit statistics.
#'
#' @export summary.HeckmanGe
#' @export
summary.HeckmanGe <- function(object, ...) {
  prop_sigmaHG <- object$prop_sigmaHG
  coeffsHG <- object$coefficients
  counts <- object$counts
  loglik <- object$loglik
  NObs <- object$nObs
  nParam <- object$nParam
  df <- object$df
  NXS <- object$NXS
  NXO <- object$NXO
  N0 <- object$N0
  N1 <- object$N1
  aic <- object$aic
  bic <- object$bic
  NE <- object$NE
  NV <- object$NV

  idxS <- seq_len(NXS)
  idxO <- seq.int(NXS + 1L, NXS + NXO)
  idxE <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
  idxV <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)

  tb1 <- miscTools::coefTable(coeffsHG[idxS], prop_sigmaHG[idxS], df = df)
  tb2 <- miscTools::coefTable(coeffsHG[idxO], prop_sigmaHG[idxO], df = df)
  tb3 <- miscTools::coefTable(coeffsHG[idxE], prop_sigmaHG[idxE], df = df)
  tb4 <- miscTools::coefTable(coeffsHG[idxV], prop_sigmaHG[idxV], df = df)

  cat("\n")
  cat("--------------------------------------------------------------\n")
  cat("       Generalized Heckman Model (Package: ssmodels)          \n")
  cat("--------------------------------------------------------------\n")
  cat("Maximum Likelihood estimation \n")
  cat("optim function with method BFGS - iterations number:", counts, "\n")
  cat("Log-Likelihood:", loglik, "\n")
  cat("AIC:", aic, "BIC:", bic, "\n")
  cat("Number of observations:", NObs, "(", N0, "censored and", N1, "observed )\n")
  cat(nParam, "free parameters ( df =", df, ")\n")
  cat("--------------------------------------------------------------\n")

  cat("Probit selection equation:\n")
  printCoefmat(tb1, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  cat("Outcome equation:\n")
  printCoefmat(tb2, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  cat("Dispersion terms:\n")
  printCoefmat(tb3, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  cat("Correlation terms:\n")
  printCoefmat(tb4, signif.stars = TRUE, signif.legend = TRUE, digits = 4)
  cat("--------------------------------------------------------------\n")
}

#' Heckman BS Model fit Function
#'
#' @description
#' Estimates the parameters of the Heckman-BS model
#'
#' @return
#'
#' Returns a list with the following components.
#'
#' Coefficients: Returns a numerical vector with the best estimated values
#' of the model parameters;
#'
#' Value: The value of function to be minimized (or maximized) corresponding
#' to par.
#'
#' loglik: Maximized value of the log-likelihood function calculated from the estimated coefficients.
#'
#' counts: Component of the Optim function. A two-element integer vector
#' giving the number of calls to fn and gr respectively. This excludes
#' those calls needed to compute the Hessian, if requested, and any calls
#' to fn to compute a finite-difference approximation to the gradient.
#'
#' hessian: Component of the Optim function, with pre-defined option
#' hessian=TRUE. A symmetric matrix giving an estimate of the Hessian
#' at the solution found. Note that this is the Hessian of the unconstrained
#' problem even if the box constraints are active.
#'
#' fisher_infoBS: Fisher information matrix
#'
#' prop_sigmaBS: Square root of the Fisher information matrix diagonal
#'
#' coefficients_link: Estimates on the optimization scale. The last component
#' is \code{rho_star}, where \code{rho = 2 / (1 + exp(-rho_star)) - 1}.
#'
#' gradient_link: Analytical score vector evaluated at \code{coefficients_link}.
#'
#' level: Selection variable levels
#'
#' nObs: Numeric value representing the size of the database
#'
#' nParam: Numerical value representing the number of model parameters
#'
#' N0: Numerical value representing the number of unobserved entries
#'
#' N1: Numerical value representing the number of complete entries
#'
#' NXS: Numerical value representing the number of parameters of the
#' selection model
#'
#' NXO: Numerical value representing the number of parameters of the
#' regression model
#'
#' df: Numerical value that represents the difference between the size
#' of the response vector of the selection equation and the number of
#' model parameters
#'
#' aic: Numerical value representing Akaike's information criterion.
#'
#' bic: Numerical value representing Schwarz's Bayesian Criterion
#'
#' initial.value: Numerical vector that represents the input values
#' (Initial Values) used in the parameter estimation.
#'
#' @details
#' The HeckmanBS() function fits the Sample Selection Model
#' based on the Birnbaum Saunders bivariate distribution,
#' it has the same number of parameters as the classical
#' Heckman model. For more information see Bastos and Barreto-Souza
#' (2020) \doi{10.1080/02664763.2020.1780570}.
#'
#'
#' @param selection Selection equation.
#' @param outcome Primary Regression Equation.
#' @param data Database.
#' @param start initial values.
#' @examples
#' data(MEPS2001)
#' attach(MEPS2001)
#' selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
#' outcomeBS <- ambexp ~ age + female + educ + blhisp + totchr + ins
#' HeckmanBS(selectEq, outcomeBS, data = MEPS2001)
#' @importFrom Rdpack reprompt
#' @importFrom stats model.frame
#' @references Bastos, F. S. and Barreto-Souza, W. (2020).
#' Birnbaum-Saunders sample selection model. \emph{Journal of Applied
#' Statistics}. \doi{10.1080/02664763.2020.1780570}.
#' @export HeckmanBS
#' @export
HeckmanBS <- function(selection, outcome, data = sys.frame(sys.parent()), start = NULL) {

  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  XS  <- components$XS
  YS  <- components$YS
  NXS <- components$NXS
  XO  <- components$XO
  YO  <- components$YO
  NXO <- components$NXO
  YSLevels <- components$YSLevels

  if (any(YS == 1L & (!is.finite(YO) | YO <= 0))) {
    stop("HeckmanBS requires strictly positive outcome values for selected observations.")
  }

  if (is.null(start)) {
    message("Start not provided using default start values.")
    start <- c(rep(0, NXS + NXO), 1, 0)
  }

  # Auxiliary transformation: maps rho_star ∈ ℝ to rho ∈ (-1, 1)
  inv_logit_2 <- function(x) 2 / (1 + exp(-x)) - 1
  loglik_BS <- function(par) {
    # Check for invalid parameters
    if (any(!is.finite(par))) return(NA)

    # Parameter indexing
    igamma <- 1:NXS
    ibeta  <- seq(tail(igamma, 1) + 1, length.out = NXO)
    iphi1  <- tail(ibeta, 1) + 1
    irho   <- iphi1 + 1

    # Parameter extraction
    gamma <- par[igamma]
    beta  <- par[ibeta]
    phi1  <- par[iphi1]
    if (!is.finite(phi1) || phi1 < 1e-3 || phi1 > 100) return(NA)
    rho_star <- par[irho]
    rho <- inv_logit_2(rho_star)
    phi2 <- 1  # Fixed value for identifiability

    # Prepare data subsets
    XS0 <- XS[YS == 0, , drop = FALSE]
    XS1 <- XS[YS == 1, , drop = FALSE]
    YO_clean <- ifelse(is.na(YO), 0, YO)
    YO1 <- YO_clean[YS == 1]
    XO1 <- XO[YS == 1, , drop = FALSE]

    # Linear predictors
    XS0.g <- exp(XS0 %*% gamma)
    XS1.g <- exp(XS1 %*% gamma)
    XO1.b <- exp(XO1 %*% beta)

    # Compute likelihood terms
    term0 <- sqrt(YO1 * (phi1 + 1)/(phi1 * XO1.b)) - sqrt((phi1 * XO1.b)/(YO1 * (phi1 + 1)))
    term1 <- exp((-phi1 / 4) * term0^2)
    term2 <- sqrt((phi1 + 1)/(phi1 * XO1.b * YO1)) + sqrt((phi1 * XO1.b)/((phi1 + 1) * YO1^3))
    term3 <- (1 / (2 * sqrt(2 * pi))) * sqrt(phi1 / 2)
    temp_rho2 <- pmax(1 - rho^2, 1e-6)
    term4 <- sqrt((phi2 + 1)/(2 * XS1.g * temp_rho2))
    term5 <- (phi2 * XS1.g)/(phi2 + 1) - 1
    term6 <- rho * sqrt(phi1 / (2 * temp_rho2))
    integrand <- term4 * term5 + term6 * term0
    term7 <- pnorm(integrand, log.p = TRUE)

    term8 <- sqrt(phi2 / 2) * (sqrt((phi2 + 1)/(phi2 * XS0.g)) - sqrt((phi2 * XS0.g)/(phi2 + 1)))
    FT2 <- pnorm(term8, log.p = TRUE)

    # Stabilize log computations
    term1[term1 < 1e-300] <- 1e-300
    logterm1 <- log(term1)
    logterm2 <- log(term2)
    if (any(!is.finite(logterm1)) || any(!is.finite(logterm2))) return(NA)

    # Final log-likelihood
    ll <- sum(logterm1 + logterm2 + log(term3) + term7) + sum(FT2)
    return(ll)
  }

  ################################################################################

  gradlik_BS <- function(par) {
    # Parameter indexing
    igamma <- 1:NXS
    ibeta  <- seq(tail(igamma, 1) + 1, length.out = NXO)
    iphi1  <- tail(ibeta, 1) + 1
    irho   <- iphi1 + 1

    # Parameter extraction
    gamma <- par[igamma]
    beta  <- par[ibeta]
    phi1  <- par[iphi1]
    rho_star <- par[irho]
    rho <- inv_logit_2(rho_star)
    phi2 <- 1  # Fixed value

    # Prepare data subsets
    XS0 <- XS[YS == 0, , drop = FALSE]
    XS1 <- XS[YS == 1, , drop = FALSE]
    YO_clean <- ifelse(is.na(YO), 0, YO)
    YO1 <- YO_clean[YS == 1]
    XO1 <- XO[YS == 1, , drop = FALSE]

    # Linear predictors
    XS0.g <- exp(as.numeric(XS0 %*% gamma))
    XS1.g <- exp(as.numeric(XS1 %*% gamma))
    XO1.b <- exp(as.numeric(XO1 %*% beta))

    # Auxiliary terms
    temp_rho2 <- pmax(1 - rho^2, 1e-6)
    term0 <- sqrt(YO1 * (phi1 + 1)/(phi1 * XO1.b)) - sqrt((phi1 * XO1.b)/(YO1 * (phi1 + 1)))
    term1 <- exp((-phi1 / 4) * term0^2)
    term2 <- sqrt((phi1 + 1)/(phi1 * XO1.b * YO1)) + sqrt((phi1 * XO1.b)/((phi1 + 1) * YO1^3))
    term3 <- (1 / (2 * sqrt(2 * pi))) * sqrt(phi1 / 2)
    term4 <- sqrt((phi2 + 1)/(2 * XS1.g * temp_rho2))
    term5 <- (phi2 * XS1.g)/(phi2 + 1) - 1
    term6 <- rho * sqrt(phi1 / (2 * temp_rho2))
    integrand <- term4 * term5 + term6 * term0
    term8 <- sqrt(phi2 / 2) * (sqrt((phi2 + 1)/(phi2 * XS0.g)) - sqrt((phi2 * XS0.g)/(phi2 + 1)))

    # Ratios of PDF to CDF
    lambda_I  <- exp(dnorm(integrand, log = TRUE) - pnorm(integrand, log.p = TRUE))
    lambda_T8 <- exp(dnorm(term8, log = TRUE) - pnorm(term8, log.p = TRUE))

    # Partial derivatives
    term9  <- (-1 / 2) * (sqrt(YO1 * (phi1 + 1)/(phi1 * XO1.b)) + sqrt((phi1 * XO1.b)/(YO1 * (phi1 + 1)))) * model.matrix(~XO1 - 1)
    term10 <- (1 / 2) * (sqrt((XO1.b * phi1)/(YO1^3 * (phi1 + 1))) - sqrt((phi1 + 1)/(YO1 * phi1 * XO1.b))) * model.matrix(~XO1 - 1)
    term11 <- (-1 / 2) * sqrt(phi2 / 2) * (sqrt((phi2 + 1)/(phi2 * XS0.g)) + sqrt((phi2 * XS0.g)/(phi2 + 1))) * model.matrix(~XS0 - 1)
    term12 <- (-1 / 2) * (sqrt(YO1/(phi1^3 * XO1.b * (phi1 + 1))) + sqrt(XO1.b/(YO1 * (phi1 + 1)^3 * phi1)))
    term13 <- (1 / 2) * (sqrt(XO1.b / (phi1 * (phi1 + 1)^3 * YO1^3)) - sqrt(1 / (phi1^3 * (phi1 + 1) * XO1.b * YO1)))
    term14 <- rho / (2 * sqrt(2 * phi1 * temp_rho2))

    # Derivatives w.r.t. parameters
    df1b     <- (-phi1 / 2) * term0 * term9
    df2b     <- term10 / term2
    df4b     <- lambda_I * term9 * term6
    df4g     <- lambda_I * (1/4) * ((XS1.g + 2) / sqrt(XS1.g * temp_rho2)) * model.matrix(~XS1 - 1)
    df5g     <- lambda_T8 * term11
    df1phi1  <- - (term0^2) / 4 - (phi1 / 2) * term0 * term12
    df2phi1  <- term13 / term2
    df3phi1  <- 1 / (8 * term3 * sqrt(pi * phi1))
    df4phi1  <- lambda_I * (term0 * term14 + term6 * term12)

    # Derivative w.r.t. rho using chain rule
    dt5_drho <- if (abs(rho) < 1e-8) sqrt(phi1 / 2) else sqrt(phi1 / (2 * temp_rho2)) + rho^2 * sqrt(phi1) / ((temp_rho2)^(3/2) * sqrt(2))
    d_rho    <- (2 * exp(-rho_star)) / (1 + exp(-rho_star))^2
    df4rho_raw <- lambda_I * (term5 * term4 * (rho / temp_rho2) + dt5_drho * term0)
    gr_rho_star <- df4rho_raw * d_rho

    # Assemble gradient matrix
    gradient <- matrix(0, nrow = length(YS), ncol = length(par))
    gradient[YS == 0, igamma] <- df5g
    gradient[YS == 1, igamma] <- df4g
    gradient[YS == 1, ibeta]  <- df1b + df2b + df4b
    gradient[YS == 1, iphi1]  <- df1phi1 + df2phi1 + df3phi1 + df4phi1
    gradient[YS == 1, irho]   <- gr_rho_star

    return(colSums(gradient))
  }

  theta_BS <- optim(start, loglik_BS, gradlik_BS, method = "BFGS", hessian = TRUE,
                    control = list(fnscale = -1, maxit = 1000))
  names(theta_BS$par) <- c(colnames(XS), colnames(XO), "phi1", "rho_star")

  irho <- length(theta_BS$par)
  coefficients_link <- theta_BS$par
  gradient_link <- gradlik_BS(coefficients_link)
  names(gradient_link) <- names(coefficients_link)
  a1 <- coefficients_link
  a4 <- theta_BS$hessian
  a5 <- .ssmodels_safe_vcov(a4)
  a6 <- .ssmodels_safe_se(a5)
  rho_star <- a1[irho]
  rho_hat <- 2 / (1 + exp(-rho_star)) - 1
  g_prime <- 2 * exp(-rho_star) / (1 + exp(-rho_star))^2
  se_rho_star <- a6[irho]
  se_rho <- abs(g_prime) * se_rho_star
  a1[irho] <- rho_hat
  names(a1)[irho] <- "rho"
  a6[irho] <- se_rho
  names(a6) <- names(a1)

  result <- list(
    coefficients     = a1,
    coefficients_link = coefficients_link,
    value            = theta_BS$value,
    loglik           = theta_BS$value,
    gradient_link    = gradient_link,
    counts           = theta_BS$counts[2],
    hessian          = a4,
    fisher_infoBS    = a5,
    prop_sigmaBS     = a6,
    level            = YSLevels,
    nObs             = length(YS),
    nParam           = length(start),
    N0               = sum(YS == 0),
    N1               = sum(YS == 1),
    NXS              = NXS,
    NXO              = NXO,
    df               = length(YS) - length(start),
    aic              = -2 * theta_BS$value + 2 * length(start),
    bic              = -2 * theta_BS$value + length(start) * log(length(YS)),
    initial.value    = start
  )
  class(result) <- c("HeckmanBS")
  return(result)
}

#' Post-process Parameter Vector for Generalized Heckman Models
#'
#' @description
#' This helper assigns names to a generalized Heckman parameter vector and,
#' for intercept-only scale/correlation models, maps log(sigma) and atanh(rho)
#' to sigma and rho.
#'
#' @param theta_par Numeric vector of estimated parameters on the optimization scale.
#' @param NXS Integer. Number of covariates in the selection equation.
#' @param NXO Integer. Number of covariates in the outcome equation.
#' @param NE Integer. Number of scale parameters.
#' @param NV Integer. Number of correlation parameters.
#' @param XS Design matrix for the selection equation.
#' @param XO Design matrix for the outcome equation.
#' @param outcomeS Design matrix or variable for the scale model.
#' @param outcomeC Design matrix or variable for the correlation model.
#'
#' @return A named numeric vector.
#' @keywords internal
#' @export
postprocess_theta <- function(theta_par, NXS, NXO, NE, NV, XS, XO, outcomeS, outcomeC) {
  sigma_names <- if (NE == 1L) "sigma" else paste0("log.sigma.", seq_len(NE))
  rho_names <- if (NV == 1L) "rho" else paste0("atanh.rho.", seq_len(NV))
  param_names <- c(colnames(XS), colnames(XO), sigma_names, rho_names)
  names(theta_par) <- param_names

  idx_sigma <- seq.int(NXS + NXO + 1L, NXS + NXO + NE)
  idx_rho <- seq.int(NXS + NXO + NE + 1L, NXS + NXO + NE + NV)

  if (NE == 1L) theta_par[idx_sigma] <- exp(theta_par[idx_sigma])
  if (NV == 1L) theta_par[idx_rho] <- tanh(theta_par[idx_rho])

  theta_par
}

# Internal numerical helpers for ssmodels

.ssmodels_clip_rho <- function(rho, eps = 1e-07) {
  pmin(pmax(rho, -1 + eps), 1 - eps)
}

.ssmodels_inv_mills <- function(eta) {
  exp(stats::dnorm(eta, log = TRUE) - stats::pnorm(eta, log.p = TRUE))
}

.ssmodels_safe_vcov <- function(hessian) {
  n <- if (is.null(hessian)) 0L else ncol(hessian)
  if (n == 0L) {
    return(matrix(NA_real_, nrow = 0L, ncol = 0L))
  }
  out <- tryCatch(solve(-hessian), error = function(e) NULL)
  if (is.null(out) || any(!is.finite(out))) {
    out <- tryCatch(MASS::ginv(-hessian), error = function(e) matrix(NA_real_, nrow = n, ncol = n))
  }
  out
}

.ssmodels_safe_se <- function(vcov) {
  se <- suppressWarnings(sqrt(diag(vcov)))
  se[!is.finite(se)] <- NA_real_
  se
}

#' Heckman-t Model fit Function
#'
#' @description
#' Estimates the parameters of the Heckman-t model. The scale, correlation and
#' degrees of freedom are optimized on unconstrained scales to improve numerical
#' stability: sigma = exp(eta_sigma), rho = tanh(eta_rho), and df = 2 + exp(eta_df).
#'
#' @param selection Selection equation.
#' @param outcome Primary regression equation.
#' @param data Database.
#' @param df Initial value for the degrees of freedom. Values greater than 2 are
#'   recommended because the model uses a finite-variance Student-t error.
#' @param start Optional starting values on the natural scale
#'   (selection, outcome, sigma, rho, df).
#'
#' @return A list with estimated coefficients, log-likelihood and diagnostics.
#'
#' @examples
#' data(MEPS2001)
#' dados <- MEPS2001[seq_len(500), ]
#' selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
#' outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins
#' HeckmantS(selectEq, outcomeEq, data = dados, df = 12)
#'
#' @importFrom Rdpack reprompt
#' @references Marchenko, Y. V. and Genton, M. G. (2012). A Heckman selection-t
#' model. \emph{Journal of the American Statistical Association}, 107(497),
#' 304-317.
#' @export HeckmantS
#' @export
HeckmantS <- function(selection, outcome, data = sys.frame(sys.parent()),
                      df, start = NULL) {

  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  XS <- components$XS
  YS <- components$YS
  NXS <- components$NXS
  XO <- components$XO
  YO <- components$YO
  NXO <- components$NXO
  YSLevels <- components$YSLevels
  YO_clean <- ifelse(is.na(YO), 0, YO)

  loglik_tS <- function(par) {
    if (any(!is.finite(par))) return(NA_real_)

    istartS <- seq_len(NXS)
    istartO <- seq.int(NXS + 1L, NXS + NXO)
    isigma <- NXS + NXO + 1L
    irho <- isigma + 1L
    iv <- irho + 1L

    gamma <- par[istartS]
    beta <- par[istartO]
    sigma <- exp(par[isigma])
    rho <- tanh(par[irho])
    nu <- 2 + exp(par[iv])

    mu2 <- drop(XS %*% gamma)
    mu1 <- drop(XO %*% beta)
    z <- (YO_clean - mu1) / sigma
    r <- sqrt(pmax(1 - rho^2, .Machine$double.eps))
    q <- sqrt((nu + 1) / (nu + z^2))
    eta <- q * ((rho * z + mu2) / r)

    ll0 <- stats::pt(-mu2, df = nu, log.p = TRUE)
    ll1 <- stats::dt(z, df = nu, log = TRUE) - log(sigma) +
      stats::pt(eta, df = nu + 1, log.p = TRUE)

    sum(ifelse(YS == 0L, ll0, ll1))
  }

  if (missing(df) || is.null(df) || !is.finite(df) || df <= 2) {
    df <- 12
  }

  if (is.null(start)) {
    start_classic <- tryCatch(step2(YS, XS, YO, XO), error = function(e) NULL)
    if (is.null(start_classic) || any(!is.finite(start_classic))) {
      message("Two-step starting values failed; using neutral default start values.")
      start_classic <- c(rep(0, NXS + NXO), sigma = 1, rho = 0)
    }
    start <- c(start_classic[seq_len(NXS + NXO)],
               sigma = start_classic[NXS + NXO + 1L],
               rho = start_classic[NXS + NXO + 2L],
               df = df)
  }

  if (length(start) != NXS + NXO + 3L) {
    stop("start must have length NXS + NXO + 3: selection, outcome, sigma, rho and df.")
  }

  start_opt <- start
  start_opt[NXS + NXO + 1L] <- log(max(start[NXS + NXO + 1L], .Machine$double.eps))
  start_opt[NXS + NXO + 2L] <- atanh(.ssmodels_clip_rho(start[NXS + NXO + 2L]))
  start_opt[NXS + NXO + 3L] <- log(max(start[NXS + NXO + 3L] - 2, .Machine$double.eps))
  names(start_opt) <- c(colnames(XS), colnames(XO), "log.sigma", "atanh.rho", "log.df.minus.2")

  theta_tS <- stats::optim(start_opt, loglik_tS, method = "BFGS",
                           hessian = TRUE,
                           control = list(fnscale = -1, maxit = 1000))

  vcov_link <- .ssmodels_safe_vcov(theta_tS$hessian)
  se_link <- .ssmodels_safe_se(vcov_link)

  coef_out <- theta_tS$par
  isigma <- NXS + NXO + 1L
  irho <- isigma + 1L
  iv <- irho + 1L
  coef_out[isigma] <- exp(theta_tS$par[isigma])
  coef_out[irho] <- tanh(theta_tS$par[irho])
  coef_out[iv] <- 2 + exp(theta_tS$par[iv])
  names(coef_out) <- c(colnames(XS), colnames(XO), "sigma", "rho", "df")

  jac <- rep(1, length(theta_tS$par))
  jac[isigma] <- coef_out[isigma]
  jac[irho] <- 1 - coef_out[irho]^2
  jac[iv] <- coef_out[iv] - 2
  vcov_nat <- diag(jac, nrow = length(jac)) %*% vcov_link %*% diag(jac, nrow = length(jac))
  se_nat <- .ssmodels_safe_se(vcov_nat)
  names(se_nat) <- names(coef_out)

  nObs <- length(YS)
  nParam <- length(start_opt)
  result <- list(
    coefficients    = coef_out,
    coefficients_link = theta_tS$par,
    value           = theta_tS$value,
    loglik          = theta_tS$value,
    counts          = theta_tS$counts[1],
    hessian         = theta_tS$hessian,
    fisher_infotS   = vcov_nat,
    prop_sigmatS    = se_nat,
    level           = YSLevels,
    nObs            = nObs,
    nParam          = nParam,
    N0              = sum(YS == 0L),
    N1              = sum(YS == 1L),
    NXS             = NXS,
    NXO             = NXO,
    df              = nObs - nParam,
    aic             = -2 * theta_tS$value + 2 * nParam,
    bic             = -2 * theta_tS$value + nParam * log(nObs),
    initial.value   = start_opt
  )
  class(result) <- c("HeckmantS", class(theta_tS))
  result
}

#' Heckman's two-step method
#'
#' @description
#' Estimate classical Heckman model starting values by Heckman's two-step method.
#'
#' @param YS Selection vector coded as 0/1.
#' @param XS Selection matrix.
#' @param YO Outcome vector.
#' @param XO Covariate matrix for the outcome equation.
#'
#' @return A numeric vector with selection coefficients, outcome coefficients,
#'   sigma and rho.
#'
#' @importFrom stats binomial glm lm.fit dnorm pnorm na.omit
#' @export
step2 <- function(YS, XS, YO, XO) {
  YS <- as.integer(YS)
  if (!all(stats::na.omit(unique(YS)) %in% c(0L, 1L))) {
    stop("YS must be coded as 0/1.")
  }

  XS <- as.matrix(XS)
  XO <- as.matrix(XO)
  selected <- YS == 1L

  if (sum(selected) <= ncol(XO)) {
    stop("There are not enough selected observations to estimate the outcome equation.")
  }

  fit1 <- stats::glm.fit(x = XS, y = YS, family = stats::binomial(link = "probit"))
  eta <- drop(XS %*% stats::coef(fit1))
  imr <- exp(stats::dnorm(eta, log = TRUE) - stats::pnorm(eta, log.p = TRUE))

  XO1 <- XO[selected, , drop = FALSE]
  YO1 <- YO[selected]
  imr1 <- imr[selected]
  eta1 <- eta[selected]

  fit2 <- stats::lm.fit(x = cbind(XO1, IMR = imr1), y = YO1)
  coef2 <- stats::coef(fit2)
  beta <- coef2[seq_len(ncol(XO))]
  imr_coef <- coef2[ncol(XO) + 1L]

  delta <- imr1 * (imr1 + eta1)
  variance <- (sum(fit2$residuals^2) + imr_coef^2 * sum(delta)) / sum(selected)
  if (!is.finite(variance) || variance <= 0) {
    variance <- stats::var(YO1, na.rm = TRUE)
  }
  sigma <- sqrt(variance)
  rho <- imr_coef / sigma
  rho <- .ssmodels_clip_rho(rho)

  out <- c(stats::coef(fit1), beta, sigma = sigma, rho = rho)
  names(out) <- c(colnames(XS), colnames(XO), "sigma", "rho")
  out
}

#' Summary of Classic Heckman Model
#'
#' @description
#' Print estimates of the parameters of the Classic Heckman model using Maximum Likelihood Estimation.
#'
#' @param object An object of class \code{HeckmanCL}.
#' @param ... Additional arguments (currently unused).
#'
#' @return
#' Prints the summary output including coefficient tables and model fit statistics.
#'
#' @export summary.HeckmanCL
#' @export
summary.HeckmanCL <- function(object, ...) {

  # Extract components from the HeckmanCL object
  fisher_infoHC <- object$fisher_infoHC       # Fisher information matrix (not used here)
  prop_sigmaHC  <- object$prop_sigmaHC        # Standard errors of the estimates
  coeffsHC      <- object$coefficients        # Estimated coefficients
  counts        <- object$counts              # Number of iterations in optimization
  value         <- object$value               # Log-likelihood at convergence
  loglik        <- object$loglik              # Log-likelihood function
  NObs          <- object$nObs                # Total number of observations
  nParam        <- object$nParam              # Number of free parameters
  df            <- object$df                  # Degrees of freedom
  NXS           <- object$NXS                 # Number of selection equation covariates
  NXO           <- object$NXO                 # Number of outcome equation covariates
  N0            <- object$N0                  # Number of censored observations
  N1            <- object$N1                  # Number of uncensored observations
  aic           <- object$aic                 # Akaike Information Criterion
  bic           <- object$bic                 # Bayesian Information Criterion

  # Build coefficient tables for each block of the model
  tb  <- miscTools::coefTable(coeffsHC, prop_sigmaHC, df = df)

  tb1 <- miscTools::coefTable(
    coeffsHC[1:NXS],
    prop_sigmaHC[1:NXS],
    df = df
  )

  tb2 <- miscTools::coefTable(
    coeffsHC[(NXS + 1):(NXS + NXO)],
    prop_sigmaHC[(NXS + 1):(NXS + NXO)],
    df = df
  )

  tb3 <- miscTools::coefTable(
    coeffsHC[(NXS + NXO + 1):(NXS + NXO + 2)],
    prop_sigmaHC[(NXS + NXO + 1):(NXS + NXO + 2)],
    df = df
  )

  # Print model summary
  cat("\n")
  cat("--------------------------------------------------------------\n")
  cat("          Classic Heckman Model (Package: ssmodels)           \n")
  cat("--------------------------------------------------------------\n")
  cat("--------------------------------------------------------------\n")
  cat("Maximum Likelihood estimation \n")
  cat("optim function with method BFGS - iterations number:", counts, "\n")
  cat("Log-Likelihood:", loglik, "\n")
  cat("AIC:", aic, "BIC:", bic, "\n")
  cat("Number of observations:", NObs, "(", N0, "censored and", N1, "observed", ")", "\n")
  cat(nParam, "free parameters", "( df =", df, ")", "\n")
  cat("--------------------------------------------------------------\n")

  # Print selection equation coefficients
  cat("Probit selection equation:\n")
  printCoefmat(tb1, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  # Print outcome equation coefficients
  cat("Outcome equation:\n")
  printCoefmat(tb2, signif.stars = TRUE, signif.legend = FALSE, digits = 4)
  cat("--------------------------------------------------------------\n")

  # Print error term estimates (sigma and rho)
  cat("Error terms:\n")
  printCoefmat(tb3, signif.stars = TRUE, signif.legend = TRUE, digits = 4)
  cat("--------------------------------------------------------------\n")
}

#' A package that provides functions to fit data affected by sample selection bias.
#'
#' Package that provides models to fit data with sample selection bias problems. Includes:
#' \describe{
#' \item{HeckmanCL(selectEq, outcomeEq, data = data, start)}{Heckman's classic model fit function. Sample selection
#' usually arises in practice as a result of partial observability of the
#' result of interest in a study. In the presence of sample selection, the
#' observed data do not represent a random sample of the population, even
#' after controlling for explanatory variables. #' That is, the data is not missing completely at random.
#' Thus, standard analysis using only complete cases will lead to
#' biased results. Heckman introduced a sample selection model to analyze
#' this data and proposed a complete likelihood estimation method under the
#' assumption of normality. Such model was called Heckman model or Tobit 2
#' model.}
#' \item{HeckmantS(selectEq, outcomeEq, data = data, df, start)}{Heckman-t model adjustment function. The Heckman-t model
#' maintains the original parametric structure of the Classic Heckman model,
#' but considers a bivariate Student's t distribution as the underlying joint
#' distribution of the selection and primary regression variable and estimates
#' the parameters by maximum likelihood.}
#' \item{HeckmanSK(selectEq, outcomeEq, data = data, lambda, start)}{Heckman-SK model adjustment function. The Heckman-sk
#' model maintains the original parametric structure of the Classic Heckman
#' model, but considers a bivariate Skew-Normal distribution as the underlying
#' joint distribution of the selection and primary regression variable and
#' estimates the parameters by maximum likelihood.}
#' \item{HeckmanBS(selectEq, outcomeBS, data = data, start)}{Heckman-BS model adjustment function. The Heckman-BS model
#' maintains the original parametric structure of the Classic Heckman model,
#' but considers a bivariate Birnbaum-Saunders distribution as the underlying
#' joint distribution of the selection and primary regression variable and
#' estimates the parameters by maximum likelihood.}
#' \item{HeckmanGe(selectEq, outcomeEq,outcomeS, outcomeC, data = data)}{
#' Function for fitting the Generalized Heckman model.
#' This model generalizes the Classic Heckman model by including covariates
#' in the dispersion and correlation structures. It allows identification
#' of variables responsible for selection bias and heteroscedasticity.}
#' }
#'
#' @importFrom stats binomial coef dnorm dt glm lm model.matrix model.response na.pass optim pnorm printCoefmat pt qnorm terms
#' @importFrom utils tail
#' @importFrom Rdpack reprompt
#'
#' @param selection Selection equation.
#' @param outcome Primary regression equation for the observed response.
#' @param outcomeS Matrix of covariates for modeling the dispersion parameter (sigma).
#' @param outcomeC Matrix of covariates for modeling the correlation parameter (rho).
#' @param df Initial value to the degree of freedom of Heckman-t model.
#' @param lambda Initial value for asymmetry parameter.
#' @param start initial values.
#' @param data Database.
#'
#' @return A list containing the estimated parameters, Hessian matrix, number of observations,
#' and additional diagnostic information. If initial values are not provided, they are
#' automatically estimated using the Heckman two-step method.
#'
#' @seealso \code{\link{HeckmanCL}}
#' @seealso \code{\link{HeckmantS}}
#' @seealso \code{\link{HeckmanSK}}
#' @seealso \code{\link{HeckmanBS}}
#' @seealso \code{\link{HeckmanGe}}
#'
#' @author Fernando de Souza Bastos, Wagner Barreto de Souza
#'
#' @keywords Heckman
#' @keywords internal
#' @name ssmodels

"_PACKAGE"


#' Medical Expenditure Panel Survey
#'
#'The MEPS is a set of large-scale surveys of families, individuals
#'and their medical providers (doctors, hospitals, pharmacies, etc.)
#'in the United States. It has data on the health services Americans use,
#'how often they use them, the cost of these services and how they are paid,
#'as well as data on the cost and reach of health insurance available
#'to American workers. The sample is restricted to persons aged between 21
#'and 64 years and contains a variable response with 3328 observations of
#'outpatient costs, of which 526 (15.8\%) correspond to unobserved expenditure
#'values and identified as zero expenditure for adjustment of the models.
#'It also includes the following explanatory variables:
#' \itemize{
#'   \item{educ: education status}
#'   \item{age: Age}
#'   \item{income: income}
#'   \item{female: gender}
#'   \item{vgood: a numeric vector}
#'   \item{good: a numeric vector}
#'   \item{hospexp: a numeric vector}
#'   \item{totchr: number of chronic diseases}
#'   \item{ffs: a numeric vector}
#'   \item{dhospexp: a numeric vector}
#'   \item{age2: a numeric vector}
#'   \item{agefem: a numeric vector}
#'   \item{fairpoor: a numeric vector}
#'   \item{year01: a numeric vector}
#'   \item{instype: a numeric vector}
#'   \item{ambexp: a numeric vector}
#'   \item{lambexp: log ambulatory expenditures}
#'   \item{blhisp: ethnicity}
#'   \item{instype_s1: a numeric vector}
#'   \item{dambexp: dummy variable, ambulatory expenditures}
#'   \item{lnambx: a numeric vector}
#'   \item{ins: insurance status}
#' }
#'
#' @source 2001 Medical Expenditure Panel Survey by the Agency
#' for Healthcare Research and Quality.
#'
#' @references
#' Cameron, A. C. and Trivedi, P. K. (2009). \emph{Microeconometrics using
#' Stata}. Stata Press.
#'
#' Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
#' Robust Estimation and Inference in Sample Selection Models}. R package
#' version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}
#'
#' Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
#' sampleSelection. \emph{Journal of Statistical Software}, 27(7).
#' \url{https://www.jstatsoft.org/article/view/v027i07}
#'
#' @examples
#' data(MEPS2001)
#' attach(MEPS2001)
#' hist(lnambx)
#' selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
#' outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins
#' HeckmanCL(selectEq, outcomeEq, data = MEPS2001)
"MEPS2001"

#' U.S. Women's Labor Force Participation
#'
#' The Mroz87 data frame contains data about 753
#' married women. These data are collected within the
#' "Panel Study of Income Dynamics" (PSID). Of the 753
#' observations, the first 428 are for women with positive
#' hours worked in 1975, while the remaining 325
#' observations are for women who did not work for
#' pay in 1975. A more complete discussion of the data
#' is given by Mroz (1987). It also
#' includes the following explanatory variables:
#' \itemize{
#'   \item{lfp: Dummy variable for labor-force participation.}
#'   \item{hours: Wife's hours of work in 1975. }
#'   \item{kids5: Number of children 5 years old or younger.}
#'   \item{kids618: Number of children 6 to 18 years old.}
#'   \item{Age: Wife's age.}
#'   \item{Educ: Wife's educational attainment, in years.}
#'   \item{wage: Wife's average hourly earnings, in 1975 dollars.}
#'   \item{repwage: Wife's wage reported at the time of the 1976 interview.}
#'   \item{hushrs: Husband's hours worked in 1975.}
#'   \item{husage: Husband's age.}
#'   \item{huseduc: Husband's educational attainment, in years.}
#'   \item{huswage: Husband's wage, in 1975 dollars.}
#'   \item{faminc: Family income, in 1975 dollars.}
#'   \item{mtr: Marginal tax rate facing the wife.}
#'   \item{motheduc: Wife's mother's educational attainment, in years.}
#'   \item{fatheduc: Wife's father's educational attainment, in years.}
#'   \item{unem: Unemployment rate in county of residence, in percentage points.}
#'   \item{city: Dummy variable = 1 if live in large city, else 0. }
#'   \item{exper: Actual years of wife's previous labor market experience.}
#'   \item{nwifeinc: Non-wife income.}
#'   \item{wifecoll: Dummy variable for wife's college attendance.}
#'   \item{huscoll: Dummy variable for husband's college attendance.}
#' }
#' @source PSID Staff, The Panel Study of Income Dynamics,
#' Institute for Social ResearchPanel Study of Income
#' Dynamics, University of Michigan, \url{https://www.icpsr.umich.edu/web/ICPSR/series/131}
#'
#' @references
#' Mroz, T. A. (1987). The sensitivity of an empirical model of married
#' women's hours of work to economic and statistical assumptions.
#' \emph{Econometrica}, 55, 765-799.
#'
#' Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
#' Robust Estimation and Inference in Sample Selection Models}. R package
#' version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}
#'
#' Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
#' sampleSelection. \emph{Journal of Statistical Software}, 27(7).
#' \url{https://www.jstatsoft.org/article/view/v027i07}
#'
#' Wooldridge, J. M. (2016). \emph{Introductory Econometrics: A Modern
#' Approach}. Nelson Education.
#'
#' @examples
#' # Wooldridge(2016): page 247
#' data(Mroz87)
#' attach(Mroz87)
#' selectEq  <- lfp ~ nwifeinc + educ + exper + I(exper^2) + age + kids5 + kids618
#' outcomeEq <- log(wage) ~ educ + exper + I(exper^2)
#' outcomeS  <- cbind(educ, exper)
#' outcomeC  <- cbind(educ, exper)
#' outcomeBS <- wage ~ educ + exper + I(exper^2)
#' outcomeBS <- wage ~ educ + exper + I(exper^2)
#' HeckmanCL(selectEq, outcomeEq, data = Mroz87)
#' HeckmanBS(selectEq, outcomeBS, data = Mroz87)
#' HeckmanSK(selectEq, outcomeEq, data = Mroz87, lambda = 1)
#' HeckmantS(selectEq, outcomeEq, data = Mroz87, df=5)
#' HeckmanGe(selectEq, outcomeEq, outcomeS, outcomeC, data = Mroz87)
#'
"Mroz87"

#' RAND Health Insurance Experiment
#'
#' 'The RAND Health Insurance Experiment (RAND HIE) was a comprehensive
#' study of health care cost, utilization and outcome in the United States.
#' It is the only randomized study of health insurance, and the only study
#' which can give definitive evidence as to the causal effects of different
#' health insurance plans. For more information about the database visit:
#' \url{https://en.wikipedia.org/w/index.php?title=RAND_Health_Insurance_Experiment&oldid=110166949}
#' accessed september 09, 2019). This data frame contains the following columns:
#' \itemize{
#' \item{plan: HIE plan number.}
#'  \item{site: Participant's  place of residence
#'  when the participant was initially enrolled.}
#'  \item{coins: Coinsurance rate.}
#'  \item{tookphys: Took baseline physical.}
#'  \item{year: Study year.}
#'  \item{zper: Person identifier.}
#'  \item{black: 1 if race of household head is black.}
#'  \item{income: Family income.}
#'  \item{xage: Age in years.}
#'  \item{female: 1 if person is female.}
#'  \item{educdec: Education of household head in years.}
#'  \item{time: Time eligible during the year.}
#'  \item{outpdol: Outpatient expenses:
#'  all covered outpatient medical services
#'  excluding dental care, outpatient psychotherapy,
#'  outpatient drugs or supplies.}
#'  \item{drugdol: Drug expenses:
#'  all covered outpatient and dental drugs.}
#'  \item{suppdol: Supply expenses:
#'  all covered outpatient supplies including dental.}
#'  \item{mentdol: Psychotherapy expenses:
#'  all covered outpatient psychotherapy services including injections
#'  excluding charges for visits in excess of 52 per year,
#'  prescription drugs, and inpatient care.}
#'  \item{inpdol: Inpatient expenses:
#'  all covered inpatient expenses in a hospital, mental hospital,
#'  or nursing home,
#'  excluding outpatient care and renal dialysis.}
#'  \item{meddol: Medical expenses:
#'  all covered inpatient and outpatient services,
#'  including drugs, supplies, and inpatient costs of newborns
#'  excluding dental care and outpatient psychotherapy.}
#'  \item{totadm: Hospital admissions:
#'  annual number of covered hospitalizations.}
#'  \item{inpmis: Incomplete Hospital Records:
#'  missing inpatient records.}
#'  \item{mentvis: Psychotherapy visits:
#'  indicates the annual number of outpatient visits for psychotherapy.
#'  It includes billed visits only.
#'  The limit was 52 covered visits per person per year.
#'  The count includes an initial visit to a psychiatrist or psychologist.}
#'  \item{mdvis: Face-to-Face visits to physicians:
#'  annual covered outpatient visits with physician providers
#'  (excludes dental, psychotherapy, and
#'  radiology/anesthesiology/pathology-only visits).}
#'  \item{notmdvis: Face-to-Face visits to nonphysicians:
#'  annual covered outpatient visits with nonphysician providers
#'  such as speech and physical therapists, chiropractors,
#'  podiatrists, acupuncturists, Christian Science etc.
#'  (excludes dental, healers, psychotherapy,
#'  and radiology/anesthesiology/pathology-only visits).}
#'  \item{num: Family size.}
#'  \item{mhi: Mental health index.}
#'  \item{disea: Number of chronic diseases.}
#'  \item{physlm: Physical limitations.}
#'  \item{ghindx: General health index.}
#'  \item{mdeoff: Maximum expenditure offer.}
#'  \item{pioff: Participation incentive payment.}
#'  \item{child: 1 if age is less than 18 years.}
#'  \item{fchild: \code{female * child}.}
#'  \item{lfam: log of \code{num} (family size).}
#'  \item{lpi: log of \code{pioff} (participation incentive payment).}
#'  \item{idp: 1 if individual deductible plan.}
#'  \item{logc: \code{log(coins+1)}.}
#'  \item{fmde: 0 if \code{idp=1},
#'  \code{ln(max(1,mdeoff/(0.01*coins)))} otherwise.}
#'  \item{hlthg: 1 if self-rated health is good
#'  -- baseline is excellent self-rated health.}
#'  \item{hlthf: 1 if self-rated health is fair
#'  -- baseline is excellent self-rated health.}
#'  \item{hlthp: 1 if self-rated health is poor
#'  -- baseline is excellent self-rated health.}
#'  \item{xghindx: \code{ghindx} (general healt index)
#'  with imputations of missing values.}
#'  \item{linc: log of \code{income} (family income).}
#'  \item{lnum: log of \code{num} (family size).}
#'  \item{lnmeddol: log of \code{meddol} (medical expenses).}
#'  \item{binexp: 1 if \code{meddol} > 0.}
#'  }
#'
#' @source \url{https://cameron.econ.ucdavis.edu/mmabook/mmadata.html}
#'
#' @references
#' Cameron, A. C. and Trivedi, P. K. (2005). \emph{Microeconometrics: Methods
#' and Applications}. Cambridge University Press.
#'
#' Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
#' Robust Estimation and Inference in Sample Selection Models}. R package
#' version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}
#'
#' Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
#' sampleSelection. \emph{Journal of Statistical Software}, 27(7).
#' \url{https://www.jstatsoft.org/article/view/v027i07}
#'
#' Wikipedia contributors (2019). RAND Health Insurance Experiment.
#' \url{https://en.wikipedia.org/wiki/RAND_Health_Insurance_Experiment}
#'
#' @keywords RandHIE
#'
#' @examples
#' ##Cameron and Trivedi (2005): Section 16.6
#' data(RandHIE)
#' subsample <- RandHIE$year == 2 & !is.na( RandHIE$educdec )
#' selectEq <- binexp ~ logc + idp + lpi + fmde + physlm + disea +
#'   hlthg + hlthf + hlthp + linc + lfam + educdec + xage + female +
#'   child + fchild + black
#'   outcomeEq <- lnmeddol ~ logc + idp + lpi + fmde + physlm + disea +
#'   hlthg + hlthf + hlthp + linc + lfam + educdec + xage + female +
#'   child + fchild + black
#'   cameron <- HeckmanCL(selectEq, outcomeEq, data = RandHIE[subsample, ])
#'   summary(cameron)
#'
"RandHIE"

#' Panel Study of Income Dynamics
#'
#' The data come from the Panel Study of Income
#' Dynamics, years 1981 to 1992 (also contains earnings
#' data from 1980). The sample consists of 579 white
#' females, who were followed over the considered period.
#' In total, there are 6,948 observations over the 12-year
#' period (1981-1992). This data frame contains the following
#' columns:
#'\itemize{
#'   \item{id: Individual identifier}
#'   \item{year: Survey year}
#'   \item{age: Calculated age in years (based on year and month of birth)}
#'   \item{educ: Years of schooling}
#'   \item{children: Total number of children in family unit, ages 0-17}
#'   \item{s: Participation dummy, =1 if worked (hours>0)}
#'   \item{lnw: Log of real average hourly earnings}
#'   \item{lnw80: Log earnings in 1980}
#'   \item{agesq: Age squared}
#'   \item{children_lag1: Number of children in t-1}
#'   \item{children_lag2: Number of children in t-2}
#'   \item{lnw2: Log of real average hourly earnings}
#'   \item{Lnw: Log of real average hourly earnings}
#' }
#' @source \url{https://www.icpsr.umich.edu/web/ICPSR/series/131}
#'
#' @references
#' Semykina, A. and Wooldridge, J. M. (2013). Estimation of dynamic panel data
#' models with sample selection. \emph{Journal of Applied Econometrics}, 28(1),
#' 47-61.
#'
#' Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
#' Robust Estimation and Inference in Sample Selection Models}. R package
#' version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}
#'
#' Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
#' sampleSelection. \emph{Journal of Statistical Software}, 27(7).
#' \url{https://www.jstatsoft.org/article/view/v027i07}
#'
#' @examples
#' data(PSID2)
#' attach(PSID2)
#' hist(Lnw)
#' selectEq <- s ~ educ+ age+ children+ year
#' outcomeEq <- Lnw ~ educ+ age+ children
#' HCinitial(selectEq,outcomeEq, data = PSID2)
#' #Note that the estimated value of rho by the two-step
#' #method is greater than 1
#' summary(HeckmanGe(selectEq,outcomeEq, 1, 1, data = PSID2))
#'
"PSID2"

#' US National Health and Nutrition Examination Study
#'
#' The US National Health and Nutrition Examination Study (NHANES) is a
#' survey data collected by the US National Center for Health Statistics.
#' The survey data dates back to 1999, where individuals of all ages are
#' interviewed in their home annually and complete the health examination
#' component of the survey. The study variables include demographic
#' variables (e.g. age and annual household income), physical measurements
#' (e.g. BMI – body mass index), health variables (e.g. diabetes status),
#' and lifestyle variables (e.g. smoking status). This data frame contains
#' the following columns:
#'\itemize{
#'   \item{id: Individual identifier}
#'   \item{age: Age}
#'   \item{gender: Sex 1=male, 0=female}
#'   \item{educ: Education is dichotomized into high school
#'   and above versus less than high school}
#'   \item{race: categorical variable with five levels}
#'   \item{income: Household income ($1000 per year) was reported
#'   as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
#'   and had 10 interval categories. }
#'   \item{Income: Household income ($1000 per year) was reported
#'   as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
#'   and had 10 interval categories. }
#'   \item{bmi: body mass index}
#'   \item{sbp: systolic blood pressure}
#' }
#'
#' @source \url{https://wwwn.cdc.gov/nchs/nhanes/ContinuousNhanes/Default.aspx?BeginYear=2003}
#'
#' @references
#' Ogundimu, E. O. and Collins, G. S. (2019). A robust imputation method for
#' missing responses and covariates in sample selection models. \emph{Statistical
#' Methods in Medical Research}, 28(1), 102-116.
#'
#' Little, R. J. and Zhang, N. (2011). Subsample ignorable likelihood for
#' regression analysis with missing data. \emph{Journal of the Royal Statistical
#' Society: Series C (Applied Statistics)}, 60(4), 591-605.
#'
#' Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
#' Robust Estimation and Inference in Sample Selection Models}. R package
#' version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}
#'
#' Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
#' sampleSelection. \emph{Journal of Statistical Software}, 27(7).
#' \url{https://www.jstatsoft.org/article/view/v027i07}
#'
#' @examples
#' data("nhanes")
#' attach(nhanes)
#' hist(Income, prob= TRUE, breaks = seq(1, 99, 0.5), xlim = c(1,10),
#' ylim = c(0,0.35), main = "Histogram of Income", xlab = "Category")
#' data2 <- subset(nhanes, !is.na(sbp))
#' data3 <- subset(data2, !is.na(bmi))
#' attach(data3)
#' data <- data3
#' data$YS <- ifelse(is.na(data$Income),0,1)
#' data$educ <- ifelse(data$educ<=2,0,1)
#' attach(data)
#' selectionEq <- YS~age+gender+educ+race
#' outcomeEq   <- sbp~age+gender+educ+bmi
#'
"nhanes"

packageStartupMessage("If you have any questions, suggestions, or comments regarding the 'ssmodels' package, please contact: fernando.bastos@ufv.br")


.onLoad <- function(lib, pkg){
   Rdpack::Rdpack_bibstyles(package = pkg, authors = "LongNames")
   invisible(NULL)
}

#' Two-Step Method for Parameter Estimation of the Heckman Model
#'
#' @description
#' Estimates classical Heckman starting values using Heckman's two-step method.
#'
#' @param selection A formula for the selection equation.
#' @param outcome A formula for the outcome equation.
#' @param data A data frame containing the variables.
#'
#' @return A numeric vector containing selection coefficients, outcome coefficients,
#'   sigma and rho.
#'
#' @export
HCinitial <- function(selection, outcome, data = sys.frame(sys.parent())) {
  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  step2(components$YS, components$XS, components$YO, components$XO)
}

#' Extract model components from formulas
#'
#' @description
#' This function extracts model matrices and response vectors used by the
#' sample-selection model fitting functions. It standardizes the selection
#' response to a numeric indicator, where 1 denotes selected/observed and
#' 0 denotes not selected/not observed.
#'
#' @param selection A formula representing the selection equation.
#' @param outcome A formula representing the outcome equation.
#' @param data A data frame containing all variables used in the formulas.
#' @param outcomeS Optional matrix, vector, one-sided formula, or 1 for the
#'   log-scale model in the generalized Heckman model.
#' @param outcomeC Optional matrix, vector, one-sided formula, or 1 for the
#'   correlation model in the generalized Heckman model.
#' @param drop.levels Logical; if TRUE, unused factor levels are dropped.
#'
#' @return A list with selection, outcome, dispersion and correlation matrices.
#'
#' @importFrom stats model.frame model.matrix model.response terms na.pass
#' @export
extract_model_components <- function(selection, outcome, data,
                                     outcomeS = NULL, outcomeC = NULL,
                                     drop.levels = TRUE) {

  make_aux_matrix <- function(aux, data, n, prefix) {
    if (is.null(aux)) {
      mat <- matrix(1, nrow = n, ncol = 1)
      colnames(mat) <- "(Intercept)"
      return(mat)
    }

    if (length(aux) == 1L && is.numeric(aux)) {
      mat <- matrix(as.numeric(aux), nrow = n, ncol = 1)
      colnames(mat) <- "(Intercept)"
      return(mat)
    }

    if (inherits(aux, "formula")) {
      mf <- model.frame(formula = aux, data = data,
                        drop.unused.levels = drop.levels,
                        na.action = na.pass)
      mat <- model.matrix(terms(mf), mf)
      return(mat)
    }

    if (is.data.frame(aux) || is.matrix(aux)) {
      mat <- as.matrix(aux)
    } else {
      mat <- matrix(aux, ncol = 1)
      colnames(mat) <- deparse(substitute(aux))
    }

    if (nrow(mat) != n) {
      stop(prefix, " must have the same number of rows as data.")
    }

    if (is.null(colnames(mat))) {
      colnames(mat) <- paste0(prefix, seq_len(ncol(mat)))
    }

    first_is_intercept <- ncol(mat) >= 1L &&
      all(is.finite(mat[, 1])) && all(abs(mat[, 1] - 1) < 1e-12)

    if (!first_is_intercept) {
      mat <- cbind("(Intercept)" = 1, mat)
    }

    mat
  }

  mfS <- model.frame(
    formula = selection,
    data = data,
    drop.unused.levels = drop.levels,
    na.action = na.pass
  )
  mtS <- terms(mfS)
  XS <- model.matrix(mtS, mfS)
  YS_raw <- model.response(mfS)

  YS_factor <- as.factor(YS_raw)
  YSLevels <- levels(YS_factor)
  if (length(YSLevels) != 2L) {
    stop("The selection equation must have exactly two observed levels.")
  }

  if (is.numeric(YS_raw) && all(stats::na.omit(unique(YS_raw)) %in% c(0, 1))) {
    YS <- as.integer(YS_raw)
  } else {
    YS <- as.integer(YS_factor == YSLevels[2L])
  }

  mfO <- model.frame(
    formula = outcome,
    data = data,
    drop.unused.levels = drop.levels,
    na.action = na.pass
  )
  mtO <- terms(mfO)
  XO <- model.matrix(mtO, mfO)
  YO <- model.response(mfO)

  Msigma <- make_aux_matrix(outcomeS, data, nrow(XS), "outcomeS")
  Mrho   <- make_aux_matrix(outcomeC, data, nrow(XS), "outcomeC")

  if (nrow(XO) != nrow(XS) || length(YO) != length(YS)) {
    stop("Selection and outcome equations must refer to the same number of rows.")
  }

  keep <- !is.na(YS) & stats::complete.cases(XS)
  keep <- keep & (YS == 0 | (!is.na(YO) & stats::complete.cases(XO)))
  keep <- keep & (YS == 0 | (stats::complete.cases(Msigma) & stats::complete.cases(Mrho)))

  XS <- XS[keep, , drop = FALSE]
  YS <- YS[keep]
  XO <- XO[keep, , drop = FALSE]
  YO <- YO[keep]
  Msigma <- Msigma[keep, , drop = FALSE]
  Mrho <- Mrho[keep, , drop = FALSE]

  storage.mode(YS) <- "integer"

  list(
    XS = XS,
    YS = YS,
    NXS = ncol(XS),
    XO = XO,
    YO = YO,
    NXO = ncol(XO),
    Msigma = Msigma,
    NE = ncol(Msigma),
    Mrho = Mrho,
    NV = ncol(Mrho),
    YSLevels = YSLevels,
    kept = keep
  )
}

test_that("HeckmanSK analytical gradient agrees with numDeriv at MEPS2001 fit", {
  testthat::skip_on_cran()
  testthat::skip_if_not_installed("numDeriv")
  testthat::skip_if_not_installed("sn")

  data("MEPS2001", package = "ssmodels")
  dados <- as.data.frame(MEPS2001)
  dados$dambexp_num <- as.integer(dados$dambexp == 1)
  dados$lnambx_obs  <- ifelse(dados$dambexp_num == 1, dados$lnambx, NA_real_)

  selectEq  <- dambexp_num ~ age + female + educ + blhisp + totchr + ins + income
  outcomeEq <- lnambx_obs  ~ age + female + educ + blhisp + totchr + ins

  fit <- ssmodels::HeckmanSK(selection = selectEq, outcome = outcomeEq, data = dados)

  comp <- ssmodels::extract_model_components(selection = selectEq,
                                             outcome = outcomeEq,
                                             data = dados)
  comp$YO[is.na(comp$YO)] <- 0

  sk_loglik <- function(theta, comp) {
    XS <- comp$XS
    XO <- comp$XO
    YS <- comp$YS
    YO <- comp$YO
    NXS <- ncol(XS)
    NXO <- ncol(XO)

    gamma <- theta[seq_len(NXS)]
    beta <- theta[seq(from = NXS + 1L, length.out = NXO)]
    sigma <- theta[NXS + NXO + 1L]
    rho <- theta[NXS + NXO + 2L]
    lambda <- theta[NXS + NXO + 3L]

    if (!is.finite(sigma) || sigma <= 0) return(NA_real_)
    if (!is.finite(rho) || abs(rho) >= 1) return(NA_real_)
    if (!is.finite(lambda)) return(NA_real_)

    eta <- as.numeric(XS %*% gamma)
    mu <- as.numeric(XO %*% beta)
    z <- (YO - mu) / sigma
    r <- 1 - rho^2
    u <- 1 + lambda^2 - (lambda * rho)^2
    q <- lambda * rho / sqrt(u)
    omega <- (eta + rho * z) / sqrt(r)

    ll_selected <- log(2) - log(sigma) +
      stats::dnorm(z, log = TRUE) +
      stats::pnorm(lambda * z, log.p = TRUE) +
      stats::pnorm(omega, log.p = TRUE)

    F0 <- sn::psn(-eta, xi = 0, omega = 1, alpha = q)
    F0 <- pmax(F0, .Machine$double.xmin)
    ll_censored <- log(F0)

    sum(ifelse(YS == 1, ll_selected, ll_censored))
  }

  theta_hat <- fit$coefficients
  g_num <- numDeriv::grad(func = sk_loglik, x = theta_hat, comp = comp,
                          method = "Richardson")

  gradient_error <- max(abs(as.numeric(fit$gradient) - as.numeric(g_num)))

  testthat::expect_true(all(is.finite(fit$gradient)))
  testthat::expect_lt(gradient_error, 1e-4)
})

#' Normal Skew Model fit Function
#'
#' @description
#' Estimates the parameters of the Sample Selection Model with Skew-Normal
#' Distribution
#'
#' @return
#'
#' Returns a list with the following components.
#'
#' Coefficients: Returns a numerical vector with the best estimated values
#' of the model parameters;
#'
#' Value: The value of function to be minimized (or maximized) corresponding
#' to par.
#'
#' loglik: Maximized value of the log-likelihood function calculated from the estimated coefficients.
#'
#' gradient: Analytical score vector evaluated at the estimated coefficients.
#'
#' counts: Component of the Optim function. A two-element integer vector
#' giving the number of calls to fn and gr respectively. This excludes
#' those calls needed to compute the Hessian, if requested, and any calls
#' to fn to compute a finite-difference approximation to the gradient.
#'
#' hessian: Component of the Optim function, with pre-defined option
#' hessian=TRUE. A symmetric matrix giving an estimate of the Hessian
#' at the solution found. Note that this is the Hessian of the unconstrained
#' problem even if the box constraints are active.
#'
#' fisher_infoSK: Fisher information matrix
#'
#' prop_sigmaSK: Square root of the Fisher information matrix diagonal
#'
#' level: Selection variable levels
#'
#' nObs: Numeric value representing the size of the database
#'
#' nParam: Numerical value representing the number of model parameters
#'
#' N0: Numerical value representing the number of unobserved entries
#'
#' N1: Numerical value representing the number of complete entries
#'
#' NXS: Numerical value representing the number of parameters of the
#' selection model
#'
#' NXO: Numerical value representing the number of parameters of the
#' regression model
#'
#' df: Numerical value that represents the difference between the size
#' of the response vector of the selection equation and the number of
#' model parameters
#'
#' aic: Numerical value representing Akaike's information criterion.
#'
#' bic: Numerical value representing Schwarz's Bayesian Criterion
#'
#' initial.value: Numerical vector that represents the input values
#' (Initial Values) used in the parameter estimation.
#'
#' @details
#' The HeckmanSK() function fits the Sample Selection Model
#' based on the Skew-normal distribution. For more information see Ogundimu
#' and Hutton (2016) \doi{10.1111/sjos.12171}.
#'
#'
#' @param selection Selection equation.
#' @param outcome Primary Regression Equation.
#' @param lambda Numeric scalar or vector with initial values for the skewness parameter.
#' If more than one value is supplied and \code{start = NULL}, the function uses
#' a multi-start strategy and keeps the fit with the largest log-likelihood.
#' @param start initial values.
#' @param data Database.
#' @examples
#' data("Mroz87")
#' attach(Mroz87)
#' selectEq <- lfp ~ huswage + kids5 + mtr + fatheduc + educ + city
#' outcomeEq <- log(wage) ~ educ+city
#' HeckmanSK(selectEq, outcomeEq, data = Mroz87, lambda = -1.5)
#' @importFrom Rdpack reprompt
#' @references Ogundimu, E. O. and Hutton, J. L. (2016). A sample
#' selection model with skew-normal distribution. \emph{Scandinavian
#' Journal of Statistics}. \doi{10.1111/sjos.12171}.
#' @export HeckmanSK
#' @export
HeckmanSK <- function(selection, outcome, data = sys.frame(sys.parent()),
                      lambda = c(-5, -3, -2, -1, -0.5, 0, 0.5, 1, 2, 3, 5),
                      start = NULL) {

  # Extract design matrices and outcomes from the selection and outcome models
  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  XS  <- components$XS       # Design matrix for selection equation
  YS  <- components$YS       # Response variable for selection equation
  NXS <- components$NXS      # Number of selection parameters
  XO  <- components$XO       # Design matrix for outcome equation
  YO  <- components$YO       # Response variable for outcome equation
  NXO <- components$NXO      # Number of outcome parameters
  YSLevels <- components$YSLevels  # Levels of the selection variable

  # Replace missing values in the outcome with zeros
  YO[is.na(YO)] <- 0

  #############################
  # Log-likelihood function
  #############################
  loglik_SK <- function(start) {
    # Define parameter indices
    NXS <- dim(model.matrix(~XS))[2] - 1
    NXO <- dim(model.matrix(~XO))[2] - 1
    istartS <- 1:NXS
    istartO <- seq(tail(istartS, 1) + 1, length = NXO)
    isigma  <- tail(istartO, 1) + 1
    irho    <- tail(isigma, 1) + 1
    ilamb1  <- tail(irho, 1) + 1

    # Extract parameters
    g      <- start[istartS]
    b      <- start[istartO]
    sigma  <- start[isigma]
    if (!is.finite(sigma) || sigma <= 0) return(NA_real_)
    rho    <- start[irho]
    if (!is.finite(rho) || abs(rho) >= 1) return(NA_real_)
    lamb1  <- start[ilamb1]

    # Linear predictors
    XS.g <- XS %*% g
    XO.b <- XO %*% b
    u2   <- YO - XO.b
    z    <- u2 / sigma
    r    <- sqrt(1 - rho^2)
    u    <- 1 + lamb1^2 - (lamb1 * rho)^2
    lstar <- -lamb1 * rho / sqrt(u)

    # Compute selection log-likelihood
    h <- pmax(sn::psn(-XS.g, 0, 1, -lstar), .Machine$double.xmin)

    # Log-likelihood vector
    ll <- ifelse(YS == 0,
                 log(h),
                 log(2) - log(sigma) + stats::dnorm(z, log = TRUE) +
                   stats::pnorm(lamb1 * z, log.p = TRUE) +
                   stats::pnorm((XS.g + rho * z) / r, log.p = TRUE))

    return(sum(ll))
  }

  #############################
  # Gradient of the log-likelihood
  #############################
  gradlik_SK <- function(start) {
    # Dimensions and parameter positions
    NXS <- ncol(XS)
    NXO <- ncol(XO)
    nObs <- length(YS)
    nParam <- NXS + NXO + 3

    istartS <- seq_len(NXS)
    istartO <- seq(from = NXS + 1L, length.out = NXO)
    isigma  <- NXS + NXO + 1L
    irho    <- NXS + NXO + 2L
    ilamb1  <- NXS + NXO + 3L

    # Extract parameters
    g     <- start[istartS]
    b     <- start[istartO]
    sigma <- start[isigma]
    rho   <- start[irho]
    lamb1 <- start[ilamb1]

    if (!is.finite(sigma) || sigma <= 0) {
      return(rep(NA_real_, nParam))
    }
    if (!is.finite(rho) || abs(rho) >= 1) {
      return(rep(NA_real_, nParam))
    }
    if (!is.finite(lamb1)) {
      return(rep(NA_real_, nParam))
    }

    # Subsets
    sel0 <- YS == 0
    sel1 <- YS == 1

    XS0 <- XS[sel0, , drop = FALSE]
    XS1 <- XS[sel1, , drop = FALSE]
    XO1 <- XO[sel1, , drop = FALSE]
    YO1 <- YO[sel1]

    # Linear predictors and auxiliary quantities
    XS0.g <- as.numeric(XS0 %*% g)
    XS1.g <- as.numeric(XS1 %*% g)
    XO1.b <- as.numeric(XO1 %*% b)

    z <- (YO1 - XO1.b) / sigma
    r <- 1 - rho^2
    u <- 1 + lamb1^2 - (lamb1 * rho)^2

    # The censored contribution is log F_SN(-eta; q), with
    # q = lambda * rho / sqrt(u). In the notation used in the paper,
    # lambda* = -q is used for P(S = 1), whereas P(S = 0) uses q.
    q <- lamb1 * rho / sqrt(u)

    # Selected observations: K1 = phi(omega)/Phi(omega),
    # K3 = phi(lambda z)/Phi(lambda z).
    omega <- (XS1.g + rho * z) / sqrt(r)
    eta   <- lamb1 * z

    K1 <- exp(stats::dnorm(omega, log = TRUE) -
                stats::pnorm(omega, log.p = TRUE))
    K3 <- exp(stats::dnorm(eta, log = TRUE) -
                stats::pnorm(eta, log.p = TRUE))

    # Censored observations: F0 = F_SN(-eta_s; q),
    # K2 = phi(eta_s) Phi(-q eta_s) / F_SN(-eta_s; q),
    # K4 = phi(eta_s sqrt(1 + lambda^2) / sqrt(u)) / F_SN(-eta_s; q).
    F0 <- sn::psn(-XS0.g, xi = 0, omega = 1, alpha = q)
    F0 <- pmax(F0, .Machine$double.xmin)

    K2 <- exp(stats::dnorm(XS0.g, log = TRUE) +
                stats::pnorm(-q * XS0.g, log.p = TRUE) -
                log(F0))

    K4 <- exp(stats::dnorm(XS0.g * sqrt(1 + lamb1^2) / sqrt(u), log = TRUE) -
                log(F0))

    gradient <- matrix(0, nrow = nObs, ncol = nParam)

    # d ell / d gamma
    # Correct selected term: K1 / sqrt(r), not K1 / r.
    if (nrow(XS1) > 0L) {
      gradient[sel1, istartS] <- XS1 * as.numeric(K1 / sqrt(r))
    }
    if (nrow(XS0) > 0L) {
      gradient[sel0, istartS] <- XS0 * as.numeric(-2 * K2)
    }

    # d ell / d beta
    if (nrow(XO1) > 0L) {
      gradient[sel1, istartO] <- XO1 * as.numeric(
        z / sigma - (lamb1 * K3) / sigma - (rho * K1) / (sigma * sqrt(r))
      )
    }

    # d ell / d sigma
    if (sum(sel1) > 0L) {
      gradient[sel1, isigma] <-
        -1 / sigma +
        z^2 / sigma -
        lamb1 * z * K3 / sigma -
        rho * z * K1 / (sigma * sqrt(r))
    }

    # d ell / d rho
    if (sum(sel1) > 0L) {
      gradient[sel1, irho] <- K1 * (z + rho * XS1.g) / (r^(3 / 2))
    }
    if (sum(sel0) > 0L) {
      gradient[sel0, irho] <- -2 * lamb1 * K4 / sqrt(2 * pi * u)
    }

    # d ell / d lambda
    # Correct censored term: (1 + lambda^2) is in the denominator.
    if (sum(sel1) > 0L) {
      gradient[sel1, ilamb1] <- K3 * z
    }
    if (sum(sel0) > 0L) {
      gradient[sel0, ilamb1] <-
        -2 * rho * K4 / ((1 + lamb1^2) * sqrt(2 * pi * u))
    }

    colSums(gradient)
  }

  #############################
  # Initial values and multi-start strategy
  #############################

  # Internal optimizer used for a single starting vector.
  fit_one_SK <- function(start_i) {
    tryCatch(
      stats::optim(
        par     = start_i,
        fn      = loglik_SK,
        gr      = gradlik_SK,
        method  = "BFGS",
        hessian = TRUE,
        control = list(fnscale = -1)
      ),
      error = function(e) {
        structure(
          list(error = conditionMessage(e), par = start_i, value = NA_real_),
          class = "try-error"
        )
      }
    )
  }

  if (is.null(start)) {
    message("Start not provided; using multi-start initial values for lambda.")

    lambda_grid <- unique(as.numeric(lambda))
    lambda_grid <- lambda_grid[is.finite(lambda_grid)]

    if (length(lambda_grid) == 0L) {
      stop("Argument 'lambda' must contain at least one finite numeric value.")
    }

    start_base <- step2(YS, XS, YO, XO)
    start_list <- lapply(lambda_grid, function(lam) c(start_base, lam))

    fit_list <- lapply(start_list, fit_one_SK)

    fit_values <- vapply(fit_list, function(fit) {
      if (inherits(fit, "try-error") || is.null(fit$value) || !is.finite(fit$value)) {
        return(NA_real_)
      }
      fit$value
    }, numeric(1))

    if (all(is.na(fit_values))) {
      stop("All HeckmanSK multi-start optimizations failed. Try supplying 'start' manually.")
    }

    best_id <- which.max(fit_values)
    theta_SK <- fit_list[[best_id]]
    start <- start_list[[best_id]]

    lambda_search <- data.frame(
      lambda_start = lambda_grid,
      logLik       = fit_values,
      convergence  = vapply(fit_list, function(fit) {
        if (inherits(fit, "try-error") || is.null(fit$convergence)) return(NA_integer_)
        fit$convergence
      }, integer(1)),
      message      = vapply(fit_list, function(fit) {
        if (inherits(fit, "try-error")) return(fit$error)
        if (is.null(fit$message)) return(NA_character_)
        as.character(fit$message)
      }, character(1)),
      sigma_hat    = vapply(fit_list, function(fit) {
        if (inherits(fit, "try-error") || is.null(fit$par) || length(fit$par) < NXS + NXO + 3) return(NA_real_)
        fit$par[NXS + NXO + 1]
      }, numeric(1)),
      rho_hat      = vapply(fit_list, function(fit) {
        if (inherits(fit, "try-error") || is.null(fit$par) || length(fit$par) < NXS + NXO + 3) return(NA_real_)
        fit$par[NXS + NXO + 2]
      }, numeric(1)),
      lambda_hat   = vapply(fit_list, function(fit) {
        if (inherits(fit, "try-error") || is.null(fit$par) || length(fit$par) < NXS + NXO + 3) return(NA_real_)
        fit$par[NXS + NXO + 3]
      }, numeric(1))
    )

  } else {
    start <- as.numeric(start)

    expected_length <- NXS + NXO + 3
    if (length(start) != expected_length) {
      stop("Argument 'start' must have length ", expected_length,
           " = NXS + NXO + 3.")
    }

    theta_SK <- fit_one_SK(start)

    if (inherits(theta_SK, "try-error") || !is.finite(theta_SK$value)) {
      stop("HeckmanSK optimization failed from the supplied 'start' values.")
    }

    lambda_search <- data.frame(
      lambda_start = start[expected_length],
      logLik       = theta_SK$value,
      convergence  = theta_SK$convergence,
      message      = if (is.null(theta_SK$message)) NA_character_ else as.character(theta_SK$message),
      sigma_hat    = theta_SK$par[NXS + NXO + 1],
      rho_hat      = theta_SK$par[NXS + NXO + 2],
      lambda_hat   = theta_SK$par[NXS + NXO + 3]
    )
  }

  #############################
  # Organize and return results
  #############################
  names(theta_SK$par) <- c(colnames(XS), colnames(XO), "sigma", "rho", "lambda")

  result <- list(
    coefficients   = theta_SK$par,
    value          = theta_SK$value,
    loglik         = theta_SK$value,
    gradient       = gradlik_SK(theta_SK$par),
    counts         = theta_SK$counts[2],
    hessian        = theta_SK$hessian,
    fisher_infoSK  = .ssmodels_safe_vcov(theta_SK$hessian),
    prop_sigmaSK   = .ssmodels_safe_se(.ssmodels_safe_vcov(theta_SK$hessian)),
    level          = YSLevels,
    nObs           = length(YS),
    nParam         = length(start),
    N0             = sum(YS == 0),
    N1             = sum(YS == 1),
    NXS            = ncol(XS),
    NXO            = ncol(XO),
    df             = length(YS) - length(start),
    aic            = -2 * theta_SK$value + 2 * length(start),
    bic            = -2 * theta_SK$value + length(start) * log(length(YS)),
    initial.value  = start,
    lambda_search  = lambda_search,
    lambda_start   = start[length(start)],
    multi_start    = nrow(lambda_search) > 1L
  )

  class(result) <- c("HeckmanSK", class(theta_SK))
  return(result)
}

#' Heckman's two-step method
#'
#' @description
#' Estimate the classical Heckman model parameters by the two-step method.
#'
#' @param selection A formula for the selection equation.
#' @param outcome A formula for the outcome equation.
#' @param data A data frame containing the variables used in the model.
#'
#' @return A numeric vector with the two-step estimates.
#'
#' @importFrom Rdpack reprompt
#' @references Heckman, J. J. (1976). The common structure of statistical
#' models of truncation, sample selection and limited dependent variables and
#' a simple estimator for such models. \emph{Annals of Economic and Social
#' Measurement}, 5(4), 475-492.
#'
#' Heckman, J. J. (1979). Sample selection bias as a specification error.
#' \emph{Econometrica}, 47, 153-161.
#' @export
twostep <- function(selection, outcome, data = sys.frame(sys.parent())) {
  components <- extract_model_components(selection = selection, outcome = outcome, data = data)
  step2(components$YS, components$XS, components$YO, components$XO)
}

---
title: "ssmodels: A package for fit the sample selection models"
output: 
  rmarkdown::html_vignette:
    fig_caption: yes
    force_captions: yes
    #highlight: pygments
    toc: yes
    #Sumário flutuante
    #toc_float: true
    #numerar seções
    number_sections: true
    #Mostrar ou esconder os códigos (show ou hide)
    #code_folding: hide
    #Diversos modelos de documentos ver outros em http://bootswatch.com/
    #theme: united
vignette: >
  %\VignetteIndexEntry{ssmodels: A package for fit the sample selection models}
  %\VignetteEngine{knitr::rmarkdown}
bibliography: "../inst/REFERENCES.bib"
biblio-style: "apalike"
---

```{r setup, include=FALSE}
library(ssmodels)
library(ggplot2)
require(knitr)
require(kfigr)
library(kableExtra)
options(knitr.table.format = "latex")
knitr::opts_chunk$set(echo = TRUE, fig.align = "center", message=FALSE, 
                      warning=FALSE, fig.height=5, fig.width=7.2)
```

# Introduction


<p style="text-align: justify;">
In order to facilitate the adjustment of the sample selection models existing in the literature, we created the ssmodels package. Our package allows the adjustment of the classic Heckman model (@heckman1976common, @heckman1979sample), and the estimation of the parameters of this model via the maximum likelihood method and two-step method, in addition to the adjustment of the Heckman-t models, introduced in the literature by @marchenko2012heckman and the Heckman-Skew model introduced in the literature by @ogundimu2016sample. We also implemented functions to adjust the generalized version of the Heckman model, introduced by @bastosBarreto, that allows the inclusion of covariables to the dispersion and correlation parameters and a function to adjust the Heckman-BS model introduced by @bastos that uses the Birnbaum-Saunders distribution as a joint distribution of the selection and primary regression variables.
</p>

# Real Databases

## MEPS 2001
<p style="text-align: justify;">
The MEPS is a set of large-scale surveys of families, individuals and their medical providers (doctors, hospitals, pharmacies, etc.) in the United States. It has data on the health services Americans use, how often they use them, the cost of these services and how they are paid, as well as data on the cost and reach of health insurance available to American workers. The sample is restricted to persons aged between 21 and 64 years and contains a variable response with 3328 observations of outpatient costs, of which 526 (15.8%) correspond to unobserved expenditure values and identified as zero expenditure for adjustment of the models. It also includes the following explanatory variables:
</p>
* educ: education status
* age: Age
* income: income
* female: gender
* vgood: a numeric vector
* good: a numeric vector
* hospexp: a numeric vector
* totchr: number of chronic diseases
* ffs: a numeric vector
* dhospexp: a numeric vector
* age2: a numeric vector
* agefem: a numeric vector
* fairpoor: a numeric vector
* year01: a numeric vector
* instype: a numeric vector
* ambexp: a numeric vector
* lambexp: log ambulatory expenditures
* blhisp: ethnicity
* instype_s1: a numeric vector
* dambexp: dummy variable, ambulatory expenditures
* lnambx: a numeric vector
* ins: insurance status
<p style="text-align: justify;">
These data were also used by @colin2009microeconometrics, @marchenko2012heckman and @zhelonkin2016robust to fit the classical Heckman and Heckman-t models and the robust version of the two-step method, respectively. We use the variable of interest $Y_{1}^{*}={\rm ambexp}$, which represents expenditure on medical services, on the logarithmic scale, since it is highly asymmetric, see `r figr("fig4", TRUE, type="Figura")`. The variable $Y_{2}^{*}={\rm dambexp},$ representing the willingness to spend, is not observed. We observe $U=1\{Y_{2}^{*}>0\},$ which represents the decision to spend on medical care.
</p>
```{r fig4, fig.align= "center", fig.width = 7.2, fig.height= 5, anchor="Figura", fig.alt= "Log of Expenditures Medical"}
library(ssmodels)
#Leitura do dados MEPS2001
data(MEPS2001)
#tornando visiveis as colunas do data-frame
attach(MEPS2001)
barfill <- "grey"
barlines <- "black"
p1 <- ggplot(MEPS2001,aes(ambexp))+geom_histogram(colour = barlines, fill = barfill)+
  scale_x_continuous(name = "(a) Expenditures Medical",
                     breaks = seq(0, 15000, 2500),
                     limits=c(0, 15000))+
  scale_y_continuous(name = "Count",
                     breaks = seq(0, 800, 100),
                     limits=c(0, 800))
p2 <- ggplot(MEPS2001,aes(lambexp))+geom_histogram(colour = barlines, fill = barfill)+
  scale_x_continuous(name = "(b) Log of Expenditures Medical",
                     breaks = seq(0, 11, 1),
                     limits=c(0, 11))+
  scale_y_continuous(name = "Count",
                     breaks = seq(0, 300, 100),
                     limits=c(0, 300))
gridExtra::grid.arrange(p1, p2, ncol=2)
```
<p style="text-align: justify;">
According @marchenko2012heckman and @zhelonkin2016robust it is natural to fit a sample selection model to such data, since the willingness to spend $(Y_{2}^{*})$ is likely to be related to the expense amount $(Y_{1}^{*}).$ However, after fitting the classic Heckman model and using the Wald, likelihood ratio, or gradient tests for 
$H_{0}:\rho=0$ against $H_{1}:\rho\neq 0,$ the conclusion is that there is not sufficient evidence $(p\textrm{-valor}>0.1)$ to reject $H_{0},$ that is, there is no bias of selection. @colin2009microeconometrics suspected this conclusion and @marchenko2012heckman argued that a more robust model could evidence the 
selection bias present in the data and rejected the normality hypothesis of the data. However, we understand that the adjustment of the dispersion and/or the correlation can be an alternative to study these data without the need to modify the assumption of normality of the errors, because as our simulations indicate, the estimations of the parameters obtained by the adjustment of the model Heckman can be more severely affected by heteroscedasticity than by incorrect distribution of error terms.
</p>
<p style="text-align: justify;">
However, due to data asymmetry, we also adjusted the Heckman-BS model. Thus, we consider the following system of equations for classic and generalized Heckman models, Heckman-t and Heckman-Skew:
</p>
\begin{align}
lnambx &= \beta_{1}+ \beta_{2}age + \beta_{3}female + \beta_{4}educ + \beta_{5}blhisp + \beta_{6}totchr + \beta_{7}ins + \epsilon_{1i}, \label{eq_reg_aplic1} \\
dambexp&= \gamma_{1}+\gamma_{2}age + \gamma_{3}female + \gamma_{4}educ + \gamma_{5}blhisp + \gamma_{6}totchr + \gamma_{7}ins + \gamma_{8}income + \epsilon_{2i}, \label{eq_sel_aplic1} 
\end{align}
where errors are defined for classic Heckman models, Heckman-t and Heckman-Skew according to 
\begin{align}\label{2:disterro}
\begin{pmatrix}\epsilon_{1i}\\
\epsilon_{2i}
\end{pmatrix} &\overset{ind.}{\sim} \mathcal{N}
\begin{bmatrix}
\begin{pmatrix}
0\\
0
\end{pmatrix}\!\!,&
\begin{pmatrix}
\sigma^{2} & \rho\sigma \\
\sigma\rho & 1
\end{pmatrix}
\end{bmatrix}, i=1,\cdots,n,
\end{align}
and for generalized Heckman models how
\begin{align}\label{2:disterro1}
\begin{pmatrix}\epsilon_{1i}\\
\epsilon_{2i}
\end{pmatrix} &\overset{ind.}{\sim} \mathcal{N}
\begin{bmatrix}
\begin{pmatrix}
0\\
0
\end{pmatrix}\!\!,&
\begin{pmatrix}
\sigma_{i}^{2} & \rho_{i}\sigma_{i} \\
\sigma_{i}\rho_{i} & 1
\end{pmatrix}
\end{bmatrix}, i=1,\cdots,n,
\end{align}
in such a way that
\begin{align*}
\log{(\sigma_{i})} &= \lambda_{1}+\lambda_{2}age + \lambda_{3}female + \lambda_{4}totchr + \lambda_{5}ins\quad \textrm{and}\\ 
\rho_{i}&=\tanh{(\kappa_{0})}, i=1,\cdots, 3328.
\end{align*}

To adjust the Heckman-BS model we consider
\begin{align*}
\log\mu_{1i} &= \beta_{1} + \beta_{2}age + \beta_{3}female  + \beta_{4}educ + \beta_{5}blhisp + \beta_{6}totchr + \beta_{7}ins,\\
\log\mu_{2i} &= \gamma_{1}+\gamma_{2}age + \gamma_{3}female + \gamma_{4}educ + \gamma_{5}blhisp + 
\gamma_{6}totchr + \gamma_{7}ins + \gamma_{8}income,\quad i=1,2,\cdots,3328. 
\end{align*}
<p style="text-align: justify;">
and $(ambexp,dambexp)$ is independently distributed with Birnbaum-Saunders (BS) Bivariate parameter distribution $\mu_{1}>0, \mu_{2}>0, \phi_{1}=\phi>0, \phi_{2}=1$ and $-1<\rho<1,$ that is, 
$$(ambexp_{i},dambexp_{i})\stackrel{ind}{\sim}\mbox{ BS}(\mu_{1i},\mu_{2i}, 
\phi,1,\rho), \mu_{1i}>0, \mu_{2i}>0, \phi>0\quad \textrm{and}\quad -1<\rho<1, i=1,\cdots, n$$
</p>
<p style="text-align: justify;">
We observed similar values for the parameters estimated by all models, mainly the parameters of the variable of interest. However, the only model that allows a direct interpretation of such parameters is the Heckman-BS model. We can, for example, affirm, based on the result of this model, that keeping the other parameters fixed, changing a unit in age represents an increase of $24.6\%$ in ambulatory expenses. For the other models, the interpretation is related to the log of outpatient expenses. See the results of the estimates of the parameters below:
</p>
```{r, warning=FALSE}
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins 
outcomeS <- cbind(age,female,totchr,ins)
outcomeC <- 1
outcomeBS <- ambexp ~ age + female + educ + blhisp + totchr + ins 
mCL <- HeckmanCL(selectEq, outcomeEq, data = MEPS2001)
mBS <- HeckmanBS(selectEq, outcomeBS, data = MEPS2001)
mSK <- HeckmanSK(selectEq, outcomeEq, data = MEPS2001,lambda = 1)
mtS <- HeckmantS(selectEq, outcomeEq, data = MEPS2001,df=12)
mGe <- HeckmanGe(selectEq, outcomeEq,outcomeS, outcomeC, data = MEPS2001)
Parameters <- c("Intercept", "age", "female", "educ", "blhisp", "totchr", "ins", "income",
   "Intercept", "age", "female", "educ", "blhisp", "totchr", "ins", "sigma", "age", "female", "totchr", "ins", "rho", "nu", "lambda")
HBS <- round(mBS$coefficients, digits = 3)
HCL <- round(mCL$coefficients, digits = 3)
HSK <- round(mSK$coefficients, digits = 3)
HtS <- round(mtS$coefficients, digits = 3)
HGe <- round(mGe$coefficients, digits = 3)
Results <- data.frame("Parameters"= Parameters,
                      "HeckmanGe" = c(HGe[1:21], "NA", "NA"), 
                      "HeckmanCL" = c(HCL[1:16], "NA", "NA", "NA", "NA", HCL[17], "NA", "NA"),
                      "HeckmanBS" = c(HBS[1:16], "NA", "NA", "NA", "NA", HBS[17], "NA", "NA"), 
                      "HeckmantS" = c(HtS[1:16], "NA", "NA", "NA", "NA", HtS[17:18], "NA" ),
                      "HeckmanSK" = c(HSK[1:16], "NA", "NA", "NA", "NA", HSK[17], "NA", HSK[18]))
kable(Results, format = "html", align = c("c", "c", "c", "c", "c"))
```
<p style="text-align: justify;">
In addition, there are also the results of the significance of the parameters. The classic Heckman model adjusted to such data indicates no correlation between the variables value spent and the decision to spend. According to @colin2009microeconometrics, such a result is suspect and should be better analyzed. 
</p>
```{r, warning=FALSE}
summary(mCL)
```
<p style="text-align: justify;">
The generalized Heckman model, on the other hand, indicates that there is the presence of heteroscedasticity in the data and, probably, due to this, the classic Heckman model is not the most suitable for modeling these data. Our model, in addition to indicating the presence of variable dispersion, also indicates a significant correlation between the variables amount spent and the decision to spend.
</p>
```{r, warning=FALSE}
summary(mGe)
```
<p style="text-align: justify;">
On the other hand, as the observed data show asymmetry, the Heckman-BS model can be a good indication to adjust such data. Note that the Heckman-BS model also indicates the presence of a significant correlation. It also presents, as previously mentioned, the advantage of parsimony and the direct interpretation of the relationship of the parameters with the response variable of interest.
</p>
```{r, warning=FALSE}
summary(mBS)
```
<p style="text-align: justify;">
The Heckman-t model, even adjusted to the expense log, indicates the presence of deviation from normality, since the degree of freedom parameter is approximately equal to $13$ and is significant. Such a model also observes the significant correlation parameter.
</p>
```{r, warning=FALSE}
summary(mtS)
```
<p style="text-align: justify;">
Finally, the Heckman-Skew model also indicates the presence of a significant correlation between the variables of interest and selection. In addition, it also indicates deviation from the normality of the transformed data. It is important to note that without the data transformation, the iterative method of estimating the parameters of the Heckman-Skew model does not converge.
</p>
```{r, warning=FALSE}
summary(mSK)
```


## Nhanes 2003-2004
<p style="text-align: justify;">
The US National Health and Nutrition Examination Study (NHANES) is a survey data collected by the US National Center for Health Statistics. The survey data dates back to 1999, where individuals of all ages are interviewed in their home annually and complete the health examination component of the survey. The study variables include demographic variables (e.g. age and annual household income), physical measurements (e.g. BMI – body mass index), health variables (e.g. diabetes status), and lifestyle variables (e.g. smoking status). This data frame contains the following columns:
</p>
* id: Individual identifier
* age: Age
* gender: Sex 1=male, 0=female
* educ: Education is dichotomized into high school and above versus less than high school
* race: categorical variable with five levels
* income: Household income ($1000 per year) was reported as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
* and had 10 interval categories. 
*Income: Household income ($1000 per year) was reported as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
* and had 10 interval categories. 
* bmi: body mass index
* sbp: systolic blood pressure

```{r, warning=FALSE}
library(ssmodels)
data(nhanes)
attach(nhanes)
perc <- function(x,data){
    nna <- ifelse(sum(is.na(x))!=0,summary(x)[[7]],0)
    perc <- ifelse(sum(is.na(x))!=0,(nna/length(data$id))*100,0)
   return(perc)
}
Variables <- c("SBP (mm Hg)", "Age (year)", "Gender", "BMI (Kg/$m^{2}$)", "Education (years)", "Race", "Income ($\\$1000$ per year)", "Numbers Obs.")
perc1 <- round(perc(sbp,nhanes), digits = 2)
perc2 <- round(perc(age,nhanes), digits = 2)
perc3 <- round(perc(gender,nhanes), digits = 2)
perc4 <- round(perc(bmi,nhanes), digits = 2)
perc5 <- round(perc(educ,nhanes), digits = 2)
perc6 <- round(perc(race,nhanes), digits = 2)
perc7 <- round(perc(Income,nhanes), digits = 2)
nObs <- length(Income)
Percentage <- c(perc1, perc2, perc3, perc4, perc5, perc6, perc7, nObs)
df <- subset(nhanes, !is.na(sbp))
df <- subset(df, !is.na(bmi))
attach(df)
perc11 <- round(perc(sbp,df), digits = 2)
perc12 <- round(perc(age,df), digits = 2)
perc13 <- round(perc(gender,df), digits = 2)
perc14 <- round(perc(bmi,df), digits = 2)
perc15 <- round(perc(educ,df), digits = 2)
perc16 <- round(perc(race,df), digits = 2)
perc17 <- round(perc(Income,df), digits = 2)
nObs1 <- length(Income)
Percentage1 <- c(perc11, perc12, perc13, perc14, perc15, perc16, perc17, nObs1)
table <- data.frame("Variables" = Variables, "Percentage of Missing"= Percentage, "Without Missing"= Percentage1)
kable(table, format = "html", align = c("c", "c", "c"))
```

```{r, warning=FALSE, fig.alt="Log Systolic blood pressure"}
library("gridExtra")
barfill <- "grey"
barlines <- "black"
p1 <- ggplot(df, aes(Income)) + geom_histogram( breaks = seq(0, 10, 0.5), aes(y = ..density..), colour = barlines, fill = barfill)+
  scale_x_continuous(name = "Income",
                   breaks = seq(0, 10, 2),
                   limits=c(0, 10))

p2 <- ggplot(df, aes(log(sbp))) + geom_histogram( colour = barlines, fill = barfill) +
  scale_x_continuous(name = "Log Systolic blood pressure")
grid.arrange(p1, p2, ncol=2)
```

```{r, warning = FALSE}
df$YS <- ifelse(is.na(df$Income),0,1)
df$educ <- ifelse(df$educ<=2,0,1)
df$Income <- ifelse(is.na(df$Income),0,df$Income)
attach(df)
selectionEq <- YS~age+gender+educ+race
outcomeEq   <- log(sbp)~age+gender+educ+bmi+Income
outcomeBS   <- sbp~age+gender+educ+bmi+Income
mCL <- HeckmanCL(selectionEq, outcomeEq, data = df)
mBS <- HeckmanBS(selectionEq, outcomeBS, data = df)
mSK <- HeckmanSK(selectionEq, outcomeEq, data = df, lambda = 0)
mtS <- HeckmantS(selectionEq, outcomeEq, data = df, df = 15)

    Parameters <- c("Intercept", "age", "gender", "educ", "race", "Intercept", "age", "gender", "educ", "bmi", "income", "sigma", "rho", "nu", "lambda")
HBS <- round(mBS$coefficients, digits = 5)
HCL <- round(mCL$coefficients, digits = 5)
HSK <- round(mSK$coefficients, digits = 5)
HtS <- round(mtS$coefficients, digits = 5)
Results <- data.frame("Parameters"= Parameters, 
                      "HeckmanCL" = c(HCL[1:13], "NA", "NA"),
                      "HeckmanBS" = c(HBS[1:13], "NA", "NA"), 
                      "HeckmantS" = c(HtS[1:13], HtS[14], "NA"),
                      "HeckmanSK" = c(HSK[1:13], "NA", HSK[14]))
kable(Results, format = "html", align = c("c", "c", "c", "c", "c"))
```

```{r, warning = FALSE}
summary(mCL)
```

```{r, warning = FALSE}
summary(mtS)
```

```{r, warning = FALSE}
summary(mBS)
```

```{r, warning = FALSE}
summary(mSK)
```



# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HeckmanBS.R
\name{HeckmanBS}
\alias{HeckmanBS}
\title{Heckman BS Model fit Function}
\usage{
HeckmanBS(selection, outcome, data = sys.frame(sys.parent()), start = NULL)
}
\arguments{
\item{selection}{Selection equation.}

\item{outcome}{Primary Regression Equation.}

\item{data}{Database.}

\item{start}{initial values.}
}
\value{
Returns a list with the following components.

Coefficients: Returns a numerical vector with the best estimated values
of the model parameters;

Value: The value of function to be minimized (or maximized) corresponding
to par.

loglik: Maximized value of the log-likelihood function calculated from the estimated coefficients.

counts: Component of the Optim function. A two-element integer vector
giving the number of calls to fn and gr respectively. This excludes
those calls needed to compute the Hessian, if requested, and any calls
to fn to compute a finite-difference approximation to the gradient.

hessian: Component of the Optim function, with pre-defined option
hessian=TRUE. A symmetric matrix giving an estimate of the Hessian
at the solution found. Note that this is the Hessian of the unconstrained
problem even if the box constraints are active.

fisher_infoBS: Fisher information matrix

prop_sigmaBS: Square root of the Fisher information matrix diagonal

coefficients_link: Estimates on the optimization scale. The last component
is \code{rho_star}, where \code{rho = 2 / (1 + exp(-rho_star)) - 1}.

gradient_link: Analytical score vector evaluated at \code{coefficients_link}.

level: Selection variable levels

nObs: Numeric value representing the size of the database

nParam: Numerical value representing the number of model parameters

N0: Numerical value representing the number of unobserved entries

N1: Numerical value representing the number of complete entries

NXS: Numerical value representing the number of parameters of the
selection model

NXO: Numerical value representing the number of parameters of the
regression model

df: Numerical value that represents the difference between the size
of the response vector of the selection equation and the number of
model parameters

aic: Numerical value representing Akaike's information criterion.

bic: Numerical value representing Schwarz's Bayesian Criterion

initial.value: Numerical vector that represents the input values
(Initial Values) used in the parameter estimation.
}
\description{
Estimates the parameters of the Heckman-BS model
}
\details{
The HeckmanBS() function fits the Sample Selection Model
based on the Birnbaum Saunders bivariate distribution,
it has the same number of parameters as the classical
Heckman model. For more information see Bastos and Barreto-Souza
(2020) \doi{10.1080/02664763.2020.1780570}.
}
\examples{
data(MEPS2001)
attach(MEPS2001)
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeBS <- ambexp ~ age + female + educ + blhisp + totchr + ins
HeckmanBS(selectEq, outcomeBS, data = MEPS2001)
}
\references{
Bastos, F. S. and Barreto-Souza, W. (2020).
Birnbaum-Saunders sample selection model. \emph{Journal of Applied
Statistics}. \doi{10.1080/02664763.2020.1780570}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{PSID2}
\alias{PSID2}
\title{Panel Study of Income Dynamics}
\format{
An object of class \code{data.frame} with 6948 rows and 13 columns.
}
\source{
\url{https://www.icpsr.umich.edu/web/ICPSR/series/131}
}
\usage{
PSID2
}
\description{
The data come from the Panel Study of Income
Dynamics, years 1981 to 1992 (also contains earnings
data from 1980). The sample consists of 579 white
females, who were followed over the considered period.
In total, there are 6,948 observations over the 12-year
period (1981-1992). This data frame contains the following
columns:
\itemize{
  \item{id: Individual identifier}
  \item{year: Survey year}
  \item{age: Calculated age in years (based on year and month of birth)}
  \item{educ: Years of schooling}
  \item{children: Total number of children in family unit, ages 0-17}
  \item{s: Participation dummy, =1 if worked (hours>0)}
  \item{lnw: Log of real average hourly earnings}
  \item{lnw80: Log earnings in 1980}
  \item{agesq: Age squared}
  \item{children_lag1: Number of children in t-1}
  \item{children_lag2: Number of children in t-2}
  \item{lnw2: Log of real average hourly earnings}
  \item{Lnw: Log of real average hourly earnings}
}
}
\examples{
data(PSID2)
attach(PSID2)
hist(Lnw)
selectEq <- s ~ educ+ age+ children+ year
outcomeEq <- Lnw ~ educ+ age+ children
HCinitial(selectEq,outcomeEq, data = PSID2)
#Note that the estimated value of rho by the two-step
#method is greater than 1
summary(HeckmanGe(selectEq,outcomeEq, 1, 1, data = PSID2))

}
\references{
Semykina, A. and Wooldridge, J. M. (2013). Estimation of dynamic panel data
models with sample selection. \emph{Journal of Applied Econometrics}, 28(1),
47-61.

Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
Robust Estimation and Inference in Sample Selection Models}. R package
version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}

Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
sampleSelection. \emph{Journal of Statistical Software}, 27(7).
\url{https://www.jstatsoft.org/article/view/v027i07}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HeckmantS.R
\name{HeckmantS}
\alias{HeckmantS}
\title{Heckman-t Model fit Function}
\usage{
HeckmantS(selection, outcome, data = sys.frame(sys.parent()), df, start = NULL)
}
\arguments{
\item{selection}{Selection equation.}

\item{outcome}{Primary regression equation.}

\item{data}{Database.}

\item{df}{Initial value for the degrees of freedom. Values greater than 2 are
recommended because the model uses a finite-variance Student-t error.}

\item{start}{Optional starting values on the natural scale
(selection, outcome, sigma, rho, df).}
}
\value{
A list with estimated coefficients, log-likelihood and diagnostics.
}
\description{
Estimates the parameters of the Heckman-t model. The scale, correlation and
degrees of freedom are optimized on unconstrained scales to improve numerical
stability: sigma = exp(eta_sigma), rho = tanh(eta_rho), and df = 2 + exp(eta_df).
}
\examples{
data(MEPS2001)
dados <- MEPS2001[seq_len(500), ]
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins
HeckmantS(selectEq, outcomeEq, data = dados, df = 12)

}
\references{
Marchenko, Y. V. and Genton, M. G. (2012). A Heckman selection-t
model. \emph{Journal of the American Statistical Association}, 107(497),
304-317.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{package}
\name{ssmodels}
\alias{ssmodels-package}
\alias{ssmodels}
\title{A package that provides functions to fit data affected by sample selection bias.}
\arguments{
\item{selection}{Selection equation.}

\item{outcome}{Primary regression equation for the observed response.}

\item{outcomeS}{Matrix of covariates for modeling the dispersion parameter (sigma).}

\item{outcomeC}{Matrix of covariates for modeling the correlation parameter (rho).}

\item{df}{Initial value to the degree of freedom of Heckman-t model.}

\item{lambda}{Initial value for asymmetry parameter.}

\item{start}{initial values.}

\item{data}{Database.}
}
\value{
A list containing the estimated parameters, Hessian matrix, number of observations,
and additional diagnostic information. If initial values are not provided, they are
automatically estimated using the Heckman two-step method.
}
\description{
Package that provides models to fit data with sample selection bias problems. Includes:
\describe{
\item{HeckmanCL(selectEq, outcomeEq, data = data, start)}{Heckman's classic model fit function. Sample selection
usually arises in practice as a result of partial observability of the
result of interest in a study. In the presence of sample selection, the
observed data do not represent a random sample of the population, even
after controlling for explanatory variables. #' That is, the data is not missing completely at random.
Thus, standard analysis using only complete cases will lead to
biased results. Heckman introduced a sample selection model to analyze
this data and proposed a complete likelihood estimation method under the
assumption of normality. Such model was called Heckman model or Tobit 2
model.}
\item{HeckmantS(selectEq, outcomeEq, data = data, df, start)}{Heckman-t model adjustment function. The Heckman-t model
maintains the original parametric structure of the Classic Heckman model,
but considers a bivariate Student's t distribution as the underlying joint
distribution of the selection and primary regression variable and estimates
the parameters by maximum likelihood.}
\item{HeckmanSK(selectEq, outcomeEq, data = data, lambda, start)}{Heckman-SK model adjustment function. The Heckman-sk
model maintains the original parametric structure of the Classic Heckman
model, but considers a bivariate Skew-Normal distribution as the underlying
joint distribution of the selection and primary regression variable and
estimates the parameters by maximum likelihood.}
\item{HeckmanBS(selectEq, outcomeBS, data = data, start)}{Heckman-BS model adjustment function. The Heckman-BS model
maintains the original parametric structure of the Classic Heckman model,
but considers a bivariate Birnbaum-Saunders distribution as the underlying
joint distribution of the selection and primary regression variable and
estimates the parameters by maximum likelihood.}
\item{HeckmanGe(selectEq, outcomeEq,outcomeS, outcomeC, data = data)}{
Function for fitting the Generalized Heckman model.
This model generalizes the Classic Heckman model by including covariates
in the dispersion and correlation structures. It allows identification
of variables responsible for selection bias and heteroscedasticity.}
}
}
\seealso{
\code{\link{HeckmanCL}}

\code{\link{HeckmantS}}

\code{\link{HeckmanSK}}

\code{\link{HeckmanBS}}

\code{\link{HeckmanGe}}
}
\author{
Fernando de Souza Bastos, Wagner Barreto de Souza
}
\keyword{Heckman}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HeckmanGe.R
\name{HeckmanGe}
\alias{HeckmanGe}
\title{Generalized Heckman Model Estimation}
\usage{
HeckmanGe(
  selection,
  outcome,
  outcomeS = 1,
  outcomeC = 1,
  data = sys.frame(sys.parent()),
  start = NULL
)
}
\arguments{
\item{selection}{Formula for the selection equation.}

\item{outcome}{Formula for the outcome equation.}

\item{outcomeS}{Covariates for the log-scale model. It may be a matrix,
vector, one-sided formula, or 1 for an intercept-only scale model.}

\item{outcomeC}{Covariates for the correlation model. It may be a matrix,
vector, one-sided formula, or 1 for an intercept-only correlation model.}

\item{data}{Data frame containing all variables.}

\item{start}{Optional starting values. If supplied on the natural classical
scale, sigma and rho are internally mapped to log(sigma) and atanh(rho).}
}
\value{
A list with the fitted model parameters and diagnostics.
}
\description{
Estimates the parameters of a generalized Heckman selection model,
allowing covariates in the scale and correlation components.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postprocess_theta.R
\name{postprocess_theta}
\alias{postprocess_theta}
\title{Post-process Parameter Vector for Generalized Heckman Models}
\usage{
postprocess_theta(theta_par, NXS, NXO, NE, NV, XS, XO, outcomeS, outcomeC)
}
\arguments{
\item{theta_par}{Numeric vector of estimated parameters on the optimization scale.}

\item{NXS}{Integer. Number of covariates in the selection equation.}

\item{NXO}{Integer. Number of covariates in the outcome equation.}

\item{NE}{Integer. Number of scale parameters.}

\item{NV}{Integer. Number of correlation parameters.}

\item{XS}{Design matrix for the selection equation.}

\item{XO}{Design matrix for the outcome equation.}

\item{outcomeS}{Design matrix or variable for the scale model.}

\item{outcomeC}{Design matrix or variable for the correlation model.}
}
\value{
A named numeric vector.
}
\description{
This helper assigns names to a generalized Heckman parameter vector and,
for intercept-only scale/correlation models, maps log(sigma) and atanh(rho)
to sigma and rho.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/step2.R
\name{step2}
\alias{step2}
\title{Heckman's two-step method}
\usage{
step2(YS, XS, YO, XO)
}
\arguments{
\item{YS}{Selection vector coded as 0/1.}

\item{XS}{Selection matrix.}

\item{YO}{Outcome vector.}

\item{XO}{Covariate matrix for the outcome equation.}
}
\value{
A numeric vector with selection coefficients, outcome coefficients,
  sigma and rho.
}
\description{
Estimate classical Heckman model starting values by Heckman's two-step method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Mroz87}
\alias{Mroz87}
\title{U.S. Women's Labor Force Participation}
\format{
An object of class \code{data.frame} with 753 rows and 22 columns.
}
\source{
PSID Staff, The Panel Study of Income Dynamics,
Institute for Social ResearchPanel Study of Income
Dynamics, University of Michigan, \url{https://www.icpsr.umich.edu/web/ICPSR/series/131}
}
\usage{
Mroz87
}
\description{
The Mroz87 data frame contains data about 753
married women. These data are collected within the
"Panel Study of Income Dynamics" (PSID). Of the 753
observations, the first 428 are for women with positive
hours worked in 1975, while the remaining 325
observations are for women who did not work for
pay in 1975. A more complete discussion of the data
is given by Mroz (1987). It also
includes the following explanatory variables:
\itemize{
  \item{lfp: Dummy variable for labor-force participation.}
  \item{hours: Wife's hours of work in 1975. }
  \item{kids5: Number of children 5 years old or younger.}
  \item{kids618: Number of children 6 to 18 years old.}
  \item{Age: Wife's age.}
  \item{Educ: Wife's educational attainment, in years.}
  \item{wage: Wife's average hourly earnings, in 1975 dollars.}
  \item{repwage: Wife's wage reported at the time of the 1976 interview.}
  \item{hushrs: Husband's hours worked in 1975.}
  \item{husage: Husband's age.}
  \item{huseduc: Husband's educational attainment, in years.}
  \item{huswage: Husband's wage, in 1975 dollars.}
  \item{faminc: Family income, in 1975 dollars.}
  \item{mtr: Marginal tax rate facing the wife.}
  \item{motheduc: Wife's mother's educational attainment, in years.}
  \item{fatheduc: Wife's father's educational attainment, in years.}
  \item{unem: Unemployment rate in county of residence, in percentage points.}
  \item{city: Dummy variable = 1 if live in large city, else 0. }
  \item{exper: Actual years of wife's previous labor market experience.}
  \item{nwifeinc: Non-wife income.}
  \item{wifecoll: Dummy variable for wife's college attendance.}
  \item{huscoll: Dummy variable for husband's college attendance.}
}
}
\examples{
# Wooldridge(2016): page 247
data(Mroz87)
attach(Mroz87)
selectEq  <- lfp ~ nwifeinc + educ + exper + I(exper^2) + age + kids5 + kids618
outcomeEq <- log(wage) ~ educ + exper + I(exper^2)
outcomeS  <- cbind(educ, exper)
outcomeC  <- cbind(educ, exper)
outcomeBS <- wage ~ educ + exper + I(exper^2)
outcomeBS <- wage ~ educ + exper + I(exper^2)
HeckmanCL(selectEq, outcomeEq, data = Mroz87)
HeckmanBS(selectEq, outcomeBS, data = Mroz87)
HeckmanSK(selectEq, outcomeEq, data = Mroz87, lambda = 1)
HeckmantS(selectEq, outcomeEq, data = Mroz87, df=5)
HeckmanGe(selectEq, outcomeEq, outcomeS, outcomeC, data = Mroz87)

}
\references{
Mroz, T. A. (1987). The sensitivity of an empirical model of married
women's hours of work to economic and statistical assumptions.
\emph{Econometrica}, 55, 765-799.

Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
Robust Estimation and Inference in Sample Selection Models}. R package
version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}

Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
sampleSelection. \emph{Journal of Statistical Software}, 27(7).
\url{https://www.jstatsoft.org/article/view/v027i07}

Wooldridge, J. M. (2016). \emph{Introductory Econometrics: A Modern
Approach}. Nelson Education.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{RandHIE}
\alias{RandHIE}
\title{RAND Health Insurance Experiment}
\format{
An object of class \code{data.frame} with 20190 rows and 45 columns.
}
\source{
\url{https://cameron.econ.ucdavis.edu/mmabook/mmadata.html}
}
\usage{
RandHIE
}
\description{
'The RAND Health Insurance Experiment (RAND HIE) was a comprehensive
study of health care cost, utilization and outcome in the United States.
It is the only randomized study of health insurance, and the only study
which can give definitive evidence as to the causal effects of different
health insurance plans. For more information about the database visit:
\url{https://en.wikipedia.org/w/index.php?title=RAND_Health_Insurance_Experiment&oldid=110166949}
accessed september 09, 2019). This data frame contains the following columns:
\itemize{
\item{plan: HIE plan number.}
 \item{site: Participant's  place of residence
 when the participant was initially enrolled.}
 \item{coins: Coinsurance rate.}
 \item{tookphys: Took baseline physical.}
 \item{year: Study year.}
 \item{zper: Person identifier.}
 \item{black: 1 if race of household head is black.}
 \item{income: Family income.}
 \item{xage: Age in years.}
 \item{female: 1 if person is female.}
 \item{educdec: Education of household head in years.}
 \item{time: Time eligible during the year.}
 \item{outpdol: Outpatient expenses:
 all covered outpatient medical services
 excluding dental care, outpatient psychotherapy,
 outpatient drugs or supplies.}
 \item{drugdol: Drug expenses:
 all covered outpatient and dental drugs.}
 \item{suppdol: Supply expenses:
 all covered outpatient supplies including dental.}
 \item{mentdol: Psychotherapy expenses:
 all covered outpatient psychotherapy services including injections
 excluding charges for visits in excess of 52 per year,
 prescription drugs, and inpatient care.}
 \item{inpdol: Inpatient expenses:
 all covered inpatient expenses in a hospital, mental hospital,
 or nursing home,
 excluding outpatient care and renal dialysis.}
 \item{meddol: Medical expenses:
 all covered inpatient and outpatient services,
 including drugs, supplies, and inpatient costs of newborns
 excluding dental care and outpatient psychotherapy.}
 \item{totadm: Hospital admissions:
 annual number of covered hospitalizations.}
 \item{inpmis: Incomplete Hospital Records:
 missing inpatient records.}
 \item{mentvis: Psychotherapy visits:
 indicates the annual number of outpatient visits for psychotherapy.
 It includes billed visits only.
 The limit was 52 covered visits per person per year.
 The count includes an initial visit to a psychiatrist or psychologist.}
 \item{mdvis: Face-to-Face visits to physicians:
 annual covered outpatient visits with physician providers
 (excludes dental, psychotherapy, and
 radiology/anesthesiology/pathology-only visits).}
 \item{notmdvis: Face-to-Face visits to nonphysicians:
 annual covered outpatient visits with nonphysician providers
 such as speech and physical therapists, chiropractors,
 podiatrists, acupuncturists, Christian Science etc.
 (excludes dental, healers, psychotherapy,
 and radiology/anesthesiology/pathology-only visits).}
 \item{num: Family size.}
 \item{mhi: Mental health index.}
 \item{disea: Number of chronic diseases.}
 \item{physlm: Physical limitations.}
 \item{ghindx: General health index.}
 \item{mdeoff: Maximum expenditure offer.}
 \item{pioff: Participation incentive payment.}
 \item{child: 1 if age is less than 18 years.}
 \item{fchild: \code{female * child}.}
 \item{lfam: log of \code{num} (family size).}
 \item{lpi: log of \code{pioff} (participation incentive payment).}
 \item{idp: 1 if individual deductible plan.}
 \item{logc: \code{log(coins+1)}.}
 \item{fmde: 0 if \code{idp=1},
 \code{ln(max(1,mdeoff/(0.01*coins)))} otherwise.}
 \item{hlthg: 1 if self-rated health is good
 -- baseline is excellent self-rated health.}
 \item{hlthf: 1 if self-rated health is fair
 -- baseline is excellent self-rated health.}
 \item{hlthp: 1 if self-rated health is poor
 -- baseline is excellent self-rated health.}
 \item{xghindx: \code{ghindx} (general healt index)
 with imputations of missing values.}
 \item{linc: log of \code{income} (family income).}
 \item{lnum: log of \code{num} (family size).}
 \item{lnmeddol: log of \code{meddol} (medical expenses).}
 \item{binexp: 1 if \code{meddol} > 0.}
 }
}
\examples{
##Cameron and Trivedi (2005): Section 16.6
data(RandHIE)
subsample <- RandHIE$year == 2 & !is.na( RandHIE$educdec )
selectEq <- binexp ~ logc + idp + lpi + fmde + physlm + disea +
  hlthg + hlthf + hlthp + linc + lfam + educdec + xage + female +
  child + fchild + black
  outcomeEq <- lnmeddol ~ logc + idp + lpi + fmde + physlm + disea +
  hlthg + hlthf + hlthp + linc + lfam + educdec + xage + female +
  child + fchild + black
  cameron <- HeckmanCL(selectEq, outcomeEq, data = RandHIE[subsample, ])
  summary(cameron)

}
\references{
Cameron, A. C. and Trivedi, P. K. (2005). \emph{Microeconometrics: Methods
and Applications}. Cambridge University Press.

Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
Robust Estimation and Inference in Sample Selection Models}. R package
version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}

Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
sampleSelection. \emph{Journal of Statistical Software}, 27(7).
\url{https://www.jstatsoft.org/article/view/v027i07}

Wikipedia contributors (2019). RAND Health Insurance Experiment.
\url{https://en.wikipedia.org/wiki/RAND_Health_Insurance_Experiment}
}
\keyword{RandHIE}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HCinitial.R
\name{HCinitial}
\alias{HCinitial}
\title{Two-Step Method for Parameter Estimation of the Heckman Model}
\usage{
HCinitial(selection, outcome, data = sys.frame(sys.parent()))
}
\arguments{
\item{selection}{A formula for the selection equation.}

\item{outcome}{A formula for the outcome equation.}

\item{data}{A data frame containing the variables.}
}
\value{
A numeric vector containing selection coefficients, outcome coefficients,
  sigma and rho.
}
\description{
Estimates classical Heckman starting values using Heckman's two-step method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MEPS2001}
\alias{MEPS2001}
\title{Medical Expenditure Panel Survey}
\format{
An object of class \code{data.frame} with 3328 rows and 22 columns.
}
\source{
2001 Medical Expenditure Panel Survey by the Agency
for Healthcare Research and Quality.
}
\usage{
MEPS2001
}
\description{
The MEPS is a set of large-scale surveys of families, individuals
and their medical providers (doctors, hospitals, pharmacies, etc.)
in the United States. It has data on the health services Americans use,
how often they use them, the cost of these services and how they are paid,
as well as data on the cost and reach of health insurance available
to American workers. The sample is restricted to persons aged between 21
and 64 years and contains a variable response with 3328 observations of
outpatient costs, of which 526 (15.8\%) correspond to unobserved expenditure
values and identified as zero expenditure for adjustment of the models.
It also includes the following explanatory variables:
\itemize{
  \item{educ: education status}
  \item{age: Age}
  \item{income: income}
  \item{female: gender}
  \item{vgood: a numeric vector}
  \item{good: a numeric vector}
  \item{hospexp: a numeric vector}
  \item{totchr: number of chronic diseases}
  \item{ffs: a numeric vector}
  \item{dhospexp: a numeric vector}
  \item{age2: a numeric vector}
  \item{agefem: a numeric vector}
  \item{fairpoor: a numeric vector}
  \item{year01: a numeric vector}
  \item{instype: a numeric vector}
  \item{ambexp: a numeric vector}
  \item{lambexp: log ambulatory expenditures}
  \item{blhisp: ethnicity}
  \item{instype_s1: a numeric vector}
  \item{dambexp: dummy variable, ambulatory expenditures}
  \item{lnambx: a numeric vector}
  \item{ins: insurance status}
}
}
\examples{
data(MEPS2001)
attach(MEPS2001)
hist(lnambx)
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
outcomeEq <- lnambx ~ age + female + educ + blhisp + totchr + ins
HeckmanCL(selectEq, outcomeEq, data = MEPS2001)
}
\references{
Cameron, A. C. and Trivedi, P. K. (2009). \emph{Microeconometrics using
Stata}. Stata Press.

Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
Robust Estimation and Inference in Sample Selection Models}. R package
version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}

Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
sampleSelection. \emph{Journal of Statistical Software}, 27(7).
\url{https://www.jstatsoft.org/article/view/v027i07}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IMR.R
\name{IMR}
\alias{IMR}
\title{Inverse Mills Ratio}
\usage{
IMR(selection, data = sys.frame(sys.parent()))
}
\arguments{
\item{selection}{A formula for the selection equation.}

\item{data}{A data frame containing the variables.}
}
\value{
A column vector (matrix with one column) containing the Inverse Mills Ratio
computed from the Probit model fitted to the selection equation.
}
\description{
Computes the column vector of the Inverse Mills Ratio (IMR)
from a Probit selection equation.
}
\examples{
data(MEPS2001)
attach(MEPS2001)
selectEq <- dambexp ~ age + female + educ + blhisp + totchr + ins + income
IMR(selectEq, data = MEPS2001)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.HeckmanGe.R
\name{summary.HeckmanGe}
\alias{summary.HeckmanGe}
\title{Summary of Generalized Heckman Model}
\usage{
\method{summary}{HeckmanGe}(object, ...)
}
\arguments{
\item{object}{An object of class \code{HeckmanGe}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
Prints the summary output including coefficient tables and model fit statistics.
}
\description{
Print estimates of the parameters of the Generalized Heckman model using Maximum Likelihood Estimation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/twostep.R
\name{twostep}
\alias{twostep}
\title{Heckman's two-step method}
\usage{
twostep(selection, outcome, data = sys.frame(sys.parent()))
}
\arguments{
\item{selection}{A formula for the selection equation.}

\item{outcome}{A formula for the outcome equation.}

\item{data}{A data frame containing the variables used in the model.}
}
\value{
A numeric vector with the two-step estimates.
}
\description{
Estimate the classical Heckman model parameters by the two-step method.
}
\references{
Heckman, J. J. (1976). The common structure of statistical
models of truncation, sample selection and limited dependent variables and
a simple estimator for such models. \emph{Annals of Economic and Social
Measurement}, 5(4), 475-492.

Heckman, J. J. (1979). Sample selection bias as a specification error.
\emph{Econometrica}, 47, 153-161.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nhanes}
\alias{nhanes}
\title{US National Health and Nutrition Examination Study}
\format{
An object of class \code{data.frame} with 9643 rows and 9 columns.
}
\source{
\url{https://wwwn.cdc.gov/nchs/nhanes/ContinuousNhanes/Default.aspx?BeginYear=2003}
}
\usage{
nhanes
}
\description{
The US National Health and Nutrition Examination Study (NHANES) is a
survey data collected by the US National Center for Health Statistics.
The survey data dates back to 1999, where individuals of all ages are
interviewed in their home annually and complete the health examination
component of the survey. The study variables include demographic
variables (e.g. age and annual household income), physical measurements
(e.g. BMI – body mass index), health variables (e.g. diabetes status),
and lifestyle variables (e.g. smoking status). This data frame contains
the following columns:
\itemize{
  \item{id: Individual identifier}
  \item{age: Age}
  \item{gender: Sex 1=male, 0=female}
  \item{educ: Education is dichotomized into high school
  and above versus less than high school}
  \item{race: categorical variable with five levels}
  \item{income: Household income ($1000 per year) was reported
  as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
  and had 10 interval categories. }
  \item{Income: Household income ($1000 per year) was reported
  as a range of values in dollar (e.g. 0–4999, 5000–9999, etc.)
  and had 10 interval categories. }
  \item{bmi: body mass index}
  \item{sbp: systolic blood pressure}
}
}
\examples{
data("nhanes")
attach(nhanes)
hist(Income, prob= TRUE, breaks = seq(1, 99, 0.5), xlim = c(1,10),
ylim = c(0,0.35), main = "Histogram of Income", xlab = "Category")
data2 <- subset(nhanes, !is.na(sbp))
data3 <- subset(data2, !is.na(bmi))
attach(data3)
data <- data3
data$YS <- ifelse(is.na(data$Income),0,1)
data$educ <- ifelse(data$educ<=2,0,1)
attach(data)
selectionEq <- YS~age+gender+educ+race
outcomeEq   <- sbp~age+gender+educ+bmi

}
\references{
Ogundimu, E. O. and Collins, G. S. (2019). A robust imputation method for
missing responses and covariates in sample selection models. \emph{Statistical
Methods in Medical Research}, 28(1), 102-116.

Little, R. J. and Zhang, N. (2011). Subsample ignorable likelihood for
regression analysis with missing data. \emph{Journal of the Royal Statistical
Society: Series C (Applied Statistics)}, 60(4), 591-605.

Zhelonkin, M., Genton, M. G. and Ronchetti, E. (2019). \emph{ssmrob:
Robust Estimation and Inference in Sample Selection Models}. R package
version 0.7. \url{https://CRAN.R-project.org/package=ssmrob}

Toomet, O. and Henningsen, A. (2008). Sample selection models in R: package
sampleSelection. \emph{Journal of Statistical Software}, 27(7).
\url{https://www.jstatsoft.org/article/view/v027i07}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract_model_components.R
\name{extract_model_components}
\alias{extract_model_components}
\title{Extract model components from formulas}
\usage{
extract_model_components(
  selection,
  outcome,
  data,
  outcomeS = NULL,
  outcomeC = NULL,
  drop.levels = TRUE
)
}
\arguments{
\item{selection}{A formula representing the selection equation.}

\item{outcome}{A formula representing the outcome equation.}

\item{data}{A data frame containing all variables used in the formulas.}

\item{outcomeS}{Optional matrix, vector, one-sided formula, or 1 for the
log-scale model in the generalized Heckman model.}

\item{outcomeC}{Optional matrix, vector, one-sided formula, or 1 for the
correlation model in the generalized Heckman model.}

\item{drop.levels}{Logical; if TRUE, unused factor levels are dropped.}
}
\value{
A list with selection, outcome, dispersion and correlation matrices.
}
\description{
This function extracts model matrices and response vectors used by the
sample-selection model fitting functions. It standardizes the selection
response to a numeric indicator, where 1 denotes selected/observed and
0 denotes not selected/not observed.
}
